/*****************************************************************************************
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *  NAME: orva_freq_fit_kri.sas                                             
 *  PURPOSE: Estimates frequency distribution
 *  SUPPORT: lajack
 *
 *  NOTE: Please see the accompanying doc for pdfs and cdfs of the distributions
 *
 *  PARAMETERS:
 *
 *  compressedData  =  dataset containing the (compressed) frequency counts ("newfrequency"), plus the firm & 
 *           business-line level KRIs (called KRI_:) and the panel dummy variables for business 
 *           line (called DUMMY_BLUID_:).  
 *           Also contains:  OFFSET (log (exposure/FSCALE) )
 *
 *  notSelected = column of variables (dummies & KRIs) that weren't selected as 
 *                significant in orva_freq_var_select.
 *
 *  parmStatus  =
 *
 *  monthlyKris =
 *
 *  ccUid       =  key for cell
 *
 *  thresholdValue = the overridding threshold of loss data, defaults to <blank>
 *
 *  defaultThreshold = specifies the default threshold for data where THRESHOLD=.
 *
 *  interval = time interval size for which the analysis needs to be done, defaults to YEAR.    
 *             possibles values: YEAR, SEMIYEAR, QTR, MONTH2, MONTH, SEMIMONTH, WEEK2, WEEK
 *
 *  fdists = distributions to fit, defaults to "POIS NEGBIN".
 *
 *  sparmdata=  the name of the severity parameter data set with the following variables:
 *              DISTRIBUTION, PARAM_1, PARAM_2, ...   
 *              This is needed to calculate p_obs (the variable name is pScaledExcess).
 *
 *  outparm  =  the name of the output statistics data set, defaults to fparm 
 *
 *  outbluidest  = Defaults to fbluidest.
 * 
 *  outcov       = Defaults to fcov.
 *
 *  numSelectedKRIs = number of KRIs (both firm-level and business-line-level) that were 
 *                    selected in orva_freq_var_select.  
 *
 *  parallel        = 1: use parallel processing
 *                    0: don't use parallel processing
 *
 *  pobsFlag       = 1: User specified a number under Probability of Observability Severity fitting
 *                 = 0: Not
 * 
 *****************************************************************************************/
%macro ORVA_freq_fit_kri(
                    compressedData   = compressed_data,
					notSelected      = not_selected,
					parmStatus       = parm_status,
					monthlyKRIs      = monthly_kris,
                    sparmData        =  ,  
                    ccUid            =  ,
                    thresholdValue   =  ,
                    defaultThreshold = 0,   
                    interval         = QTR,
                    fdists           = "POIS NEGBIN",    
                    outParm          = fparm,
                    outBluidEst      = fbluidest,
                    outCov           = fcov,
					parallel         = 0,
					pobsFlag         = 0
					);                     

/*%orvh_displaypaths;*/
    %let status=0;
    %let reason=Successful;
    %let abort=0;
    %let fdist=;
    %let sdist=;
    %let ndist=0;
    
    /*- create the status data set upfront with unknown status-*/
    data &outparm;
        format status 8. reason $100.;
        status=1;
        reason="Unknown";
    run;
    
    /*- remove anything in other output datasets up front -*/
    data &outbluidest; 
        set _NULL_; 
    run;
    data &outcov; 
        set _NULL_; 
    run;
 
    /*- parameter checking -*/
    %if &sparmdata=%str() %then %do;
        %put ERROR: The parameter SPARMDATA= must be specified.;
        %let abort=1;
        %let reason=Missing SPARMDATA= parameter;
        %goto DONE;
    %end;

    %if &compressedData=%str() %then %do;
        %put ERROR: The parameter compressedData= must be specified.;
        %let abort=1;
        %let reason=Missing compressedData= parameter;
        %goto DONE;
    %end;

    %if &parmStatus=%str() %then %do;
        %put ERROR: The parameter parmStatus= must be specified.;
        %let abort=1;
        %let reason=Missing parmStatus= parameter;
        %goto DONE;
    %end;


	/* Set a flag to indicate that a genmod call hasn't been executed yet. */
	/* (In orva_ffitkri_dist, we call proc genmod with Poisson, to get initial param estimates */
	/*  for Proc Countreg with EITHER Poisson OR NegBin.) */
	%let genmod_results_exist = 0;



	/*- get the parameters -*/
	data _null_;
		set &parmStatus;
		call symput("numSelectedKRIs", numSelectedKRIs);
		call symput("numSelectedBLs", numSelectedBLs);
		call symput("SelectedKRIs", SelectedKRIs);
		call symput("numBLUIDsThisCell", numBLUIDsThisCell);
		call symput("extraBL", extraBL); /* the BLUID of the BL that did not get a dummy var named for him */
	run;

	%if %eval(&numSelectedBLs) GT 0 %then %let possiblyDummyVars = dummy_bluid_: ; 
	%else %let possiblyDummyVars = ; 

	
	data _parm_;
        set &sparmdata;
        _NAME_="col1";
        call symput("sdist",DISTRIBUTION);
        drop CCUID;
    run;
    %if &SYSERR^=0 & &SYSERR ^= 4 %then %do;
        %put ERROR: Unable to create temporary parm data.;
        %let abort=1;
        %let reason=Unable to create temporary parm data;
        %goto DONE;    
    %end;

	proc transpose data=_parm_ out=_parm_;
    run;
    %if &SYSERR^=0 & &SYSERR ^= 4 %then %do;
        %put ERROR: Unable to transpose temporary parm data.;
        %let abort=1;
        %let reason=Unable to transpose temporary parm data;
        %goto DONE;    
    %end;

    data _null_;
        set _parm_;
        call symput(_name_,col1);
    run;
    %if &SYSERR^=0 & &SYSERR ^= 4 %then %do;
        %put ERROR: Unable to create parm macros.;
        %let abort=1;
        %let reason=Unable to create parm macros;
        %goto DONE;    
    %end;
	
    %put *******************************************;
    %put Severity;
    %put Distribution :      &sdist;
    %put *******************************************;


    %let sdist=%upcase(&sdist);
    %orvd_&sdist /* force load of macros */

	/*- get the probability of exceeding the threshold -*/
    /* First, check whether KRI's were used and kept */
    proc sql noprint;
        select COUNT_KRI into : KRI_ADJUST from &sparmdata;
    quit;

    %if &KRI_ADJUST NE 0 %then %do;
		%let param_1 = &orig_param_1;
        %orva_get_kri_log(data=&sparmdata);
    %end;
    %else %do;
        %let KRI_LOG=0;
    %end;
    %put "KRI_LOG= &KRI_LOG";

	%let avg_pScaledExcess = 0; /* initializing */
   	data _kriblLossespobs_ (drop=pScaledExcess_sum  pScaledExcess_count);
			set &compressedData end=last;

			if THRESHOLD=0 then do;
				pScaledExcess=1;
			end;
			else do;
				%Sdf&sdist(THRESHOLD/exp(&KRI_LOG), pScaledExcess); 
			end;

        if pScaledExcess LE 0 then offset = offset - 708; /* -708 is MACLOGSMALL, i.e. ln(smallest double) */
   	  else offset = offset + log(pScaledExcess); 

		pScaledExcess_sum + pScaledExcess;
		pScaledExcess_count + 1;
		if last then do;
			call symput("avg_pScaledExcess", pScaledExcess_sum / pScaledExcess_count);
		end;
    run;

    /******************** check Mean Probability of Observability ********************/
	%if  %sysevalf(&avg_pScaledExcess < 0.1, BOOLEAN) and (&pobsFlag eq 0) %then %do; 
        %put WARNING: Mean Probability of Observability is &avg_pScaledExcess, which is less than the minimum value 0.1.;
		%put WARNING: Frequency fitting is aborting, because the severity distribution is likely inappropriate.;
		%put WARNING: To force frequency fitting, enter &avg_pScaledExcess in the User Specified box on the Fitting Options screen.;
		%let reason= Mean Probability of Observability is less than 0.1.  The severity distribution is inappropriate;
		%let abort= 2;
        %goto DONE;		
	%end;	

	proc sort data=_kriblLossespobs_;
		by bluid date;
	run;


    /********************** loop over distributions *****************************/
	data _NULL_;
       now = datetime();
       call symput('_TimeStart_',now);
    run;
    
    %global ORVP_STATUS;
    %let ORVP_STATUS= 1;
    %if ("&parallel" eq "1") %then %do;
	    %orvp_distribute_ffitkri(dists=&fdists);
        %if (&ORVP_STATUS ne 0) %then %do;
            %put WARNING: No processors available to perform parallel fits. Reverting to sequential fitting.;
        %end;
	%end;
	%if (&ORVP_STATUS ^= 0) %then %do;
	    %let ndist=1;
	    %let dist=%scan(&fdists, &ndist, ' ');
	    %do %while (&dist ne );
	        
	        %orva_ffitkri_dist(fdist=&dist,ndist=&ndist);
	 
	        %let ndist=%eval(&ndist+1);
	        %let dist=%scan(&fdists, &ndist, ' ');
	    %end;
	%end;

    data _NULL_;
       now = datetime();
	   elapsed = now-symget('_TimeStart_');
       call symput('_TimeElapsed_',elapsed);
    run;
    %put ORVA_Freq_Fit_Kri Elapsed Time: &_TimeElapsed_;
    
    /************** End of the distribution loop ***************************************/

    /* Finding the mean/variance  */



    %let ndist=1;
    %let dist=%scan(&fdists, &ndist, ' ');
    %do %while (&dist ne );	 
        %let ndist=%eval(&ndist+1);
        %let dist=%scan(&fdists, &ndist, ' ');
    %end;
    %let ndist=%eval(&ndist-1);

/* Finding the mean/variance  */
    %let fitstatuslist = ; /* initializing */
	%let DeletedBLUIDs = ; /* initializing */
	%do i=1 %to &ndist;  
		/* Make sure orva_ffitkri_dist didn't end abnormally (i.e. that the fit didn't fail for this dist */
		data _null_;
			set countout_est_&i;
			call symput("fitstatus&i", status);
		run;
		%if &&fitstatus&i NE 0 %then %do;
			data countout_est_&i;
				set countout_est_&i;
				FPARAM_1 = .; 
				FPARAM_2 = .; 
				Mean = .;
				Variance = .;
			run;
			%goto END_NDIST_LOOP;
		%end;
		/*- Create the Frequency KRI Model  -*/
		data _null_;
			format TheModel $2000.;
			set countout_est_&i;
			TheModel="0";  /* Do not modify or delete this line!!! */
			TheModel = trim(TheModel) || " + " || Intercept;  /* Do not modify or delete this line!!! */
			%let firstKRI=%scan(&SelectedKRIs, 1, ' ');
	   		%if (&firstKRI ne ) %then %do;  
	       		array MyKRIs[&numSelectedKRIs] &SelectedKRIs;
				do ki=1 to &numSelectedKRIs;
					TheModel = trim(TheModel) || " + " || MyKRIs(ki) || " * " || vname(MyKRIs(ki));   
				end;
			%end;
			call symputX("TheFreqKriModel", TheModel);
		run;                  
		%if &SYSERR^=0 & &SYSERR ^= 4 %then %do;
			%put ERROR: Unable to construct the freq KRI model string.;
		    data countout_est_&i; 
			    set countout_est_&i;			
				status=1;
        		reason="Unable to construct the freq KRI model string.";
		    run;
			%goto END_NDIST_LOOP;
		%end;
			
/*		data _means_&i;*/
/*			set &monthlyKRIs;  Was previously "_kriblLosses_selected_" in freq_var_select macro */
/*			meanWithoutPanel = exp(&TheFreqKriModel);*/
/*		run;*/
/*		proc sort data=_means_&i;*/
/*			by bluid;*/
/*		run;*/
/*		proc summary data=_means_&i;*/
/*			var meanWithoutPanel;*/
/*			by bluid;*/
/*			weight adjexposure;*/
/*			output out=_means_summary_&i mean=avgMeanWithoutPanel;*/
/*		run;*/
/**/
/*		*/
/*		 these 2 datasets must be sorted before merging */
/*		proc sort data=_means_summary_&i;*/
/*			by bluid;*/
/*		run;*/

		proc sort data=&monthlyKRIs;
			by bluid;
		run;

%if &numSelectedBLs GT 0 %then %do;
		proc sort data=fbluidest_&i;
			by bluid;
		run;

		data _mergedKRIsAndEstimates_&i (drop=deletedBLUIDs);
			merge fbluidest_&i &monthlyKRIs;
			by bluid;
			length DeletedBLUIDs $ 1024;
			retain DeletedBLUIDs " ";
			/* correction for BLUIDs that had no losses in this column: */
			if missing(date) then do;
				DeletedBLUIDs = trim(DeletedBLUIDs) || " " || BLUID;  /* Do not modify or delete this line!!! */
				call symput("DeletedBLUIDs", DeletedBLUIDs);
				delete; /* i.e., delete any BLUIDs that were in fbluidest_&i who weren't in &monthlyKRIs */
			end;
/*			if missing(Estimate) then meanWith = exp(&TheFreqKriModel);*/
/*			else meanWith = exp(&TheFreqKriModel+Estimate);*/
		run;
		%put DeletedBLUIDs=&DeletedBLUIDs;

		/* Now delete (from fbluidest_&i) those BLUIDs that had no losses in this column... */
		/* Because this will affect the input to freq_kri_effect. */
	    %let numdeletedBLs=1;
	    %let currBLUID=%scan(&DeletedBLUIDs, &numdeletedBLs, ' ');
	    %do %while (&currBLUID ne );	 
			data fbluidest_&i;
				set fbluidest_&i;
				if bluid = &currBLUID then delete;
			run;
	        %let numdeletedBLs=%eval(&numdeletedBLs+1);
	        %let currBLUID=%scan(&DeletedBLUIDs, &numdeletedBLs, ' ');
	    %end;
	    %let numdeletedBLs=%eval(&numdeletedBLs-1);

		%put numdeletedBLs=&numdeletedBLs;

/*		data _jr_&i;*/
/*			merge fbluidest_&i _means_summary_&i;*/
/*			by bluid;*/
/*			correction for BLUIDs that had no losses in this column: */
/*			if missing(avgMeanWithoutPanel) then delete;*/
/*		run; */
%end;
%else %do;
		data _mergedKRIsAndEstimates_&i;
			set &monthlyKRIs;
/*			meanWith = exp(&TheFreqKriModel);*/
		run;
/*		data _jr_&i;*/
/*			set _means_summary_&i;  no correction necessary (for BLUIDs that had no losses in this column) */
/*		run; */
%end;

		data _mergedKRIsAndEstimates_&i;
			set _mergedKRIsAndEstimates_&i;
			if missing(Estimate) then meanWith = exp(&TheFreqKriModel);
			else meanWith = exp(&TheFreqKriModel+Estimate);
		run;

		proc summary data=_mergedKRIsAndEstimates_&i;
			var meanWith;
			by BLUID;
			weight adjexposure;
			output out=_MeanWith_summary_&i mean=avgMeanWith;
		run;

		data _TheNewResults_&i;
			set _MeanWith_summary_&i end=last;
			retain MTot 0;
			MTot=Mtot+avgMeanWith;
			if last then do;
				call symput("TheParam_1", MTot);
			end;	
		run;


/*		data _TheResults_&i;*/
/*			set _jr_&i end=last;*/
/*			retain MTot 0;*/
/*			if missing(Estimate) then Mean = avgMeanWithoutPanel;*/
/*			else Mean = avgMeanWithoutPanel * exp(Estimate);*/
/*			MTot=Mtot+Mean;*/
/*			if last then do;*/
/*				call symput("TheParam_1", MTot);*/
/*			end;	*/
/*		run;*/

		data countout_est_&i; 
			set countout_est_&i;
			Mean = &TheParam_1;
			if compare(Distribution, 'POIS') EQ 0 then do;
				Variance = &TheParam_1;
				FPARAM_1 = &TheParam_1;
			end;
			else do;
				/* Temporary fix for lack of convergence of Negative Binomial --BEGIN-- */
				if Mean<0.01 then do;
					status=1;
					reason="Convergence failed";
				end;
				/* Temporary fix for lack of convergence of Negative Binomial --END-- */

				Variance = Mean + Mean*_Alpha;
				/* FPARM1 is n (from SAS documentation for CDF function for NegBin) 
				   or r (from OpRisk VaR documentation for NegBin).  The number of successes. */
				FPARAM_1 = &TheParam_1 / _Alpha; 
				FPARAM_2 = _Alpha; /* Note that 1/(1+_Alpha) is the probability of success p. */
			end;
			if missing(Mean) OR missing(Variance) then do;
				status=1;
        		reason="Missing mean and/or variance";
			end;
		run;
%END_NDIST_LOOP:
	%end; /* Finding the mean/variance */


    data &outparm;
        set
            %do i=1 %to &ndist;
                countout_est_&i 
            %end;
        ;
        COUNT_KRI = &numSelectedKRIs;
        length SelectedKRIs $ 1024;
        SelectedKRIs = "&SelectedKris";
		KRI_Intercept = Intercept;
    run;
    
    data &outbluidest;
        set
            %do i=1 %to &ndist;
				%if &&fitstatus&i EQ 0 %then %do;  /* only append if fitstatus was 0 (i.e. success) */
	                fbluidest_&i 
				%end;
            %end;
        ;
    run;
	 

    data &outcov;
        set
            %do i=1 %to &ndist;
				%if &&fitstatus&i EQ 0 %then %do; /* only append if fitstatus was 0 (i.e. success) */
					CovMat_&i 
				%end;
            %end;
        ;
    run;

%DONE:

    /***** something went really bad *****/
    %if &abort eq 1 %then %do;
        %put ERROR: orva_freq_fit_kri macro ended abnormally.;
        data &outparm;
            format status 8. reason $100.;
            status=1;
            reason="&reason";
        run;
    %end;    
    /***** unable to fit --->  problem happened in this macro *****/
    %else %if &abort eq 2 %then %do;
        /*** setup resultant dataset with a failure status and reason for each distribution requested ***/
        data &outParm;
	        format status 8. reason $100.  distribution $12.;
	        %let ndist=1;
	        %let dist=%scan(&fdists, &ndist, ' ');
	        %do %while (&dist ne );	 
	           distribution = "&dist";
	           reason = "&reason";
	           status = 1;
	           output;
	           %let ndist=%eval(&ndist+1);
	           %let dist=%scan(&fdists, &ndist, ' ');
	        %end;
	        %let ndist=%eval(&ndist-1);
        run;
        /*** give up ***/
        %let ndist = 0;
    %end;


    /*- delete unwanted data sets  -*/
	proc datasets library=work memtype=data nolist nodetails;
    	delete  
			_parm_
			_kriblLossespobs_
          	_fitsummary_         
		      %do i=1 %to &ndist;
		          countout_est_&i 
		          %if %eval(&numSelectedKRIs) GT 0 %then %do;
					fbluidest_&i   
		          	fbluidesta_&i   
				  %end;
		          _jr_&i
				  CovMat_&i
			  %end;
        ;
	run;
    quit;

%mend orva_freq_fit_kri;


/*- find moments for fitted distribution -*/
%macro FindFreqMoments(parmdata=,fdist=);
    /* Needs to be called first */
    %orvd_&fdist;

    data &parmdata(drop= m1 m2 m3 m4 fm2 fm3 fm4);
        set &parmdata;
        %Moments&fdist
    run;
%mend;
