/*****************************************************************************************
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *  NAME:    %orva_freq_var_select.sas                                             
 *  PURPOSE: Selects significant regressors to be used later in frequency fitting
 *           11/06:  added override of selection procedure w/in proc reg 
 *  SUPPORT: lajack
 *
 *  NOTE: Please see the accompanying doc for pdfs and cdfs of the distributions
 *
 *  PARAMETERS:
 *
 *  data  =  Loss Data : dataset with the following variables: FIRST_DATE_OF_EVENT (internal only)
 *           FINANCIALDATE, THRESHOLD, AMOUNT, INTERNAL_BL_UID, CCVUID_0, CCVUID_1, CCUID.
 *
 *  missingExposure = Output from orva_range_merge macro.  Must have global_start and global_end dates.
 *                    Will not exist if dataType == C (consortium data), because orva_range_merge isn't called
 *                    for consortium data.  
 *
 *  ccUid =  key for current cell.
 *
 *  ccvUid_0 = cross-classification UID for cell containing internal business lines 
 *
 *  thresholdValue = the overridding threshold of loss data, defaults to <blank>
 *
 *  defaultThreshold = specifies the default threshold for data where THRESHOLD=.
 *
 *  interval = time interval size for which the analysis needs to be done, defaults to YEAR.    
 *             possibles values: YEAR, SEMIYEAR, QTR, MONTH2, MONTH, SEMIMONTH, WEEK2, WEEK
 *
 *
 *  selection = the model-selection option to be used to choose the best set of KRIs (regressors).  
 *              This should be a string, provided by the user, and it may take on 
 *              one of the following 5 values: Forward, Backward, MinR, MaxR, Stepwise.  
 *              Note that this is analogous to, but separate from, the parameter that 
 *              orva_sev_fit_kri gets for selecting the best severity KRIs.  
 *              Defaults to FORWARD.
 *
 *  KRIs = space-delimited list of the variable names of the businessline-level KRIs.
 *
 *  KRIsFirm  = space-delimited list of the variable names of the firm-level KRIs.
 * 
 *  kriblData = name of the global KRIBL dataset.
 *              Must have the following variables:
 *                  BLUID_KRI, RISK_FACTOR_VARIABLE_DATE, and KRIs, if any.
 *              
 *
 *  kriblDataFirm = name of the global KRIBFIRM dataset.
 *                  Must have the following variables:
 *                  RISK_FACTOR_VARIABLE_DATE, and KRIs, if any.
 *
 *  dataType = I (internal data) or C (consortium data).
 *
 *  outCompressedData = Output dataset containing (compressed) frequency counts, plus 
 *          the firm & businessline level KRIs (called KRI_:) and the panel dummy 
 *          variables for businesslines (called DUMMY_BLUID_:).  
 *          Also contains:  OFFSET (log (exposure/FSCALE) )  
 * 			Defaults to compressed_data.
 *
 *  outParmStatus  = Parameter status dataset output by macro.
 *
 *  outNotSelected = Column of variables (KRIs and dummies) that weren't selected as 
 *              significant by Proc Reg.  Defaults to not_selected.
 *
 *  outMonthlyKRIs = 
 *
 *****************************************************************************************/
%macro ORVA_freq_var_select(
                    data              =  ,
					missingExposure   =  ,
					global_range      =  ,
                    selection         =  FORWARD,
                    KRIs              =  ,
                    KRIsFirm          =  ,
					KRIsSev           =  ,
                    kriblData         =  ,
                    kriblDataFirm     =  ,
					internalScaling   =  ,
					ccUid             =  ,
                    ccvUid_0          =  ,
                    thresholdValue    =  ,
                    defaultThreshold  =  0,   
                    interval          =  QTR,
                   	outCompressedData =  compressed_data,
					outParmStatus     =  parm_status,
					outNotSelected    =  not_selected, 
					outMonthlyKRIs    =  monthly_kris,
					dataType          =  I,
					ccData            =  );                     

/*%orvh_displaypaths;*/


    %let status=0;
    %let reason=Successful;
    %let abort=0;
    %let fdist=;
    %let sdist=;

    /*- create the status data set upfront with unknown status-*/
    data &outParmStatus;
        format status 8. reason $100.;
        status=1;
        reason="Unknown";
    run;
    
    /*- remove anything in other output datasets up front -*/
    data &outCompressedData; 
        set _NULL_; 
    run;

	data &outNotSelected;
		set _NULL_;
	run;

    data &outMonthlyKRIs; 
        set _NULL_; 
    run;

	/*- error checking -*/
    %if &data=%str() %then %do;
        %put ERROR: The parameter DATA= must be specified.;
        %let abort=1;
        %let reason=MISSING DATA= PARAMETER;
        %goto DONE;
    %end;
    %if &global_range=%str() %then %do;
        %put ERROR: The parameter GLOBAL_RANGE= must be specified.;
        %let abort=1;
        %let reason=MISSING GLOBAL_RANGE= PARAMETER;
        %goto DONE;
    %end;

	/* so that we can use this macro for consortium data */
    %if &internalScaling=%str() %then %do;
		%let internalScalingData = 0;
	%end;
	%else %do;
		%let ds_nobs=0;
		%let dsid=%sysfunc(open(&internalScaling,I));
		%if &dsid NE 0 %then %do;
			%let ds_nobs=%sysfunc(attrn(&dsid,nlobs));
			%let rc=%sysfunc(close(&dsid));
		%end;
		%if &ds_nobs LE 0 %then %let internalScalingData = 0;
		%else %let internalScalingData = 1;
    %end;

 	/*- make sure we have data in our cell -*/
	data _workdata_; set &data; where CCUID=&ccUid; run;

	%let ds_nobs=0;
   	%let dsid=%sysfunc(open(_workdata_,I));
	%if &dsid NE 0 %then %do;
		%let ds_nobs=%sysfunc(attrn(&dsid,nlobs));
		%let rc=%sysfunc(close(&dsid));
	%end;
	%if &ds_nobs LE 0 %then %do;
        %put ERROR: No observations available in data.;
        %let abort=1;
        %let reason=No observations available in data;
        %goto DONE;
    %end;

	/* Will have missingExposure when dataType = I, not when dataType = C */
	%let MissingExposureInfo = 0;
	%if &dataType EQ I %then %do;
        %if &missingExposure=%str() %then %do;		
			%put ERROR: The parameter missingExposure= must be specified for Internal Data.;
        	%let abort=1;
        	%let reason=MISSING missingExposure= PARAMETER;
        	%goto DONE;
		%end;
	
		%let ds_nobs=0;
		%let dsid=%sysfunc(open(&missingExposure,I));
		%if &dsid NE 0 %then %do;
			%let ds_nobs=%sysfunc(attrn(&dsid,nlobs));
			%let rc=%sysfunc(close(&dsid));
		%end;
		%if &ds_nobs LE 0 %then %do;
			%if &dataType EQ I %then %do;
				%put ERROR: The file &missingExposure is empty, for Internal Data.  Cannot access global start and end dates.;
    	   		%let abort=1;
        		%let reason=Cannot access global start and end dates in file given by missingExposure= PARAMETER;
        		%goto DONE;
			%end;
		%end;
		%else %do;
			data _null_;
				set &missingExposure;
				if _N_=1 then do;
					if (internal_bl_uid NE -1) then call symput ("MissingExposureInfo", 1);
				end;
			run;
		%end;
    %end;
	
	/* Extract the global_start and global_end dates from the global_range table */
	data _null_;
		set &global_range;
		call symput("global_start", start_date);
		call symput("global_end", end_date);
	run;

	%if &dataType = C %then %do;
		%let bluidVar = basel_bl_uid;
	%end;
	%else %do;
		%let bluidVar = internal_bl_uid;
	%end;

	/* Create list of CCUIDs that go into this cell */
	%let CC_list=;
	%let CC_commalist=;
	%let CCVUID_1=;
	%let numCCUIDs=;
 	%GetCCUIDList();

	/*- data sanity check -*/

    data _dataThisCellNewThresh_(rename=(&bluidVar=bluid)); /* subset the data for those losses in this cell */
        set &data end=LAST; 
		where CCUID in (&CC_commalist) and missing(AMOUNT) EQ 0; /* only keep obs with non-missing amounts */
        retain NNeg 0 NExcess 0;
        if AMOUNT<0 then NNeg+1;
        else do;
		/* For use in 3.3, if user selects "use only losses above threshold for freq fitting" */
/*			%if &thresholdValue NE %str() %then %do;*/
/* 			   THRESHOLD = &thresholdValue;  */
/*			   tempthreshold = THRESHOLD;*/
/*            %end;*/
            
			if THRESHOLD=. then THRESHOLD=&defaultThreshold;
       
		    /* later, when the dataset _work_&CCUID_curr is created, 
		    a missing THRESHOLD is set to the mean threshold for losses in that CCUID */
       
			if AMOUNT>THRESHOLD then NExcess+1;
		end;

        if LAST then do;
            call symput("NNeg", NNeg);
            call symput("NExcess", NExcess);   
        end;
    run;

    %if &SYSERR^=0 & &SYSERR ^= 4 %then %do;
        %put ERROR: Unable to check data for sanity.;
        %let abort=1;
        %let reason=Unable to check data for sanity;
        %goto DONE;
    %end;

    %if &NNeg>=1 %then %do;
        %put Warning: There are &NNeg observations with negative loss amounts.;
        %let abort=0;
        %let reason=Negative Loss Amounts;
    %end;

    %if &NExcess<1 %then %do;
        %put ERROR: There are no observations above the threshold.;
        %let abort=1;
        %let reason=NO VALID OBSERVATIONS;
        %goto DONE;
    %end;


	%let numKRIs=1;
    %let KRI_curr=%scan(&KRIs, &numKRIs, ' ');
    %do %while (&KRI_curr ne );
        %let numKRIs = %eval(&numKRIs+1);
        %let KRI_curr = %scan(&KRIs, &numKRIs, ' ');
    %end;
    %let numKRIs=%eval(&numKRIs-1);

    %let numKRIsFirm=1;
    %let KRI_curr=%scan(&KRIsFirm, &numKRIsFirm, ' ');
    %do %while (&KRI_curr ne );
        %let numKRIsFirm = %eval(&numKRIsFirm+1);
        %let KRI_curr = %scan(&KRIsFirm, &numKRIsFirm, ' ');
    %end;
    %let numKRIsFirm=%eval(&numKRIsFirm-1);
 
	%let allfreqKRIs = &KRIs &KRIsFirm;

/*    %let numKRIsSev=1;*/
/*    %let KRI_curr=%scan(&KRIsSev, &numKRIsSev, ' ');*/
/*    %do %while (&KRI_curr ne );*/
/*        %let numKRIsSev = %eval(&numKRIsSev+1);*/
/*        %let KRI_curr = %scan(&KRIsSev, &numKRIsSev, ' ');*/
/*    %end;*/
/*    %let numKRIsSev=%eval(&numKRIsSev-1);*/


	/* 	Cannot simply add NUMKRIS and NUMKRISFIRM, because there might be some severity KRIs that we need to 
	keep around for calculating p_obs (a.k.a. pScaledExcess) in the freq_fit_kri macro.  
	So, we must detect whether there are KRIs in the &kribldata dataset. */ 

	proc contents data=&kribldata noprint out=_foo(keep=name);
    run;

	data _null_;
		set _foo end=last;
       	if compare(scan(name, 1, '_'), 'KRI') EQ 0 then sumkris + 1;

		if last then do;
			call symput("numBLKRIsPresent", sumkris);
		end;
    run;

	/*  */
	
	%if &numBLKRIsPresent GT 0 %then %let possiblyKRIs = KRI_:;
	%else %let possiblyKRIs = ;

	/*- create dataset with all business lines for our cell -*/
	/*  NOTE:  _dataThisCellNewThresh_ contains EITHER:
	      -  only one unique CCUID (whenever &CCUID is for an "interior" cell, OR:
          -  multiple CCUIDs (whenever &CCUID is for a "total" cell (either row, col, or total-total). 
		NOTE that NONE of the CCUIDs in _dataThisCellNewThresh_ will be the CCUID of a total cell. */
	options sortdup=logical;
	proc sort data=_dataThisCellNewThresh_(keep= bluid ccuid ccvuid_0 ccvuid_1) nodup out=_map;
		by bluid;
	run;

	/* Must only do this block if numBLKRIsPresent>0; otherwise, we just have the "fake" &kribldata dataset... */
	%if numBLKRIsPresent GT 0 %then %do;
		/* Create kribldata_mod:  A modified &kribldata dataset, which creates a new obs for the
		1st of month in which global_start falls, with same KRI values as the first available obs w/in
		each bluid by-group.  THIS SHOULD BE UNNECESSARY IF RELEVANT KRI OBS AREN'T THROWN OUT IN THE DATA LOAD. 
		RE-VISIT THIS AT 3.3. !!! */
		proc sort data=&kribldata out=mysortedkribl;
			by bluid_kri risk_factor_variable_date ;
		run;
		data kribldata_mod;
			set mysortedkribl; 
			by bluid_kri risk_factor_variable_date ;

			if first.bluid_kri EQ 1 and first.risk_factor_variable_date EQ 1 then do;
				if risk_factor_variable_date GE intnx('month', &global_start, 0) 
						AND RISK_FACTOR_VARIABLE_DATE LE &global_end   
				then do;
					
					if ( intnx('month', RISK_FACTOR_VARIABLE_DATE, 0) GT intnx('month', &global_start, 0) ) then do;
						output; /*  output the current one */
					/*  create a new obs for 1st of month in which global_start falls, with same KRI values */
						RISK_FACTOR_VARIABLE_DATE = intnx('month', &global_start, 0) ;
						output;
					end;
					else do;  
						output; 
					end;
				end;
			end;
			else do;  
				output;  
			end;
		run;
	%end;
	%else %do;
		data kribldata_mod; set &kribldata; run;
	%end;

    /*-  From kribldata_mod, create _kribl_curr:  the KRIBL dataset subsetted for just those BLs 
	in all CCUIDs in &CC_commalist -*/
	/* how does this look if CONSORTIUM data???? */
	proc sort data=kribldata_mod /*&kribldata*/ out=kribldata_sorted (rename=(BLUID_KRI=bluid));
		by BLUID_KRI;
	run;
	data _kribl_curr_try(drop=CCUID CCVUID_0 CCVUID_1); /* drop CCUID, CCVUID_0, and CCVUID_1, 
	because after the merge, they don't mean anything */ 
        merge kribldata_sorted _map;
		by bluid;
		if missing(CCUID) then delete; /* gets rid of obs from kribldata_sorted that didn't match 
		with a bluid in _map */
		
		/*  create an obs for exactly the global start date, 
		using an obs from earlier in the month in which global_start falls */

		/* ???? must only do this if-then-end if numBLKRIsPresent>0; 
		otherwise, we just have the "fake" &kribldata dataset...??? */
 		%if numBLKRIsPresent GT 0 %then %do;
			if ( RISK_FACTOR_VARIABLE_DATE GE intnx('month', &global_start, 0) 
				AND RISK_FACTOR_VARIABLE_DATE LE &global_end )  
			then do;
				if ( intnx('month', RISK_FACTOR_VARIABLE_DATE, 0) EQ intnx('month', &global_start, 0) ) then do;
					RISK_FACTOR_VARIABLE_DATE = &global_start;
				end;
				output;
			end;
		%end;
		%else %do;
			output;
		%end;
	run;

	data _kribl_curr;
		set _kribl_curr_try(rename=(RISK_FACTOR_VARIABLE_DATE=date));
	run;

    /*- create dataset with all businesslines for our cell -*/
    options sortdup=logical;
    proc sort data=_map (keep = bluid) NODUP out=_cellbls; 
       by bluid; 
    run;

	/* Count the number of BLs in the given file EXCEPT THE LAST ONE */
	/* Thus _cellbls has only the bls that will get a dummy var (i.e. one less than the total) */
	%let numDummyVars= 0;
	%let numBLUIDsThisCell= 0;
	%let extraBL= 0;
    data _cellbls _cellbls_all; 
        set _cellbls end=last;
        bl_index+1;
        if last then do;
			/* even if no freqKRIs are used, we still create panel Dummy vars */
			call symput("numBLUIDsThisCell", bl_index);
			call symput("numDummyVars", bl_index-1);	
			call symput("extraBL", bluid); /* the BLUID of the BL that does not get a dummy var named for him */
			output _cellbls_all;
        end;
        else do;
			output _cellbls;
			output _cellbls_all;
		end;
    run;

/* ************************************** */
/*  Extract obs from missingexposure that correspond to ALL bluids in our cell */

	/* ??????????? MUST do this for all CCUIDs that go into a total cell */
	%let MissInfoForThisCell = 0;
	/* initialize each flag "MissInfoForThisCell&CCUID_curr" to 0 */
	%let numCCUIDs=1;
	%let CCUID_curr=%scan(&CC_list, &numCCUIDs, ' ');
    %do %while (&CCUID_curr ne );
		%let MissInfoForThisCell&CCUID_curr = 0; 
        %let numCCUIDs = %eval(&numCCUIDs+1);
        %let CCUID_curr = %scan(&CC_list, &numCCUIDs, ' ');
    %end;
    %let numCCUIDs=%eval(&numCCUIDs-1);



	%if &MissingExposureInfo %then %do;

	    data miss_exposure_allCCUIDS(rename=(new_month=date ) 
						    drop=start_missing_date end_missing_date global_start global_end );
	        if _N_ eq 1 then do;
	            declare hash blHash(dataset: '_cellbls_all', hashexp: 100);
	            rc = blHash.defineKey('BLUID');
	            rc = blHash.defineData('BL_INDEX');
	            rc = blHash.defineDone();
	            call missing(BL_INDEX);
	        end;
	        set &missingExposure end=last; where CCUID in (&CC_commalist);
	        bluid=&bluidVar;
	        rc = blHash.find();
	        if ( (rc = 0) AND (new_month GE &global_start) AND (new_month LE &global_end) ) then do;
				output_flag + 1;
				output; 
			end;
			if last then do;
				if output_flag then call symput("MissInfoForThisCell", 1);
				else call symput("MissInfoForThisCell", 0);
			end;
	        drop rc bl_index &bluidVar output_flag;
	    run;
		
		%if &MissInfoForThisCell %then %do; /* Create individual datasets for each CCUID */
			%let numCCUIDs=1;
			%let CCUID_curr=%scan(&CC_list, &numCCUIDs, ' ');
		    %do %while (&CCUID_curr ne );
				data miss_exposure&CCUID_curr(drop=flag);
					set miss_exposure_allCCUIDs end=last; 
					retain flag 0; 
					if CCUID = &CCUID_curr then do;
						flag = 1;
						output;
					end;
					if last then do;
						if flag then call symput("MissInfoForThisCell&CCUID_curr", 1);
						else call symput("MissInfoForThisCell&CCUID_curr", 0);
					end;
				run;

				%if &&MissInfoForThisCell&CCUID_curr %then %do;	
					proc sort data=miss_exposure&CCUID_curr out=miss_exposure&CCUID_curr; 
						by bluid date;
					run;
				%end;


		        %let numCCUIDs = %eval(&numCCUIDs+1);
		        %let CCUID_curr = %scan(&CC_list, &numCCUIDs, ' ');
		    %end;
		    %let numCCUIDs=%eval(&numCCUIDs-1);
		%end;			

	%end;


	%if &internalScalingData %then %do; /* this input dataset only has YEARLY data */
		data freqscalei(rename=(bluid_sc=bluid) drop=sscale sscale_init cpi_adj i year);
			format date DATE9.;
	        if _N_ eq 1 then do;
	            declare hash blHash(dataset: '_cellbls_all', hashexp: 100);
	            rc = blHash.defineKey('bluid');
	            rc = blHash.defineData('BL_INDEX');
	            rc = blHash.defineDone();
	            call missing(BL_INDEX);
	        end;
	        set &internalScaling;
	        bluid=bluid_sc;
	        rc = blHash.find();
			if rc = 0 then do;
				/* this input dataset only has YEARLY data */
				if (year=year(&global_start)) AND (day(&global_start) NE 1) then do;
					date = &global_start; /* adjust date up to &global_start date */
					output;
				end;
				if 1 then do i=1 to 12;
					date = mdy(i,1,year);  
		     		 /*check that date falls between global_start and global_end */
					if ((date GE &global_start) AND (date LE &global_end)) then output;
				end;
			end;
	        drop rc bl_index;
	    run;

		proc sort data=freqscalei out=freqscalei; 
			by bluid date;
		run;
	%end;

/* ************************************** */

    /* sort by date in order to match-merge with Firm-Level KRI data set */
	%let numKRIsPresent = &numBLKRIsPresent; /* initialize */
    %if "&kriblDataFirm" ne "" %then %do; 
		%let ds_nobs=0;
	   	%let dsid=%sysfunc(open(&kriblDataFirm,I));
		%if &dsid NE 0 %then %do;
			%let ds_nobs=%sysfunc(attrn(&dsid,nlobs));
			%let ds_nvars=%sysfunc(attrn(&dsid,nvars));
			%let rc=%sysfunc(close(&dsid));
		%end;
		%if ((&ds_nobs GT 0) and (&ds_nvars GT 0)) %then %do;

			data _firmKRIdata_;
		        set &kriblDataFirm(rename=(RISK_FACTOR_VARIABLE_DATE=date));
		        keep date KRI_:; 
				if ( date GE intnx('month', &global_start, 0) AND date LE &global_end )  then do;
					if ( intnx('month', date, 0) EQ intnx('month', &global_start, 0) ) then do;
						date = &global_start;
					end;
					output;
				end;
		    run; 

			proc contents data=_firmKRIdata_ noprint out=_foofirm(keep=name);
		    run;

			data _null_;
				set _foofirm end=last;
		       	if compare(scan(name, 1, '_'), 'KRI') EQ 0 then sumfirmkris + 1;

				if last then do;
					call symput("sumfirmkris", sumfirmkris);
				end;
		    run;
			%let numKRIsPresent = &sumfirmkris + &numBLKRIsPresent;
			%if &numKRIsPresent GT 0 %then %let possiblyKRIs = KRI_:;  /* I just changed from "if &numBLKRIsPresent" */
			%else %let possiblyKRIs = ;
			/* if no BLKRIs were chosen for freq fitting, then _kribl_curr contains just one obs per
			internal BLUID that exists in this cell.

			So we need to expand _kribl_curr to be 
			one obs per month per internal bluid...  This will ensure the following merge will put 
			a bluid with each obs...*/
 			
			%if &numBLKRIsPresent EQ 0 %then %do; /* Recall: &numBLKRIsPresent is the number of BL KRIs detected 
			in the input dataset &kribldata, whether they are to be used for freq or sev or neither.  
			Must only do this block if there are no KRIs in the system. */
				/* Goal:  if there are 5 internalbluids in this cell and 11 months in the range between
				global start and global end, then there will be 5*11=55 observations in the new _kribl_curr 
				dataset, after this block executes.*/

				/*  maybe make use of &numBLUIDsThisCell ..???*/

				%let counter = 1;
				%do %while (&counter LE &numBLUIDsThisCell);
					data _null_;
						set _cellbls_all;
						if _n_ EQ &counter then do;
							call symput("curr_bluid", bluid);
						end;
					run;
					%let curr_bluid = %left(&curr_bluid);
					data _kribl_curr_one&curr_bluid;
						set _firmKRIdata_;
						bluid = &curr_bluid;
					run;
					data _kribl_curr_expanded; /* stack the datasets */
						%if &counter EQ 1 %then %do;  set _kribl_curr_one&curr_bluid; %end;
						%else %do; set _kribl_curr_expanded _kribl_curr_one&curr_bluid; %end;
					run;
					data _kribl_curr;
						set _kribl_curr_expanded;
					run;
					%let counter = %eval(&counter+1);
				%end;
			%end;

			proc sort data=_kribl_curr; 
		        by date; 
		    run;

		    data _kribl_curr;
		        merge _kribl_curr _firmKRIdata_ ;
		        by date;
		    run; 
		%end;
    %end;
    /* Now sort appropriately to merge with the fscale data & the loss data */
    proc sort data=_kribl_curr; 
        by bluid date; 
    run;
	
	%let invalidFscales = 0;
	%if &internalScalingData %then %do;
		data _kribl_curr;
			merge _kribl_curr  freqscalei;  
			by bluid date;
			if missing(FSCALE) then FSCALE = 1; /* takes care of boundary month */
			if (FSCALE LE 0) then do;  /* check whether FSCALE is invalid */
				FSCALE = 1; 
				call symput("invalidFscales", %eval(&invalidFscales+1));
			end;
		run;
	%end;
	%else %do;
		data _kribl_curr;
			set _kribl_curr;
			if missing(FSCALE) then FSCALE = 1;
			/* There shouldn't be any fscales at all in _kribl_curr because I didn't keep that variable earlier */
			if (FSCALE LE 0) then do;  /* check whether FSCALE is invalid */
				FSCALE = 1; 
				call symput("invalidFscales", %eval(&invalidFscales+1));
			end;
		run;
	%end;
	%if &invalidFscales > 0 %then %do;
		%put Warning: There were &invalidFscales invalid values of FSCALE (less than equal to 0).;
        %let abort=0;
        %let reason=Invalid FSCALE values;
	%end;




	/* ??????????Maybe start here????  (i.e. encapsulate this all into a macro that would be called 
	multiple times for each CCUID that feeds into a TOTAL cell...? */

	data _kribllosses_exposure3;
		set _NULL_;
	run;
	
	%let numCCUIDs=1;
	%let CCUID_curr=%scan(&CC_list, &numCCUIDs, ' ');
    %do %while (&CCUID_curr ne );

		/* ************************************ */

	    /* Subset the losses to get only those losses in our cell */
		data _work_&CCUID_curr(keep=threshold bluid FINANCIALDATE amount CCVUID_0 CCVUID_1 CCUID);
				set _dataThisCellNewThresh_;   
				where CCUID=&CCUID_curr and FINANCIALDATE GE &global_start AND FINANCIALDATE LE &global_end; 
				%if &dataType = I %then %do;
					startofmonth = intnx("month", FINANCIALDATE, 0 ) ;
					if startofmonth LT &global_start then FINANCIALDATE = &global_start;
					else FINANCIALDATE = startofmonth ;
				%end;
				%else %do;
					startofmonth = intnx("month", FINANCIALDATE, 0 ) ;
					if startofmonth LT &global_start then FINANCIALDATE = &global_start;
					else FINANCIALDATE = startofmonth ;
				%end;				
		run;

		/* verify there ARE losses for this CCUID */
		%let ds_nobs=0;
	   	%let dsid=%sysfunc(open(_work_&CCUID_curr,I));
		%if &dsid NE 0 %then %do;
			%let ds_nobs=%sysfunc(attrn(&dsid,nlobs));
			%let rc=%sysfunc(close(&dsid));
		%end;
		%if (&ds_nobs LE 0) %then %goto NEXTCC;  
		/* We are done for this CCUID.  Now do the next CCUID in &CC_list.  */


	    proc sort data = _work_&CCUID_curr out =_work_&CCUID_curr;
	        by bluid FINANCIALDATE;
	    run;

		/* Find the mean Threshold,  */
		proc summary data=_work_&CCUID_curr;
			var THRESHOLD;
			output out=_worksummary_&CCUID_curr mean=avg;
		run;
		data _null_;
			set _worksummary_&CCUID_curr;
			call symput("meanThreshold", avg);
		run;

		/* Insert mean Threshold wherever THRESHOLD is missing; Keep only losses that exceed THRESHOLD */
		/* No --- keep ALL loss events, regardless of threshold; might change in 3.3. */
/*		data _work_&CCUID_curr;*/
/*			set _work_&CCUID_curr;*/
/*			if THRESHOLD=. then THRESHOLD=&meanThreshold;*/
/*			if amount > THRESHOLD;   the "subsetting if" */
/*		run;*/

		%let invalidFscales = 0;
		%if &numKRIsPresent GT 0 %then %do;
			options sortdup=logical;
		    proc sort data=_work_&CCUID_curr (keep= bluid rename=(bluid=thebluid)) NODUP out=_bls_&CCUID_curr; 
		       by thebluid; 
		    run;
    		data _bls_&CCUID_curr; 
        		set _bls_&CCUID_curr;
        		bl_index+1;
			run;

			data _kribl_curr_&CCUID_curr;
		        if _N_ eq 1 then do;
		            declare hash blHash(dataset: "_bls_&CCUID_curr", hashexp: 100);
		            rc = blHash.defineKey('thebluid');
		            rc = blHash.defineData('BL_INDEX');
		            rc = blHash.defineDone();
		            call missing(BL_INDEX);
		        end;
		        set _kribl_curr;
		        thebluid=bluid;
		        rc = blHash.find();
				if rc = 0 then output;
		        drop rc bl_index thebluid;
		    run;

			data _kriblLosses_&CCUID_curr(compress=binary);
			/* !!!!!! in this merge, need to use just _kribl_curr observations for the bls in &ccuid_curr */
		        merge _kribl_curr_&CCUID_curr   
		              _work_&CCUID_curr(rename=(FINANCIALDATE=date ));
		        by bluid date;
				if missing(FSCALE) then FSCALE = 1;
				if (FSCALE LE 0) then do;  /* check whether FSCALE is invalid */
					FSCALE = 1; 
					call symput("invalidFscales", %eval(&invalidFscales+1));
				end;

				/* add threshold if missing */
/*				%if &thresholdValue NE %str() %then %do;*/
/*	 			   THRESHOLD = &thresholdValue;  */
/*	            %end;*/
/*	            %else %if &defaultThreshold NE %str() %then %do;*/
/*	               if THRESHOLD=. then THRESHOLD=&defaultThreshold;*/
/*	            %end;*/
/*	            %else %do;*/
/*	               if THRESHOLD=. then threshold = &meanThreshold; */
/*	            %end;*/
				if THRESHOLD=. then threshold = &meanThreshold;


		    run;
		%end;
		%else %do;
			data _kriblLosses_&CCUID_curr(compress=binary);
		        set _work_&CCUID_curr(rename=(FINANCIALDATE=date ));
				if missing(FSCALE) then FSCALE = 1;
				if (FSCALE LE 0) then do;  /* check whether FSCALE is invalid */
					FSCALE = 1; 
					call symput("invalidFscales", %eval(&invalidFscales+1));
				end;
				/* threshold was already set for _work_&CCUID_curr */
			run;
		%end;
		%if &invalidFscales > 0 %then %do;
			%put Warning: There were &invalidFscales invalid values of FSCALE (less than equal to 0).;
	        %let abort=0;
	        %let reason=Invalid FSCALE values;
		%end;


	    proc sort data = _kriblLosses_&CCUID_curr out =_kriblLosses_&CCUID_curr(compress=binary);
	        by bluid date threshold;
	    run;

	    data _kriblLosses_&CCUID_curr(compress=binary);
	        set _kriblLosses_&CCUID_curr;
	        by bluid date threshold;
	        if first.threshold=1 then frequency = 0;
	        if amount NE . then do;
	            frequency+1;
	        end;
	        if last.threshold=1 then output;
	    run;

		data _kriblLosses_exposure_&CCUID_curr(/*drop=DaysInMonth start_of_curr_interval end_of_curr_interval */ compress=binary );
			
			%if &&MissInfoForThisCell&CCUID_curr %then %do;
				merge _kriblLosses_&CCUID_curr  miss_exposure&CCUID_curr;  by bluid date;
			%end;
			%else %do;
				set _kriblLosses_&CCUID_curr;
			%end;
		
			/* obs from dataset "_kriblLosses_" will have missing(exposure) */
			if missing(exposure) and missing(date) then do;
				put "ERROR: date is missing (_kriblLosses_exposure_ data step).";
			end;
			DaysInMonth = intck("day", date, intnx("month", date, 1));
			if missing(DaysInMonth) then put "ERROR: DaysInMonth is missing (_kriblLosses_exposure_ data step).";
			/* Adjustment for &global_end falling in the middle of a month */
			/* new OCT06:  added "+1" */
			if (date+DaysInMonth GT &global_end) then DaysInMonth = &global_end - date + 1; 
			start_of_curr_interval = intnx("&interval", date, 0);
			if missing(start_of_curr_interval) then put "ERROR: start_of_curr_interval is missing (_kriblLosses_exposure_ data step).";
			end_of_curr_interval = intnx("&interval", date, 1);
			if missing(end_of_curr_interval) then put "ERROR: end_of_curr_interval is missing (_kriblLosses_exposure_ data step).";
			if missing(exposure) then exposure = DaysInMonth / (end_of_curr_interval - start_of_curr_interval);
			if missing(exposure) then put "ERROR: exposure is missing (_kriblLosses_exposure_ data step).";		

			/* data step for _kribllosses_exposure3_ will delete those obs with 0 exposure */

		run;

		data _kribllosses_exposure2_&CCUID_curr(drop=prev_: next_:);
			set _kribllosses_exposure_&CCUID_curr end=last;
			by bluid date; /* is this right??? */
			retain prev_date prev_DaysInMonth prev_FSCALE;
			/* "date" will be the first of a month for every obs, except possibly the first obs (in a by group?), 
			in which case "date" could = &global_start */
			if first.bluid EQ 1 and date EQ &global_start then do; /* testing whether the first obs in the bygroup */
				if missing(frequency) then frequency = 0; /* i.e. this obs is from the missing table */
				%SaveCurrValuesIntoPrevVariables();
				%SetFSCALEtoCurrOrPrevOrOne();
				%SetTHRESHOLDtoCurrOrPrevOrMean();	
				output;
			end;
			else if first.bluid EQ 1 and date GT &global_start then do;
			/* Need to create one obs with a 0 count for the period from &global_start 
			up to the month of the current obs.   */
				%SaveCurrValuesIntoNextVariables();
				date = &global_start; 
				frequency = 0;
				DaysInMonth = next_date - date; /* this interval may be shorter or longer than a month... */
				/* denominator is the length of time, in days, of the fit interval (e.g. yearly, quarterly,etc.) */
				exposure = DaysInMonth/(end_of_curr_interval - start_of_curr_interval); 
				%SetFSCALEtoCurrOrPrevOrOne();
				%SetTHRESHOLDtoCurrOrPrevOrMean();
				output;
				%SaveCurrValuesIntoPrevVariables();
				%ResetCurrValuesWithNextVariables();  /* I'm reversing these */		
				%SetFSCALEtoCurrOrPrevOrOne();
				%SetTHRESHOLDtoCurrOrPrevOrMean();	
				if missing(frequency) then frequency = 0; /* i.e. this obs is from the missing table */
				output;
				%SaveCurrValuesIntoPrevVariables();
			end; /* now the first and 2nd obs are good to go */
			else if first.bluid EQ 1 and date LT &global_start then do;
				/* ERROR!!!! */
				exposure = 0;
				put "ERROR: date is before global_start date (_kriblLosses_exposure2 data step).";
			end;
			/* now go through and add extra obs where needed, with 0 counts */
			else if date GT prev_date + prev_DaysInMonth then do;
				%SaveCurrValuesIntoNextVariables();
				date = prev_date + prev_DaysInMonth; 
				frequency = 0;
				DaysInMonth = next_date - date; /* this interval may be shorter or longer than a month... */
				/* denominator is the length of time, in days, of the fit interval (e.g. yearly, quarterly,etc.) */
				exposure = DaysInMonth /(end_of_curr_interval - start_of_curr_interval);
				%SetFSCALEtoCurrOrPrevOrOne();
				%SetTHRESHOLDtoCurrOrPrevOrMean();
				output;
				%SaveCurrValuesIntoPrevVariables();
				%ResetCurrValuesWithNextVariables();  /* I'm reversing these */	
				%SetFSCALEtoCurrOrPrevOrOne();
				%SetTHRESHOLDtoCurrOrPrevOrMean();	
				if missing(frequency) then frequency = 0; /* i.e. this obs is from the missing table */
				output;		
				%SaveCurrValuesIntoPrevVariables();
			end;	
			else do;  /* no need to add an extra observation BEFORE this one, 
					but must save date, DaysInMonth, FSCALE, and THRESHOLD to use when handling the next obs */
				%SaveCurrValuesIntoPrevVariables();
				%SetFSCALEtoCurrOrPrevOrOne();
				%SetTHRESHOLDtoCurrOrPrevOrMean();	
				if missing(frequency) then frequency = 0; /* i.e. this obs is from the missing table */
				output;		
			end;
			if last.bluid EQ 1  then do; /* if this is the last obs for this by-group */
				/* note that we already outputted this obs in the final "else do" above... we only need to 
					check whether we should create a new obs AFTER this obs... */
				if date+DaysInMonth LT &global_end then do; /* might need an obs AFTER this one, up until global_end */
					date = date+DaysInMonth;
					frequency = 0;
					DaysInMonth = &global_end - date + 1;
					/* denominator is the length of time, in days, of the fit interval (e.g. yearly, quarterly,etc.) */
					exposure = DaysInMonth / (end_of_curr_interval - start_of_curr_interval);
					%SetFSCALEtoCurrOrPrevOrOne();
					%SetTHRESHOLDtoCurrOrPrevOrMean();
					output;
				end;
			end;
		run;

		data _kribllosses_exposure3_&CCUID_curr;
			set _kribllosses_exposure2_&CCUID_curr;
			/* FSCALE was previously set to 1 (in _kriblLosses_) if missing. */
			/* Create a new var for each count with non-zero exposure:  namely, the log of the (count+.1) */
			if exposure NE 0 then do;
				if missing(FSCALE)  
					then put "ERROR: FSCALE is missing (_kriblLosses_exposure_ data step).";
				log_adjcount = log(frequency*(1/exposure)*FSCALE+.1);
			end;
			else delete;   /* delete obs with zero exposure */
		run;

		/* ************************************ */

		/* now concatenate 'em;  i.e. stack them */
		data _kribllosses_exposure3;
			set _kribllosses_exposure3 _kribllosses_exposure3_&CCUID_curr;
		run;


%NEXTCC:
        %let numCCUIDs = %eval(&numCCUIDs+1);
        %let CCUID_curr = %scan(&CC_list, &numCCUIDs, ' ');
    %end;  /*  %do %while (&CCUID_curr ne ); */

	%let numCCUIDs=%eval(&numCCUIDs-1);
	/* END of LOOP on CCUIDs */






	    /* Make PanelDummyVars And Log Adj Counts */
	    /* If there are M unique bluids, then create M-1 dummy variables. */

	%if &numDummyVars GT 0 %then %do;	
	    data _trcellbls;
	        set _cellbls;
	        format bluidname $40.;
	        bluidname = "dummy_bluid_" || left(bluid);
	    run;
	    proc transpose data=_trcellbls out=_trcellbls;
	        id bluidname; var bluid;
	    run;

	    data _kriblLosses_exposure3(drop=_NAME_  compress=binary) ;
	        merge _kriblLosses_exposure3 _trcellbls ;
	    run; 
		data _cellbls (rename=(bluid=thebluid)); 
			set _cellbls; 
		run;

		/* Set dummy vars to 0 or 1 */
	    data _kribllosses_exposure3 (/*drop= BLUID_KRI*/ compress=binary);
	        if _N_ eq 1 then do;
	            declare hash blHash(dataset: '_cellbls', hashexp: 100);
	            rc = blHash.defineKey('thebluid');
	            rc = blHash.defineData('BL_INDEX');
	            rc = blHash.defineDone();
	            call missing(BL_INDEX);
	        end;
	        set _kribllosses_exposure3;
	        array dummy_[&numDummyVars] dummy_bluid_:;
	        do i = 1 to &numDummyVars;
	            dummy_[i] = 0;
	        end;
	        thebluid=bluid;
	        rc = blHash.find();
	        if rc = 0 then dummy_[BL_index]=1;
	        drop rc bl_index i thebluid;
	    run;
	%end;









%if (&numDummyVars GT 0) %then %let possiblyDummyVars = dummy_bluid_: ;
%else %let possiblyDummyVars = ;

%if ((&numDummyVars GT 0) OR (%eval(&numKRIs+&numKRIsFirm) GT 0)) %then %do;
	
	/* Select the variables with significant impact on the counts */
	proc reg data=_kriblLosses_exposure3 outest=_regResults_a   /* tableout noprint*/;
		/*array _kri_[&numKRIs] &KRIs; */ /* can't use arrays in proc reg !!! */
		/* Don't use KRI_: or you'll use the severity KRIs too, which is wrong */
		model log_adjcount = &allfreqKRIs &possiblyDummyVars / selection = &selection;
	run;
 
    proc transpose data=_regResults_a(drop=_RMSE_ Intercept log_adjcount) out=_regResults_;
        id _TYPE_; 
    run;

	/* save the orig for debugging purposes */
/*	data _regResultsOrig_;  */
/*		set _regResults_;*/
/*	run;*/

	/* override selection procedure w/in proc reg */
/*	data _regResults_;*/
/*		set _regResults_;*/
/*		if abs(PARMS) < 2E-8 then delete;  */
/*	run;*/

	data _regResults_ &outNotSelected(rename=(PARMS=Estimate)); 
		set _regResults_ end=last;
		length SelectedKRIs $ 1024;
		retain SelectedKRIs "";

		if PARMS NE . then do;
			if compare(scan(_NAME_, 2, '_'), 'bluid') EQ 0 then numSelectedBLs + 1;
			else do;
				numSelectedKRIs + 1;
				SelectedKRIs =  trim(SelectedKRIs) || " " || trim(_NAME_)  ;
			end;
			output _regResults_;
		end;
		else do;
			output &outNotSelected;
		end;
		if last then do;
			call symput("numSelectedKRIs", numSelectedKRIs);
			call symput("numSelectedBLs", numSelectedBLs);
			call symput("SelectedKRIs", trim(SelectedKRIs));
		end;
		drop numSelectedKRIs numSelectedBLs SelectedKRIs;
	run;
	%if &numSelectedBLs EQ 0 %then %do;
		%let possiblyDummyVars = ;
		data _kriblLosses_selected_(compress=binary);
			set _kriblLosses_exposure3;
			keep bluid date THRESHOLD frequency adjexposure &possiblyKRIs;
			if exposure EQ 0 then delete;
			else if missing(exposure) then put "Warning!!! exposure is missing (_kriblLosses_selected_ data step).";
			if missing(FSCALE) then put "Warning!!! FSCALE is missing (_kriblLosses_selected_ data step).";
			adjexposure=exposure/FSCALE;
			if missing(adjexposure) then put "Warning!!! adjexposure is missing (_kriblLosses_selected_ data step).";
		run;
	%end;
	%else %do;
		/* Set up a temporary file to write our data step code to */
	    data _NULL_;
	        length path $ 256;
	        path = pathname("work");
	        call symput('path',trim(path));
	    run;
	    %if &SYSERR NE 0 & &SYSERR NE 4 %then %do;
	        %put ERROR: Unable to obtain path for temporary workspace.;
	        %let abort=1;
	        %let reason=Unable to obtain path for temporary workspace;
	        %goto DONE;
	    %end;
	    %put &path;     
	    filename tmp "&path\freq_fit_tmp";

	/*  if _regResults_ has no observations, i.e. no bls were selected,
		then this data step won't run, thus file tmp will be empty,
		thus kriblLosses_selected will never get created.   */

		data _null_;
	        set _regResults_ end=last;
	        file tmp;
	        if _N_ = 1 then do;
	            put 'data _kriblLosses_selected_(compress=binary);';
	            put 'set _kriblLosses_exposure3 ;';
				put 'keep bluid date THRESHOLD frequency adjexposure &possiblyKRIs '; 
			end;
			/* We are keeping ALL the KRIs, even the severity KRIs, which we'll need 
			in orva_freq_fit_kri, for computing pScaledExcess.  We're also keeping 
			the selected KRIs, if any, and the selected dummy_bluid's.   */
			if compare(scan(_NAME_, 2, '_'), 'bluid') EQ 0 then put _NAME_;
	        if last then do;
				put ' ; if exposure EQ 0 then delete; ';
				put ' else if missing(exposure) then put "ERROR: exposure is missing (_kriblLosses_selected_ data step).";';
				put ' if missing(FSCALE) then put "ERROR: FSCALE is missing (_kriblLosses_selected_ data step).";';
				put ' adjexposure=exposure/FSCALE;';
				put ' if missing(adjexposure) then put "ERROR: adjexposure is missing (_kriblLosses_selected_ data step).";';
				put ' run; ';
			end;
	    run;
	    %if &SYSERR NE 0 & &SYSERR NE 4 %then %do;
	        %put ERROR: Unable to create or to write to code file &path/LJ_ABC.;
	        %let abort=1;
	        %let reason=Unable to create or to write to code file &path/LJ_ABC.;--
	        %goto DONE;
	    %end;
	        
	    /* run the program we just made */
	    %inc "&path\freq_fit_tmp";

	    %if &SYSERR NE 0 & &SYSERR NE 4 %then %do;
	        %put ERROR: Unable to run code in &path/freq_fit_tmp.;
	        %let abort=1;
	        %let reason=Unable to run code in &path/freq_fit_tmp;
	        %goto DONE;
	    %end;
	%end;

	proc sort data = _kriblLosses_selected_ out =_kriblLosses_selected_2(compress=binary);
        /*by KRI_: dummy_bluid_:  threshold ;*/ 
		by &possiblyKRIs &possiblyDummyVars  threshold;
    run;

	/* Collapse observations with identical KRIs/dummy_bluid's/threshold into a single obs */
	data &outCompressedData (drop=adjexposure frequency numObservations);
        set _kriblLosses_selected_2 end=last;
		/* by KRI_: dummy_bluid_:  threshold;*/
		by &possiblyKRIs &possiblyDummyVars  threshold;
        if first.threshold=1 then do;
			newfrequency = 0;
			combinedexposure = 0;
		end;
        newfrequency+frequency;
		if missing(adjexposure) then put "ERROR: adjexposure is missing (&outCompressedData data step).";
		combinedexposure+adjexposure;
        if last.threshold=1 then do;
			offset = log(combinedexposure);
			output; 
			numObservations+1;
		end;
		if last then call symput("numObservations", numObservations);
    run;

	%if &numObservations < 100 %then %do;
		data &outCompressedData;
			set _kriblLosses_selected_2(rename=(frequency=newfrequency adjexposure=combinedexposure));
			offset = log(combinedexposure);
		run;
	%end;
%end;

	%else %do;
		%let numSelectedKRIs = 0;
		%let numSelectedBLs = 0;
		%let SelectedKRIs = ;
		data _kriblLosses_selected_;
			set _kriblLosses_exposure3;
			keep bluid date THRESHOLD frequency adjexposure &possiblyKRIs;
			if exposure=0 then delete;
			if missing(exposure) then put "ERROR: exposure is missing (_kriblLosses_selected data step, no KRIs & no BLs).";
			if missing(FSCALE) then put "ERROR: FSCALE is missing (_kriblLosses_selected data step, no fKRIs & no BLs).";
			adjexposure = exposure/FSCALE;
		run;
		%if &numKRIsPresent GT 0  %then %do;
			proc sort data = _kriblLosses_selected_ out =_kriblLosses_selected_2;
        		by KRI_: threshold ;
    		run;

			/* Collapse observations with identical KRIs/dummy_bluid's/threshold into a single obs */
		   	data &outCompressedData (drop=adjexposure frequency);
		        set _kriblLosses_selected_2 end=last;
		        by KRI_:  threshold;
		        if first.threshold=1 then do;
					newfrequency = 0;
					combinedexposure = 0;
				end;
		        newfrequency+frequency;
				if missing(adjexposure) then put "ERROR: adjexposure is missing (&outCompressedData data step, no fKRIs & no BLs).";
				combinedexposure+adjexposure;
		        if last.threshold=1 then do;
					offset = log(combinedexposure);
					output; 
					numObservations+1;
				end;
				if last then call symput("numObservations", numObservations);
		    run;
		    %if &numObservations < 100 %then %do;
				data &outCompressedData;
					set _kriblLosses_selected_2(rename=(frequency=newfrequency adjexposure=combinedexposure));
					offset = log(combinedexposure);
				run;
			%end;		    
		    
		%end;
		%else %do;   
			data &outCompressedData;
				set _kriblLosses_selected_(rename=(adjexposure=combinedexposure frequency=newfrequency));
				if missing(combinedexposure) then put "ERROR: adjexposure and combinedexposure are missing (&outCompressedData data step).";
				offset = log(combinedexposure);
			run;
		%end;
	%end;

	data &outMonthlyKRIs;  /* needed to compute meanWithoutPanel in orva_freq_fit_kri */
		set _kriblLosses_selected_; 
	run;	

	data &outParmStatus;
		numSelectedKRIs = &numSelectedKRIs;
		numSelectedBLs = &numSelectedBLs;
		SelectedKRIs = "&SelectedKRIs";
		GlobalStartDate = &global_start;
		GlobalEndDate = &global_end;
		numBLUIDsThisCell = &numBLUIDsThisCell;
		extraBL = &extraBL; /* the BLUID of the BL that did not get a dummy var named for him */
	run;


	/*  ******************************************************************  */


	%DONE:

    %if &abort %then %do;
        %put ERROR: orva_freq_var_select macro ended abnormally.;

        data &outParmStatus;
            format status 8. reason $100.;
            status=1;
            reason="&reason";
        run;
    %end;    

    /*- delete unwanted data sets -*/
  proc datasets library=work memtype=data nolist nodetails;
      delete  
          _work_ 
          _cellbls
		  _cellbls_all
          _trcellbls
          _kribl_curr
		  miss_exposure
		  freqscalei
          _firmKRIdata_
          _kribllosses_
		  _kriblLosses_exposure_
		  _kriblLosses_exposure2
		  _kriblLosses_exposure3
          _kribllosses_selected_
		  _kriblLosses_selected_2
          _regresults_
          ;
  run;
    quit;

%mend orva_freq_var_select;

%macro GetCCUIDList();
	%let CC_list=;
	data _null_;
		set &ccdata; 
		if CCUID = &CCUID; /* this is the subsetting if */
		call symput("CCVUID_1", CCVUID_1);
	run;
	%if &CCVUID_0 EQ 0 %then %do;
		%if &CCVUID_1=0 %then %do; /* total-total cell */
			data _bone_;  /* change this to _null_ later */
				set &ccdata;
				length tmp $20000; 
				retain tmp "";
				if CCVUID_0 NE 0;
				if CCVUID_1 NE 0;
				tmp = trim(tmp) || " " || left(ccuid);				
				call symput("CC_list", trim(tmp));
			run;
		%end;

		%else %do; /* a cell in the total row that's NOT the total-total cell  */
			data _bone_;
				set &ccdata;
				length tmp $20000; 
				retain tmp "";
				if CCVUID_0 NE 0;
				if CCVUID_1 = &ccvuid_1;
				tmp = trim(tmp) || " " || left(ccuid);				
				call symput("CC_list", trim(tmp));
			run;
		%end;
	%end;
	%else %if &CCVUID_1=0 %then %do; /* a cell in the total col that's NOT the total-total cell  */
		data _bone_;
			set &ccdata;
			length tmp $20000; 
			retain tmp "";
			if CCVUID_1 NE 0;
			if CCVUID_0 = &ccvuid_0;
			tmp = trim(tmp) || " " || left(ccuid);					
			call symput("CC_list", trim(tmp));				
		run;		
	%end;
	%else %do;
		%let CC_list = &CCUID;
	%end;

	
    %let CCUID_curr=%scan(&CC_list, 1, ' ');
	%let CC_commalist=&CCUID_curr;
	%let numCCUIDs=2;
	%let CCUID_curr=%scan(&CC_list, &numCCUIDs, ' ');
    %do %while (&CCUID_curr ne );
		%let CC_commalist = &CC_commalist , &CCUID_curr; 
        %let numCCUIDs = %eval(&numCCUIDs+1);
        %let CCUID_curr = %scan(&CC_list, &numCCUIDs, ' ');
    %end;
    %let numCCUIDs=%eval(&numCCUIDs-1);

	%put LAURAS CC_commalist is &CC_commalist;

%mend GetCCUIDList;

%macro SaveCurrValuesIntoPrevVariables();
			prev_date = date;
			prev_DaysInMonth = DaysInMonth;
			prev_FSCALE = FSCALE;
			prev_THRESHOLD = THRESHOLD;
%mend SaveCurrValuesIntoPrevVariables;

%macro SaveCurrValuesIntoNextVariables();
	next_date = date; 
	next_frequency = frequency; 
	next_DaysInMonth = DaysInMonth;
	next_exposure = exposure;
	next_FSCALE = FSCALE;
	next_THRESHOLD = THRESHOLD;
%mend SaveCurrValuesIntoNextVariables;

%macro ResetCurrValuesWithNextVariables();
	date = next_date;
	frequency = next_frequency;
	DaysInMonth = next_DaysInMonth;
	FSCALE = next_FSCALE; /* will be missing IFF exposure is 0 */
	THRESHOLD = next_THRESHOLD;
	exposure = next_exposure;
%mend ResetCurrValuesWithNextVariables;

%macro SetFSCALEtoCurrOrPrevOrOne();			
	/* Let's allow FSCALE for this newly created obs to be same as the already existing obs,
	unless it's missing... */
	if missing(FSCALE) then do;
		/* sets it to 1 if prev_FSCALE was also missing */
		if missing(prev_FSCALE) then FSCALE = 1;  
		else FSCALE = prev_FSCALE; 
	end;
%mend SetFSCALEtoCurrOrPrevOrOne;	
%macro SetTHRESHOLDtoCurrOrPrevOrMean();
	/* Let's allow THRESHOLD for this newly created obs to be same as the already existing obs,
	unless it's missing... */	
	if missing(THRESHOLD) then do;
		/* sets it to mean if prev_THRESHOLD was also missing */
		if missing(prev_THRESHOLD) then THRESHOLD = &meanThreshold;  
		else THRESHOLD = prev_THRESHOLD; 
	end;
%mend SetTHRESHOLDtoCurrOrPrevOrMean;
