/*-------------------------------------------------------------------
 * Copyright (C), 2005
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 * %orva_get_hill_estimate(data=, tailpnts=)
 *
 * Computes the tail index for a data set based on Hill's algorithm.
 
 * Input:
 *    data = loss data with loss value=AMOUNT
 *    tailpnt = subset of the tail numbers use in hill's method
 *
 *-------------------------------------------------------------------*/

%macro orva_get_hill_estimate(  data = ,
											useKRI =,
											tailpnts=100
                              );

	%if &useKRI = NO %then %do;
         %let AMOUNT_VAR= amount;
	%end;
	%else %do;
			%let AMOUNT_VAR= KRI_ADJ_AMOUNT;
	%end;
	
	/* driver program for bootstrapping to find optimal cutoff  */
	%let N1_L = 50;
	%let N1_U = 60;
	%let N1_INC = 10;
	%let BOOT_SIZE = 10;
    %let K_L = 5;
    %let K_U = 95;
	%let K_INC = 10;

	
	/* sort the data to get the last TAILPNTS ----------------*/
	proc sort data=&data out=_tmpData; 
		by &AMOUNT_VAR; 
	run;
	
	proc sql noprint;
	select count(*) into :NOBS
	from _tmpData;
	quit;


	data _work_; 
		set _tmpData;
		if _n_ > &nobs-&tailpnts then output;
	run;
	%if %sysevalf( &tailpnts < &nobs, boolean ) %then %do;
	%let NOBS=&tailpnts;
	%end;
	
	/* input data sorted on ascending order */
	%get_bootsamp(_work_, _samp1_, _samp2_, &AMOUNT_VAR, &NOBS, &N1_L, &N1_U, &N1_INC, &BOOT_SIZE);
	
	/* keep stats for every bootstrap sample */
	data _out1_;
		n=0;
		index=0;
		k=0;
		q=0;
	run;

	data _out2_;
		n=0;
		index=0;
		k=0;
		q=0;
	run;

	%do n1=&N1_L %to &N1_U %by &N1_INC;
		%let n2=%eval(&n1*&n1/&NOBS);
			data samp1;
				set _samp1_;
				if n=&n1;
			run;
			
			data samp2;
				set _samp2_;
				if n=&n2;
			run;
	
	
		%do nboot=1 %to &BOOT_SIZE;
			%let f1=%eval((&nboot-1)*&n1+1);
			%let l1=%eval(&f1+&n1-1);
			%let f2=%eval((&nboot-1)*&n2+1);
			%let l2=%eval(&f2+&n2-1);
		
			data temp1;
				set samp1(firstobs=&f1 obs=&l1);
			run;
		
			data temp2;
				set samp2(firstobs=&f2 obs=&l2);
			run;
		
			%do k= &K_L %to &K_U %by &K_INC;
				%compute_q(temp1, &nboot, &AMOUNT_VAR,_out1_, &n1, &k);
				%compute_q(temp2, &nboot, &AMOUNT_VAR, _out2_, &n2, &k);
			%end;
		%end;
	%end;

	/* delete dummy obs when created */
	data _out1_;
		set _out1_;
		if _N_=1 then delete;
	run;

	data _out2_;
		set _out2_;
		if _N_=1 then delete;
	run;

	/*- find Q -*/
	proc sort data=_out1_;
		by n k;
	run;

	proc summary data=_out1_;
		var q;
		by n k;
		output out=stat1 mean=Q;
	run;

	proc sort data=_out2_;
		by n k;
	run;

	proc summary data=_out2_ noprint;
		var q;
		by n k;
		output out=stat2 mean=Q;
	run;

	/* for an n1 or n2, find a k with min Q (min asymptotic mse) */
	proc sort data=stat1;
		by n q;
	run;

	data stat11;
		set stat1;
		by n;
		if first.n then output;
	run;

	proc sort data=stat2;
		by n q;
	run;

	data stat21;
		set stat2;
		by n;
		if first.n then output;
	run;


	data stat;
		set stat11(rename=(n=n1 k=k1 q=q1));
		set stat21(rename=(n=n2 k=k2 q=q2));
		R=q1*q1/q2;
	run;
	
	proc sort data=stat;
		by R;
	run;
	
	data _NULL_;
		set stat;
		if _N_ = 1;
		K_1 = k1*k1 / k2;
		K_2 = log(k1)*log(k1) / ( 2*log(n1) - log(k1) )**2 ;
		K_3 = (log(n1) - log(k1))/log(n1);
		K= K_1 * (K_2 ** K_3);  /* Tail index percentage */
		k = floor(k * &NOBS /100);
		call symput("k", k );    /* This is the tail index */
	run;

	/* Now find the observation with that index */
	data _null_; 
		set _work_;

		if _n_ = &k then do;
			call symput( "PARAM_xLimit", &AMOUNT_VAR);
		end;
	run;


	proc datasets library=work memtype=data nolist;
		delete stat stat1 stat2 stat21 stat11 _samp1_ _samp2_ samp1 samp2
		_out1_ _out2_ _work_;
	quit;

%mend;

/*- generate bootstrap samples -*/
%macro get_bootsamp(
  inputds,    /* input data set */
  outds1,     /* output data set */
  outds2,     /* output data set */
  var,        /* variable of interest */
  nobs,       /* number of observations in input data set */
  lsize,      /* min resample size */
  hsize,      /* max resample size */
  step,       /* step size of resample size */
  nboot       /* number of bootstrap samples */
  );

  /*- initialize the data sets -*/  
  data &outds1;n=0;&var=0;run;
  data &outds2;n=0;&var=0;run;

  /*- loop over the grid of n1's -*/  
  %do n1 = &lsize %to &hsize %by &step;
     %let n2=%eval((&n1*&n1)/&nobs);
     %do i = 1 %to &nboot;
        data temp1;
        retain seed 0;
        call ranuni(seed,p);
        p=int(p*&nobs)+1;
        s=&n1;
        set &inputds point=p nobs=s;
        n=&n1;
        boot=&i;
        j+1;
        if j > &n1 then stop;
        keep n &var;
        run;
        
        data temp2;
        retain seed 0;
        call ranuni(seed,p);
        p=int(p*&nobs)+1;
        s=&n2;
        set &inputds point=p nobs=s;
        n=&n2;
        boot=&i;
        j+1;
        if j>&n2 then stop;
        keep n &var;
        run;

        /*- append the sample -*/
        proc append base=&outds1 data=temp1;run;         
        proc append base=&outds2 data=temp2;run;

     %end;
   %end;

   data &outds1;set &outds1;if _n_=1 then delete;run;
   data &outds2;set &outds2;if _n_=1 then delete;run;

   /*- values need to be sorted -*/
   proc sort data=&outds1;by n &var;run;
   proc sort data=&outds2;by n &var;run;   
   
%mend get_bootsamp;



%macro compute_q(
    inputds,  /* input data set         */
    index,
    var,      /* variable for analysis  */
    outputds, /* output data set        */
    nobs,     /* number of observations */
    K,        /* cutoff K (%)           */
    );


  data _null_;
  call symput('CUTOBS', int(&nobs*&K/100));
  run;
  
	data _null_;
		set &inputds(firstobs=&CUTOBS obs=&CUTOBS);
		call symput('CUTOFF', &var);
	run;
  
	/*- get the data for Hill's estimate -*/
	data _texceed; 
		set &inputds(firstobs=&CUTOBS);
		x1 = log(&var)-log(&CUTOFF);
		x2= x1*x1;
	run;
 
	proc summary data=_texceed mean noprint;
		var x1 x2;
		output out=_hillout mean=m1 m2;
	run;

	data _hillout; 
		set _hillout;
		q=(m2-2*m1*m1)*(m2-2*m1*m1);
	run;

	data out;
		set  _hillout(keep=q);
		index=&index;
		K = &K;
		n=&nobs;  
	run;

  proc append base=&outputds data=out;
  run;

%mend compute_q;














     







