/*****************************************************************************************
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 * NAME:    ORVA_GetMomts.sas                                             
 * PURPOSE: Calculates frequency and severity moments and aggregations for all cells.           
 *          Calculates the aggregates, mean, and count for the Loss Effects Breakdown.
 * SUPPORT: sasdje
 *
 * PARAMETERS:
 *
 *   lossData      =  the name of the loss data with the following variables:
 *                    AMOUNT, FINANCIALDATE CCUID.
 *   Global_range  =  global range dataset for this project.
 *   interval      =  DATE interval (YEAR, MONTH ...) Default is YEAR
 *   outSevMomts   =  MEAN, VARIANCE, STD, SKEWNESS, KURTOSIS, COUNT, MINLOSS, MAXLOSS, CCV_n, SUM (for AMOUNT)
 *                    LEGAL_LIABILITY (aggregate), LEGAL_LIABILITY_MEAN, LEGAL_LIABILITY_N (count)
 *                    REGULATORY_ACTION (aggregate), REGULATORY_ACTION_MEAN, REGULATORY_ACTION_N (count)
 *                    RESTITUTION (aggregate), RESTITUTION_MEAN, RESTITUTION_N (count)
 *                    LOSS_TO_ASSETS (aggregate), LOSS_TO_ASSETS_MEAN, LOSS_TO_ASSETS_N (count)
 *                    LOSS_OF_RECOURSE (aggregate), LOSS_OF_RECOURSE_MEAN, LOSS_OF_RECOURSE_N (count)
 *                    WRITE_DOWNS (aggregate), WRITE_DOWNS_MEAN, WRITE_DOWNS_N (count)
 *   outFreqMomts  =  MEAN, VARIANCE, CCV_n
 *   outStatus     =  Success or failure - default is status
 *
 *****************************************************************************************/
 %macro ORVA_GetMomts( lossData       =,
                       global_range   =,
                       interval       =YEAR,
                       outSevMomts    =osstat,              
                       outFreqMomts   =ofstat,              
                       outStatus      =status
                     );     

        /*- create the status data set up front -*/
        %let status=0;
        %let reason=Successful;
        %let abort=0;
        data &outStatus;
                format status 8. reason $100.;
                status=1;
                reason="Unknown";
        run;
 
        /*- error checking -*/
        %if &lossData=%str() %then %do;
                %put ERROR: The parameter lossData= must be specified.;
                %let abort=1;
                %let reason=MISSING lossData= PARAMETER;
                %goto DONE;
        %end;
        
        /*- check for empty dataset -*/
        %let dsid=%sysfunc(open(&lossData,I));
        %let nLosses=%sysfunc(attrn(&dsid,nlobs));
        %let rc=%sysfunc(close(&dsid));
        %if &nLosses=0 %then %do;
                %put ERROR: No observations available in data.;
                %let abort=1;
                %let reason=No observations available in data;
                %goto DONE;
        %end;
               
        /*- make sure datasets are sorted -*/
        proc sort data=&lossData out=_sortedData; by CCUID FINANCIALDATE; run;

		 /* Get rid of dummy observations -------------*/
		%let ds_name=%scan(&lossdata,2,".");
        %if &ds_name =rollupi %then %do;
            data VARPRJ.lossi(compress=binary); set VARPRJ.lossi;
			    if amount = . then delete;
			run;
		%end;


         /* Adjust missing ranges --------------------*/
         /* combine the wgts from scaling with exposure size */
       data _freqcnts; set _sortedData;
          if exposure_size ^= . then do;
                 /* consortium and public data are missing */
              if fscale = 0 then exp_fscale = 0;
              else               exp_fscale = (1/fscale) * exposure_size;
          end; 
          else
              exp_fscale = fscale;
           
               /* Handle missing ranges ----------------*/
          if  exposure_size = 0 then fscale = 0;
          else fscale = 1;
          keep FINANCIALDATE CCUID fscale exp_fscale;
       run;

           /* Fscale now contains both numbers from scale screen and exposure_size*/
       data _sortedData; set _sortedData;
          if exposure_size ^= . and exposure_size > 0 then
              fscale = fscale * (1/exposure_size); 
          if exposure_size = 0 then fscale = 0;
          if amount=. then delete;
          drop exposure_size;
       run;

       
       data _null_; set &global_range;
	   call symput( "global_end", "'" || put(end_date, date9.) ||"'d");
	   call symput( "global_start", "'" || put(start_date, date9.) ||"'d");
        run;

        /*- First get FREQUENCY numbers -*/
        proc timeseries data=_freqcnts out=_freqcnts1;
          id FINANCIALDATE interval=&interval acc=total setmissing=0 start=&global_start end=&global_end;
          var fscale ;
          by CCUID;
        run;

        proc timeseries data=_freqcnts out=_freqcnts2;
          id FINANCIALDATE interval=&interval acc=total setmissing=missing start=&global_start end=&global_end;
          var  exp_fscale;
          by CCUID;
        run;

         
        /* average the exposure for each period ---------------*/
        data _freqcnts; merge _freqcnts1 _freqcnts2;
          by ccuid financialdate;          
          if exp_fscale = 0 then delete;
          if exp_fscale = . then exp_fscale = 1;
          if fscale = 0 then exp_fscale = 1;
          else
          if fscale ^= . then exp_fscale = exp_fscale / fscale;
          cnt_per_exp = fscale / exp_fscale;
        run;
       

        /* compute the average and the variance ----------*/
        proc summary data=_freqcnts vardef=weight;
          output out=_freqMoments1(drop=_type_ _freq_) mean =MEAN 
                 var=VARIANCE N=n;
          var cnt_per_exp;
          weight exp_fscale;
          by CCUID;
        run;
 
        proc summary data=_freqcnts;
          output out=_freqMoments2(drop=_type_ _freq_) mean=ave_exp;
          var exp_fscale;
          by CCUID;
        run;

        proc sort data=_FreqMoments1;by CCUID;
        run; 
        proc sort data=_FreqMoments2;by CCUID;
        run; 

            /*- clean up final datasets -*/
        data _FreqMoments; merge _FreqMoments1 _FreqMoments2;
         by CCUID; 
        run;   
   

        /* Adjust variance for exposure - Have  ---------------------*/
        /* var = ( sum[ cnt_per_exp **2 * (exp/sum exp)] - 
                 ( sum [cnt_per_exp *(exp/sum exp)]) **2)  ---*/
        /* Need Var' = var * ave_exp * n/(n-1)  --------------*/
        data _freqMoments;set _freqMoments;
           VARIANCE = VARIANCE * ave_exp * (n/(n-1));
           drop n ave_exp;
        run;

         /* Set zero values to missing so counts are correct ----------------------*/
        data _sortedData; set _sortedData; 
	   if LEGAL_LIABILITY=0 then do; LEGAL_LIABILITY=.; end;
	   if REGULATORY_ACTION=0 then do; REGULATORY_ACTION=.;  end;
	   if RESTITUTION=0 then do; RESTITUTION=.;  end;
	   if LOSS_TO_ASSETS=0 then do; LOSS_TO_ASSETS=.;  end;
	   if LOSS_OF_RECOURSE=0 then do; LOSS_OF_RECOURSE=.;  end;
	   if WRITE_DOWNS=0 then do; WRITE_DOWNS=.;  end;
	   if NON_INSURANCE_RECOVERIES=0 then NON_INSURANCE_RECOVERIES=.;  
	run;


        /*- now get the SEVERITY numbers -*/
        proc summary data=_sortedData;
          output out=_SevMoments1(drop=_type_ _freq_) mean=MEAN var=VARIANCE skew=SKEWNESS kurt=KURTOSIS n=COUNT min=MINLOSS max=MAXLOSS sum=SUM;
         
          var AMOUNT; 
          by CCUID;
        run;

        /* do this separately because different stats are needed for this group */
        proc summary data=_sortedData;
          
          output mean= LEGAL_LIABILITY_MEAN REGULATORY_ACTION_MEAN RESTITUTION_MEAN 
                       LOSS_TO_ASSETS_MEAN LOSS_OF_RECOURSE_MEAN WRITE_DOWNS_MEAN 
                       NON_INSURANCE_RECOVERIES_MEAN
                 sum= LEGAL_LIABILITY REGULATORY_ACTION RESTITUTION LOSS_TO_ASSETS 
                      LOSS_OF_RECOURSE WRITE_DOWNS NON_INSURANCE_RECOVERIES
	             n= LEGAL_LIABILITY_N REGULATORY_ACTION_N RESTITUTION_N 
                    LOSS_TO_ASSETS_N LOSS_OF_RECOURSE_N WRITE_DOWNS_N 
                    NON_INSURANCE_RECOVERIES_N
	          out=_SevMoments2(drop=_type_ _freq_);

          var LEGAL_LIABILITY REGULATORY_ACTION RESTITUTION LOSS_TO_ASSETS 
              LOSS_OF_RECOURSE WRITE_DOWNS NON_INSURANCE_RECOVERIES;
          by CCUID;
        run;
        proc sort data=_SevMoments1;by CCUID;
        run; 
        proc sort data=_SevMoments2;by CCUID;
        run; 

            /*- clean up final datasets -*/
        data &outSevMomts; merge _sevMoments1 _sevMoments2;
         by CCUID;
          STD = sqrt(VARIANCE);
          Kurtosis=Kurtosis+3;
          /*- remove double entry for total entries in ccData -*/
          if COUNT>0 then output;
        run;    
        
        data &outFreqMomts; set _freqMoments;
          /*- remove double entry for total entries in ccData -*/
          if MEAN ne . then output;
          run;
          
            /*- finalize status dataset -*/
%DONE:
    data &outStatus;
                format status 8. reason $100.;
                status=0;
                reason="";
    run;
    
        %if &abort %then %do;
                %put ERROR: orva_getmomts macro ended abnormally.;

                data &outstatus;
                        format status 8. reason $100.;
                        status=1;
                        reason="&reason";
                run;
        %end;  
        
    proc datasets library=work memtype=data nolist;
        delete _freqcnts _freqMoments _sevMoments1 _sevMoments2 _sortedData;
    run; quit;
%mend;

        
