 /*****************************************************************************************                                             
 *  Copyright (C), 2005                                                                                                                 
 *  SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.                                                                  
 *                                                                                                                                      
 *  NAME:    orva_kri_transpose.sas                                                                                                     
 *  PURPOSE: Denormalizes the KRI datasets by merging the Variables with the Values                                                     
 *           and transposing the result.  Also expands the interval on a monthly basis                                                  
 *           and forecasts KRI values up to the Global EndDate (plus 1 year).  This macro is                                            
 *           called during data versioning and is followed by a call to %orva_aggregate                                                 
 *           which fills in missing KRI values from parent/child values (if possible).                                                  
 *                                                                                                                                      
 *           If there is no KRI data, the final dataset simply contains                                                                 
 *           only the &startDate.                                                                                                       
 *                                           
 *           WARNING - this macro infers the data's INTERAL using the first 2 observations
 *                     in the dataset.  Thus all KRI's are assumed to have the same
 *                     INTERVAL.  This should be changed in the furture!
 *                                                                                            
 *  SUPPORT: stchri                                                                                                                     
 *                                                                                                                                      
 *  PARAMETERS:                                                                                                                         
 *    kriVals     = the name of the KRI Values dataset.                                                                                 
 *    kriVars     = the name of the KRI Variables dataset.                                                                              
 *    cpiData     = already expanded Internal CPI dataset to be used as special KRI.                                                    
 *    version     = the current version UID.                                                                                            
 *    bl          = true  - handling Business Line KRIs (default).                                                                      
 *                  false - handling Firm KRIs.                                                                                         
 *    startDate   = global start date of data.                                                                                          
 *    endDate     = global end date of data.                                                                                            
 *    kriOut      = name of the output denormalized KRI dataset with these variables:                                                   
 *                    BLUID_KRI - business line UID or KRI                                                                              
 *                    BL_LEVEL - business line level                                                                                    
 *                    RISK_FACTOR_VARIABLE_DATE - KRI Date (monthy interval)                                                            
 *                    UID_PARENT_<N> - BLUID of parents                                                                                 
 *                    any number of KRIs                                                                                                
 *    ostat       = name of the out status dataset.  Will contain the following:                                                        
 *                    status = 0 on success, 1 otherwise.                                                                               
 *                    reason = printable message.                                                                                       
 *                                                                                                                                      
 * NOTES: see sample code at end of file.                                                                                               
 *                                                                                                                                      
 *****************************************************************************************/                                             
%macro orva_kri_transpose(kriVals=  ,                                                                                                   
                          kriVars=  ,                                                                                                   
                          cpiData=  ,                                                                                       
                          version=  ,                                                                                                   
                          bl=       true,                                                                                               
                          startDate=,                                                                                                   
                          endDate=  ,                                                                                                   
                          kriOut=   ,                                                                                                   
                          ostat=    );                                                                                                  
                                                                                                                                        
   /*** create the status dataset up front ***/                                                                                         
   %let status=0;                                                                                                                       
   %let reason=Successful;                                                                                                              
   data &ostat;                                                                                                                         
        format status 8. reason $100.;                                                                                                  
        status=1;                                                                                                                       
        reason="Unknown";                                                                                                               
   run;                                                                                                                                 
                                                                                                                                        
   /***** increment end date by 1 year *****/                                                                                           
   data _NULL_;                                                                                                                         
        date = &endDate;                                                                                                                
      date = intnx('YEAR', date, 1);                                                                                                    
      call symput('endDate', date);                                                                                                     
   run;                                                                                                                                 
                                                                                                                                        
   /*********** Subset KRI data by version, merge, and sort ***********/                                                                
   %let nobs=0;                                                                                                                         
   %if "&kriVals" ne "" and "&kriVars" ne "" %then %do;                                                                                 
                                                                                                                                        
         /*** subset KRI values for current VERSION ***/                                                                                                      
         data _tmpKriVals; set &kriVals; where VERSION_UID=&version;                                                                    
            format RF_DATE monyy7.;                                                                                                     
            RF_DATE= RISK_FACTOR_VARIABLE_DATE;                                                                                         
            drop RISK_FACTOR_VARIABLE_DATE;                                                                                             
         run;                                                                                                                           
         proc sort data= _tmpKriVals; by RISK_FACTOR_VARIABLE_UID; run;                                                                 
                                                                                                                                        
         /*** subset KRI variables names for current VERSION ***/                                                                                             
         data _tmpKriVars; set &kriVars; where VERSION_UID=&version; run;                                                               
         proc sort data= _tmpKriVars; by RISK_FACTOR_VARIABLE_UID; run;                                                                 
                                                                                                                                        
         /*** Merge KRI Values with KRI Variables names ***/                                                                            
         data _tmpKriVal;                                                                                                               
            merge _tmpKriVals (keep= RISK_FACTOR_VARIABLE_UID                                                                           
                                  RF_DATE                                                                                               
                                  %if &bl eq true %then %do; INTERNAL_BL_UID %end;                                                      
                                  RISK_FACTOR_VALUE)                                                                                    
                  _tmpKriVars (keep= RISK_FACTOR_VARIABLE_UID                                                                           
                                  RISK_FACTOR_VARIABLE_NAME);                                                                           
            by RISK_FACTOR_VARIABLE_UID;                                                                                                
              if RF_DATE eq . then delete;                                                                                              
         run;                                                                                                                           
                                                                                                                                        
         /*** Do we have any KRI Data? ***/                                                                                             
         %let dsid=%sysfunc(open(_tmpKriVal,I));                                                                                        
         %if &dsid ^= 0 %then %do;                                                                                                      
            %let nobs=%sysfunc(attrn(&dsid,nlobs));                                                                                     
            %let rc=%sysfunc(close(&dsid));                                                                                             
         %end;                                                                                                                          
   %end;                                                                                                                                
                                                                                                                                        
   /************************* munge KRI data ************************/                                                                  
   %let NUM_KRI_VARS= 0;                                                                                                                
   %if &nobs gt 0 %then %do;                                                                                                            
                                                                                                                                        
         /*** create NAMES and LABELS macros for use later ***/                                                                               
         options sortdup=logical;                                                                                                       
         proc sort data= _tmpKriVal(keep=RISK_FACTOR_VARIABLE_UID RISK_FACTOR_VARIABLE_NAME)                                            
                   out= _tmpKriUnique nodup;                                                                                            
            by RISK_FACTOR_VARIABLE_UID;                                                                                                
         run;                                                                                                                           
         data _tmpKriUnique; set _tmpKriUnique end=LAST;                                                                                
            retain names; length names $ 1024;                                                                                          
            retain labels; length labels $ 1024;                                                                                        
            labels = trim(left(labels)) || '$' || trim(left(RISK_FACTOR_VARIABLE_NAME));                                                
            RISK_FACTOR_VARIABLE_NAME = compress("KRI_" || RISK_FACTOR_VARIABLE_UID);                                                   
            names = trim(left(names)) || '$' || trim(left(RISK_FACTOR_VARIABLE_NAME));                                                  
            if LAST then call symput( 'NAMES', names );                                                                                 
            if LAST then call symput( 'LABELS', labels );                                                                               
          if LAST then call symput( 'NUM_KRI_VARS', _N_ );                                                                              
         run;                                                                                                                             
                                                                                                                                        
         /*** rename KRI variables as KRI_<risk_factor_variable_uid> ***/                                                                                                 
         data _tmpKriVal; set _tmpKriVal;                                                                                               
              RISK_FACTOR_VARIABLE_NAME = compress("KRI_" || RISK_FACTOR_VARIABLE_UID);                                                 
              drop RISK_FACTOR_VARIABLE_UID;                                                                                            
         run;                                                                                                                           
                                                                                                                                        
         /*** transpose KRI Values ***/                                                                                                 
         proc sort data=_tmpKriVal nodups;                                                                                              
             by %if &bl eq true %then %do;                                                                                              
                   INTERNAL_BL_UID                                                                                                      
                %end;                                                                                                                   
                RF_DATE;                                                                                                                
         run;                                                                                                                           
         proc transpose data=_tmpKriVal out=_tmpKriTrans;                                                                               
              by %if &bl eq true %then %do;                                                                                             
                 INTERNAL_BL_UID                                                                                                        
               %end;                                                                                                                    
               RF_DATE;                                                                                                                 
              id RISK_FACTOR_VARIABLE_NAME;                                                                                             
              var RISK_FACTOR_VALUE;                                                                                                    
         run;                                                                                                                           
         %if &SYSERR^=0 & &SYSERR ^= 4 %then %do;                                                                                       
                %put ERROR: Unable to transpose KRI Values;                                                                             
                  %let abort=1;                                                                                                         
                  %let reason=Unable to transpose KRI Values;                                                                           
                %goto DONE;                                                                                                             
         %end;     
                                                                                                                              
         /**************** rename DATE and find First KRI Date *********************/ 
         data _tmpKriTrans; set _tmpKriTrans;                                                                                               
             rename RF_DATE= RISK_FACTOR_VARIABLE_DATE;                                                                                 
             drop _NAME_;                                                                                                               
         run;                                        
         proc sort data= _tmpKriTrans; by RISK_FACTOR_VARIABLE_DATE; run;
         data _NULL_; set _tmpKriTrans; 
             if _N_ eq 1 then do;
			   format startdate date.;
			   startdate = &StartDate;
			   if RISK_FACTOR_VARIABLE_DATE lt startdate 
                  then startdate= RISK_FACTOR_VARIABLE_DATE;
               startdate = INTNX('MONTHLY', startdate, 0);
               call symput("FirstKRIDate", startdate);
             end;
         run;                                                                     
                                                                                                                                        
         /***************** get LAST_DATE and INTERVAL from the data ******************/   
         /**
          ** Also makes sure there is an entry for each KRI at FirstKRIDate.
          ** 
          ** Note that the INTERVAL is taken from the first 2 rows, and NOT
          ** on a KRI by KRI basis.  This requires that all KRIs use the same 
          ** interval!  Should be fixed at next version!
          **/                                                    
         %let LAST_DATE = &FirstKRIDate;                                                                                                   
         %let INTERVAL= YEAR;                                                                                                           
         %if &bl eq true %then %do;                                                                                                     
               proc sort data= _tmpKriTrans; by INTERNAL_BL_UID RISK_FACTOR_VARIABLE_DATE; run;                                       
               data _tmpKri; set _tmpKriTrans; by INTERNAL_BL_UID;                                                                    
                   drop saveDate prevDate interval;                                                                                      
                                                                                                                                        
                   /*- get interval based of first 2 rows -*/                                                                            
                   retain prevDate 0;                                                                                                  
                   format interval $8.;                                                                                                
                   retain interval 'YEAR';                                                                                             
                   if prevDate ne 0 then do;                                                                                           
                      if intck('YEAR', prevDate, RISK_FACTOR_VARIABLE_DATE) eq 0 then do;                                               
                         if  intck('QTR', prevDate, RISK_FACTOR_VARIABLE_DATE) eq 0 then do;                                            
                            interval = 'MONTH';                                                                                         
                         end;                                                                                                           
                         else if interval ne 'MONTH' then do;                                                                           
                            interval = 'QTR';                                                                                           
                         end;                                                                                                           
                     end;                                                                                                              
                     call symput('INTERVAL', interval);                                                                                
                   end;                                                                                                                  
                   prevDate = RISK_FACTOR_VARIABLE_DATE;                                                                               
                                                                                                                                        
                   /*- add &FirstKRIDate to each Businessline if not there -*/                                                            
                   if first.INTERNAL_BL_UID and RISK_FACTOR_VARIABLE_DATE > &FirstKRIDate then do;                                          
                     saveDate = RISK_FACTOR_VARIABLE_DATE;                                                                              
                     RISK_FACTOR_VARIABLE_DATE = &FirstKRIDate;                                                                            
                     output;                                                                                                            
                     RISK_FACTOR_VARIABLE_DATE = saveDate;                                                                              
                   end;                                                                                                                  
                   output;                                                                                                               
                   if last.INTERNAL_BL_UID and RISK_FACTOR_VARIABLE_DATE gt symget("LAST_DATE") then do;                                 
                       call symput("LAST_DATE", RISK_FACTOR_VARIABLE_DATE);                                                               
                   end;                                                                                                                  
               run;                                                                                                                     
         %end;                                                                                                                          
         %else %do;                                                                                                                     
               proc sort data= _tmpKriTrans; by RISK_FACTOR_VARIABLE_DATE; run;                                                         
               data _tmpKri; set _tmpKriTrans end=last;                                                                                 
                  drop saveDate prevDate interval;                                                                                      
                                                                                                                                        
                    /*- get interval based of first 2 rows -*/                                                                            
                    retain prevDate 0;                                                                                                  
                    format interval $8.;                                                                                                
                    retain interval 'YEAR';                                                                                             
                    if prevDate ne 0 then do;                                                                                           
                      if intck('YEAR', prevDate, RISK_FACTOR_VARIABLE_DATE) eq 0 then do;                                               
                         if  intck('QTR', prevDate, RISK_FACTOR_VARIABLE_DATE) eq 0 then do;                                            
                            interval = 'MONTH';                                                                                         
                         end;                                                                                                           
                         else if interval ne 'MONTH' then do;                                                                           
                            interval = 'QTR';                                                                                           
                         end;                                                                                                           
                      end;                                                                                                              
                      call symput('INTERVAL', interval);                                                                                
                  end;                                                                                                                  
                    prevDate = RISK_FACTOR_VARIABLE_DATE;                                                                               
                                                                                                                                        
                  /*- add &FirstKRIDate to each Businessline if not there -*/                                                            
                  if _N_ eq 1 and RISK_FACTOR_VARIABLE_DATE > &FirstKRIDate then do;                                                       
                     saveDate = RISK_FACTOR_VARIABLE_DATE;                                                                              
                     RISK_FACTOR_VARIABLE_DATE = &FirstKRIDate;                                                                            
                     output;                                                                                                            
                     RISK_FACTOR_VARIABLE_DATE = saveDate;                                                                              
                  end;                                                                                                                  
                  output;                                                                                                               
                  if last then do;                                                                                                      
                     call symput("LAST_DATE", RISK_FACTOR_VARIABLE_DATE);                                                               
                  end;                                                                                                                  
               run;                                                                                                                     
         %end;                                                                                                                          
                                                                                                                                        
         /***************** forecast until the Global EndDate ******************/                                                         
         %let NUM_LEADS= 0;                                                                                                             
         data _NULL_;                                                                                                                   
           if &LAST_DATE ge &endDate then numleads = 0;                                                                                 
           else numleads = intck("&INTERVAL",&LAST_DATE,&endDate);                                                                      
           call symput("NUM_LEADS", numleads);                                                                                          
         run;                                                                                                                           
                                                                                                                                    
         %if &NUM_LEADS ne 0 %then %do;                                                                                                 
               proc forecast data=_tmpKri out=_tmpKriPred                                                                               
                             interval=&INTERVAL lead=&NUM_LEADS                                                                         
                             method= EXPO trend= 3;                                                                                     
                  id RISK_FACTOR_VARIABLE_DATE;                                                                                         
                    %if &bl eq true %then %do; by INTERNAL_BL_UID; %end;                                                                
               run;                                                                                                                     
                                                                                                                                           
               /*** merge forecasted data back in ***/                                                                                  
               proc sort data=_tmpKri;                                                                                                  
                 by %if &bl eq true %then %do; INTERNAL_BL_UID %end;                                                                     
                    RISK_FACTOR_VARIABLE_DATE;                                                                                           
               run;                                                                                                                       
               proc sort data=_tmpKriPred;                                                                                              
                by %if &bl eq true %then %do; INTERNAL_BL_UID %end;                                                                     
                   RISK_FACTOR_VARIABLE_DATE ;                                                                                          
             run;                                                                                                                       
               data _tmpkrimerge; merge _tmpKri _tmpKriPred;                                                                            
                  by %if &bl eq true %then %do; INTERNAL_BL_UID %end;                                                                   
                   RISK_FACTOR_VARIABLE_DATE;                                                                                           
                  array pred[&NUM_KRI_VARS];                                                                                            
                    array kri[&NUM_KRI_VARS] KRI_:;                                                                                     
                    do i=1 to &NUM_KRI_VARS;                                                                                            
                       if kri[i] eq . then kri[i] = pred[i];                                                                            
                    end;                                                                                                                
                    drop _TYPE_ _LEAD_ pred: i;                                                                                         
             run;                                                                                                                       
               data _tmpKri; set _tmpkrimerge; run;                                                                                     
                                                                                                                                           
         %end; /* we had something to forecast */                                                                                       
                                                                                                                                        
                                                                                                                                        
         /************* Expand Intervals to MONTH *************/                                                                        
         %if &INTERVAL ne MONTH %then %do;                                                                                              
               %if &bl eq true %then %do;                                                                                               
                       proc expand data= _tmpKri out= _tmpKriExpand                                                                     
                           from=&INTERVAL to= MONTH observed= beginning method= step;                                                   
                          by INTERNAL_BL_UID;                                                                                           
                          id RISK_FACTOR_VARIABLE_DATE;                                                                                 
                     run;                                                                                                               
               %end;                                                                                                                    
               %else %do;                                                                                                               
                     proc expand data= _tmpKri out= _tmpKriExpand                                                                       
                           from=&INTERVAL to= MONTH observed= beginning method= step;                                                   
                          id RISK_FACTOR_VARIABLE_DATE;                                                                                 
                     run;                                                                                                               
               %end;                                                                                                                    
               data _tmpKri; set _tmpKriExpand; run;                                                                                    
        %end;                                                                                                                            
                                                                                                                                        
                                                                                                                                        
         /*** Add labels to KRI columns ***/                                                                                            
         data &kriOut; set _tmpKri;                                                                                                     
             %if &bl eq true %then %do;                                                                                                 
                  rename INTERNAL_BL_UID=BLUID_KRI;                                                                                     
               %end;                                                                                                                    
                  %let num=1;                                                                                                           
               %let name=%scan(&NAMES, &num, '$');                                                                                      
               %let label=%scan(&LABELS, &num, '$');                                                                                    
               %do %while (&name ne );                                                                                                  
                   label &name = "&label";                                                                                              
                 %let num=%eval(&num+1);                                                                                                
                 %let name=%scan(&NAMES, &num, '$');                                                                                    
                 %let label=%scan(&LABELS, &num, '$');                                                                                  
               %end;                                                                                                                    
         run;                                                                                                                           
                                                                                                                                        
   %end; /* we had KRI data to munge */                                                                                                 
                                                                                                                                        
                                                                                                                                        
   /********************** Handle CPI **************************/                                                                       
                                                                                                                                        
      /*** Do we have CPI KRI Data? ***/                                                                                                
      %let cpinobs= 0;                                                                                                                  
      %if "&cpiData" ne "" %then %do;                                                                                                   
         %let dsid=%sysfunc(open(&cpiData,I));                                                                                          
         %if &dsid ^= 0 %then %do;                                                                                                      
            %let cpinobs=%sysfunc(attrn(&dsid,nlobs));                                                                                  
            %let rc=%sysfunc(close(&dsid));                                                                                             
         %end;                                                                                                                          
      %end;                                                                                                                             
                                                                                                                                        
      /*** Merge CPI data into firm data only ***/                                                                                      
    %if &bl ne true and &cpinobs ne 0 %then %do;                                                                                        
         data _kriCPI;        
           set &cpiData;     
           format RISK_FACTOR_VARIABLE_DATE DATE.;
           RISK_FACTOR_VARIABLE_DATE = CPI_DATE;                                                                                        
           KRI_0 = log(CPI_VALUE);                                                                                                      
             label KRI_0 = 'LogCPI';                                                                                                    
           keep RISK_FACTOR_VARIABLE_DATE KRI_0;                                                                                        
         run;                                                                                                                           
                                                                                                                                        
        /*** Merge into KRI data ***/                                                                                                   
       %if &nobs gt 0 %then %do;                                                                                                         
           proc sort data=_kriCPI; by RISK_FACTOR_VARIABLE_DATE; run;                                                                   
           proc sort data=&kriOut; by RISK_FACTOR_VARIABLE_DATE; run;                                                                   
           data &kriOut;                                                                                                                  
                merge &kriOut _kriCPI;                                                                                                     
                by RISK_FACTOR_VARIABLE_DATE;                                                                                           
           run;                                                                                                                           
        %end;                                                                                                                           
        %else %do;                                                                                                                      
           data &kriOut; set _kriCPI; run;                                                                                              
        %end;                                                                                                                           
   %end;                                                                                                                                
                                                                                                                                        
   /*** Do we have KRI Data? ***/                                                                                                       
   %let dsid=%sysfunc(open(&kriOut,I));                                                                                                 
   %if &dsid ^= 0 %then %do;                                                                                                            
      %let nobs=%sysfunc(attrn(&dsid,nlobs));                                                                                           
      %let rc=%sysfunc(close(&dsid));                                                                                                   
   %end;                                                                                                                                
   %if &nobs eq 0 %then %do;                                                                                                            
       %put NOTE: No KRI variables defined.;                                                                                            
       data &kriOut;                                                                                                                    
          format RISK_FACTOR_VARIABLE_DATE date.;                                                                                     
          RISK_FACTOR_VARIABLE_DATE = &startDate;                                                                                       
       run;                                                                                                                             
   %end;                                                                                                                                
                                                                                                                                        
%DONE:                                                                                                                                  
                                                                                                                                        
      data &ostat;                                                                                                                      
          format status 8. reason $100.;                                                                                                
          status=&status;                                                                                                               
          reason="&reason";                                                                                                             
      run;                                                                                                                              
                                                                                                                                        
   /*- cleaup temporary datasets -*/                                                                                                    
   proc datasets library=work memtype=data nolist;                                                                                      
            delete _tmpKri: _kriCPI;                                                                                                    
   run; quit;                                                                                                                           
                                                                                                                                        
%mend orva_kri_transpose;                                                                                                               
                                                                                                                                        
/**************************************                                                                                                 
data _NULL_;                                                                                                                            
   format startDate DATE.;                                                                                                              
   format endDate DATE.;                                                                                                                
   startDate ="10dec1998"d;                                                                                                             
   endDate = "20Jan2007"d;                                                                                                              
   call symput("StartDate", startDate);                                                                                                 
   call symput("EndDate", endDate);                                                                                                     
   put startDate=;                                                                                                                      
   put endDate=;                                                                                                                        
run;                                                                                                                                    
options noMPRINT noSYMBOLGEN;                                                                                                               
%orva_kri_transpose(kriVals= VARORG.risk_factor_value_firm,                                                                             
                    kriVars= VARORG.risk_factor_variable,                                                                               
                    version= 1004,                                                                                                      
                    cpiData= VARMETA._1004_cpii,                                                                                        
                    bl= false,                                                                                                          
                    startDate= &StartDate,                                                                                              
                    endDate= &EndDate,                                                                                                  
                    kriOut= krifirm_1004,                                                                                               
                    ostat= ostat);                                                                                                      
                                                                                                                                        
%orva_kri_transpose(kriVals= VARORG.risk_factor_value_bl,                                                                               
                    kriVars= VARORG.risk_factor_variable,                                                                               
                    version= 1004,                                                                                                      
                    bl= true,                                                                                                           
                    startDate= &StartDate,                                                                                              
                    endDate= &EndDate,                                                                                                  
                    kriOut= kriBl_1004,                                                                                                 
                    ostat= ostat);                                                                                                      
                                                                                                                                        
****************************************/                                                                                               