/******************************************************************************
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *   Title: orvd_GAMMA.sas
 *   
 *   -Provides support for the GAMMA distribution-
 *
 *	This file contains macro fragments to implement calculation specific to the 
 *	GAMMA distribution
 *
 *  The macro %ORVD_GAMMA is at the end of this file. It is an empty macro that takes
 *  no parameter and returns no feedback. In normal usage, this macro needs to be 
 *  called before trying to call any other macros depending on the fragments in this file.
 *
 *  Calling the %ORVD_GAMMA macro makes the SAS system parse and compile the whole file 
 *  so that the other macro fragment then become defined and available for other 
 *  macros to call.
 *
 *  Of course this is only guaranteed to work if there is no syntax error in any of 
 *  the fragments. 
 *
 *  The list of other macro files that rely on fragments in this file:
 *
 *   File                    Fragments it depends on (Approx Line # where fragement is called)
 *   orva_cdfplot.sas        CdfGAMMA(170)
 *   orva_gfit.sas           CdfGAMMA(188)
 *   orva_histplot.sas       CdfGAMMA(176), PdfGAMMA(177)
 *   orva_los_sim.sas        SdfGAMMA(263), SimGAMMA(408)
 *   orva_probplot.sas       CdfGAMMA(160)
 *   orva_quantplot.sas         QuantGAMMA(118)
 *   orva_sfitall.sas        FitGAMMA(304), MakeLLGoodGAMMA(320), NoMissingMOMGAMMA(379)
 *   orva_sfitall_model.sas  ModelFitGAMMA(287), MomentsGAMMA(391)
 *   orva_sfitall_mom.sas    MomFitGAMMA(78)
 *
 ******************************************************************************/

/*****************************************************************************************
 *   Needed for: %orva_cdfplot     ~line 170
 *               %orva_gfit        ~line 188
 *               %orva_histplot    ~line 176
 *               %orva_probplot    ~line 160
 *
 *   What it does  :  Evaluate the cumulative distribution function (CDF)
 *
 *   Parameters    :
 *              x     = value at which the CDF is evaluated
 *        returnVal = name of variable inton which result is stored
 *
 *****************************************************************************************/
%macro CdfGAMMA(x, returnVal);
    &returnVal = probgam(&x/&param_1,&param_2);
%mend CdfGAMMA;

/*****************************************************************************************
 *   Needed for: %orva_loss_sim    ~line 263
 *
 *   What it does  :  Evaluate the survival distribution function (SDF)
 *
 *   Parameters    :
 *              x     = value at which the SDF is evaluated
 *        returnVal = name of variable inton which result is stored
 *
 *****************************************************************************************/
%macro SdfGAMMA(x, returnVal);
    &returnVal = sdf("GAMMA",&x,&param_2,&param_1);
%mend SdfGAMMA;

/*****************************************************************************************
 *   Needed for: %orva_histplot    ~line 176
 *
 *   What it does  :  Evaluate the density distribution function (CDF)
 *
 *   Parameters    :
 *              x     = value at which the density is evaluated
 *        returnVal = name of variable inton which result is stored
 *
 *****************************************************************************************/
%macro PdfGAMMA(x, returnVal);
    &returnVal = pdf("GAMMA",&x,&param_2,&param_1);
%mend PdfGAMMA;


/*****************************************************************************************
 *   Needed for: %orva_quantplot    ~line 118
 *
 *   What it does  :  Generates a dataset named _temp_ containing probability values
 *                    and their corresponding quantiles
 *
 *   No Parameters    
 *
 *****************************************************************************************/
%macro QuantGAMMA;
	data _temp_;
		set _temp_ nobs=nobs end=eof;
		retain truncdf;

		if _N_=1 then do;
			if THRESHOLD = 0 then do;
				truncdf = 0;
			end;
			else do;
			%CDFGAMMA(THRESHOLD,truncdf);
		end;
		end;

		cdf=ecdf*(1-truncdf)+truncdf;
		quant=gaminv(cdf,&param_2)*&param_1;;
	run;
%mend QuantGAMMA;

/*****************************************************************************************
 *   Needed for: %orva_loss_sim    ~line 408
 *
 *   What it does  :  Simulates a random value
 *
 *   No Parameters    :  
 *
 *****************************************************************************************/
%macro SimGAMMA(loss);
    &loss = rand("GAMMA", Param_2) * Param_1;
%mend SimGAMMA;

/*****************************************************************************************
 *   Needed for: %orva_sfitall    ~line 304
 *
 *   What it does  :  Actually does the minimization using PROC NLP
 *
 *   No Parameters    
 *
 *****************************************************************************************/
%macro InitFitGAMMA;
    %let nparms=2;
    
    /*- initial guess based on moments -*/
    data _null_;
        param_1   = &sample_variance/&sample_mean;
        param_2  = &sample_mean/param_1;

    	call symput("param_1_init",param_1);
        call symput("param_2_init",param_2);
    run;
	
	%put ******************************;
	%put Distribution   =  &dist;
	%put Observations   =  &obsCount;
	%put Param_1_init    =  &Param_1_init;
	%put Param_2_init    =  &Param_2_init;
	%put ******************************;
%mend InitFitGAMMA;

/*****************************************************************************************
 *   Needed for: %orva_sfitall    ~line 304
 *
 *   What it does  :  Actually does the minimization using PROC NLP
 *
 *   No Parameters    
 *
 *****************************************************************************************/
%macro FitGAMMA;
	ods listing close;
    ods output 
		IterStop           =  _logl_
		ConvergenceStatus  =  _cs_ 
		covmat             =  _covmat_ 
		;	

    proc nlp 
		data   =  _workdata_ 
		cov    =  4 pcov 
		maxit  =  10000 
		maxfu  =  10000 
		outest =  _parmnlp_ 
		tech   =  qn
		;

        parms 
			Param_1 = &Param_1_init, 
			Param_2 = &Param_2_init
			;

        bounds 
			Param_2>0, 
			Param_1>0
			;

        max logf;
        
		if &THRESHOLDPERCENT >= 0 then do;
			lsdf = - &THRESHOLDPERCENT/(1-&THRESHOLDPERCENT)*
						log(probgam(THRESHOLD/param_1,param_2));     
		end; 
		else if THRESHOLD=0 then do;
			lsdf=0;
		end;
		else do;
			lsdf = logsdf("GAMMA",THRESHOLD, Param_2, Param_1);
		end;

        logf = (Param_2 - 1)*log(AMOUNT)-AMOUNT/Param_1
				-Param_2*log(Param_1)-LGamma(Param_2)-lsdf;
    run;
    quit;

	ods listing;
%mend FitGAMMA;

/*****************************************************************************************
 *   Needed for: %orva_sfitall    ~line 304
 *
 *   What it does  :  Actually does the minimization using PROC NLP
 *
 *   No Parameters    
 *
 *****************************************************************************************/
%macro FitKriGAMMA;
	ods listing close;
    ods output 
		IterStop           =  _logl_
		ConvergenceStatus  =  _cs_ 
		covmat             =  _covmat_ 
		;	

    proc nlp 
		data   =  _LogWorkdata_KRI_ 
		cov    =  4 pcov 
		maxit  =  10000 
		maxfu  =  10000 
		outest =  _parmnlp_ 
		tech   =  qn
		;

        parms 
			Param_1 = &Param_1_init, 
			Param_2 = &Param_2_init
			;

        bounds 
			Param_2>0, 
			Param_1>0
			;

        max logf;
                if &THRESHOLDPERCENT >= 0 then do;
		    lsdf = - &THRESHOLDPERCENT/(1-&THRESHOLDPERCENT)*
                                log(probgam(THRESHOLD/param_1/KRI_SCALE,param_2));     
                   
                end; 
                else
		if THRESHOLD=0 then do;
			lsdf=0;
		end;
		else do;
			lsdf = logsdf("GAMMA",THRESHOLD, Param_2, Param_1 * KRI_SCALE);
		end;

        logf = (Param_2 - 1)*LogAMOUNT - AMOUNT / Param_1 / KRI_SCALE
				-Param_2*( log(Param_1) + KRI_LOG ) - LGamma(Param_2)-lsdf;

	run;
    quit;

	ods listing;
%mend FitKriGAMMA;

/*****************************************************************************************
 *   Needed for: %orva_sfitall    ~line 320
 *
 *   What it does  :  Modify loglikelihood for cases when we had to take the log of the 
 *					  observations first. Here, it does nothing.
 *
 *   NO Parameters    
 *
 *****************************************************************************************/
%macro MakeLLGoodGAMMA;
    %let sumlog=0;
%mend MakeLLGoodGAMMA;

%macro MakeLLGoodKriGAMMA;
    %let sumlog=0;
%mend MakeLLGoodKriGAMMA;

/*****************************************************************************************
 *   Needed for: %orva_sfitall    ~line 379
 *
 *   What it does  :  Checks whether some moments are missing.
 *					  Here does nothing because all moments exist for Gamma distribution
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro NoMissingMOMGAMMA(parmdata=);

%mend NoMissingMOMGAMMA;

/*****************************************************************************************
 *   Needed for: orva_sfitall_model.sas
 *								%FindMoments ~line 392
 *
 *   What it does  :  Calculates the moments
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro MomentsGAMMA;
    mean     = Param_1 * Param_2;
	std      = Param_1 * sqrt(Param_2);
    skewness = 2/sqrt(Param_2);
    kurtosis = 3 + 6/Param_2;
%mend MomentsGAMMA;

%macro RealMomentsGAMMA;
    %MomentsGAMMA
%mend RealMomentsGAMMA;


/*****************************************************************************************
 *   Needed for: %orva_sfitall_mom.sas      ~line 78
 *
 *   What it does  :  Does method of moments estimation and assigns the corresponding 
 *                    values to the location, param_1, param_2, and shape2 macros
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro MomFitGAMMA;
    data _null_;
		Param_2 = &mean*&mean/&variance;
		Param_1 = &variance/&mean;
		call symput("Param_1", Param_1);
		call symput("Param_2", Param_2);
	run;
%mend MomFitGAMMA;

/*****************************************************************************************
 *   Needed for: %orvr_sfitstats.sas      ~line 78
 *
 *   What it does  :  Displays the estimated parameters
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro ParamsGAMMA;
	data foo (keep=Variable Estimate StdErr TValue probt);
		format Variable $12.;
		set &sparm;
		Variable="Scale";
		Estimate=param_1;
		StdErr=sqrt(cov_1);
		TValue=estimate/StdErr;
		probt=(1-probt(abs(tvalue),obscount -&KRI_ADJUST-&Param_count))*2;
		output;
		Variable="Shape1";
		Estimate=param_2;
		StdErr=sqrt(cov_3);
		TValue=estimate/StdErr;
		probt=(1-probt(abs(tvalue),obscount -&KRI_ADJUST-&Param_count))*2;
		output;
	run;

	data _NULL_;set foo;
       file print ods=(template='Stat.Reg.ParameterEstimates');
       put _ods_;
	run;
%mend ParamsGAMMA;

/*****************************************************************************************
 *   Needed for: %orvr_sfitstats.sas      ~line 78
 *
 *   What it does  :  Displays the estimated parameters
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro CovmatGAMMA;
	data foo (keep=_name_ param_1 param_2);
		format _name_ $12.;
		set &sparm;
		_name_="Scale";
		param_1=cov_1;
		param_2=cov_2;
		output;
		_name_="Shape1";
		param_1=cov_2;
		param_2=cov_3;
		output;
    run;

	proc print data=foo noobs label;
		var _name_ param_1 param_2 ;
		label _name_='00'x param_1="Scale" param_2="Shape1" ;
	run;
%mend CovmatGAMMA;


/*****************************************************************************************
 *   What it does  :  Calculates the loglikelihood contribution for each observation
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro LLGAMMA(var,retVal);
		if THRESHOLD=0 then do;
			lsdf=0;
		end;
		else do;
			lsdf = logsdf("GAMMA",THRESHOLD,param_2,param_1);
		end;

        &retVal = logpdf("GAMMA",AMOUNT,param_2,param_1)-lsdf;
%mend LLGAMMA;

/*****************************************************************************************
 *   Needed to be called before anything else is available
 *****************************************************************************************/
%macro ORVD_GAMMA;
    
    %let EXTRA_PARAMS= ;
    %let NEEDED_INPUT= ;
    %let SCALE_TRANSFORM= ID;
    %let DIST_NAME= GAMMA;
    %let PARAM_COUNT= 2;
 
 	data GAMMA_STATUS;
		status          = 0;
	run;
	
%mend ORVD_GAMMA;
