/******************************************************************************
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *   Title: orvd_LOGNGPD.sas
 *   
 *   -Provides support for the mixture of a LogNormal distribution
 *   -to represent the body and a Generalized Pareto to represent the tail
 *
 *	This file contains macro fragments to implement calculation specific to the 
 *	LogNormal distribution
 *
 *  The macro %ORVD_LOGNGPD is at the end of this file. It is an empty macro that takes
 *  no parameter and returns no feedback. In normal usage, this macro needs to be 
 *  called before trying to call any other macros depending on the fragments in this file.
 *
 *  Calling the %ORVD_LOGN macro makes the SAS system parse and compile the whole file 
 *  so that the other macro fragment then become defined and available for other 
 *  macros to call.
 *
 *  Of course this is only guaranteed to work if there is no syntax error in any of 
 *  the fragments. 
 *
 *  The list of other macro files that rely on fragments in this file:
 *
 *   File                    Fragments it depends on (Approx Line # where fragement is called)
 *   orva_cdfplot.sas        CdfLOGNGPD(170)
 *   orva_gfit.sas           CdfLOGNGPD(188)
 *   orva_histplot.sas       CdfLOGNGPD(176), PdfLOGN(177)
 *   orva_los_sim.sas        SdfLOGNGPD(263), SimLOGN(408)
 *   orva_probplot.sas       CdfLOGNGPD(160)
 *   orva_qqplot.sas         QuantLOGNGPD(118)
 *   orva_sfitall.sas        FitLOGNGPD(304), MakeLLGoodLOGNGPD(320), NoMissingMOMLOGNGPD(379)
 *   orva_sfitall_model.sas  ModelFitLOGNGPD(287), MomentsLOGNGPD(391)
 *   orva_sfitall_mom.sas    MomFitLOGNGPD(78)
 *
 ******************************************************************************/

/*****************************************************************************************
 *   Needed for: %orva_cdfplot     ~line 170
 *               %orva_gfit        ~line 188
 *               %orva_histplot    ~line 176
 *               %orva_probplot    ~line 160
 *
 *   What it does  :  Evaluate the cumulative distribution function (CDF)
 *
 *   Parameters    :
 *              x     = value at which the CDF is evaluated
 *        valueHolder = name of variable inton which result is stored
 *
 *****************************************************************************************/
/* 
	Old Parameters ----------- New Parameters
   ++++++++++++++++++++++++++++++++++++++++++++
    	location	----------> param_1 (LOGN's location parameter)
        scale	    ----------> param_2 (LOGN's scale parameter)
	    shape1      ----------> param_4 (GPD's shape1 parameter)
        shape2      ----------> param_3 (GPD's scale parameter)
*/

%macro CdfLOGNGPD(x,returnVal);
    if &x LE &Param_xLimit then do;
        &returnVal  = cdf("LOGNORMAL",&x,(&Param_1), &Param_2) * &Param_pnLimit/&Param_pLimit;    
	end;
	else do;
		if &Param_4 = 0 then do;
            &returnVal = &Param_pnLimit + (1-&Param_pnLimit)*(1-exp(-(&x-&Param_xLimit)/&Param_3));
		end;
		else do;
            &returnVal = &Param_pnLimit + (1-&Param_pnLimit)*(1-(1 + &Param_4*(&x-&Param_xLimit)/&Param_3)**(-1/&Param_4));
		end;
	end;
%mend CdfLOGNGPD;

%macro VCdfLOGNGPD(x,returnVal);
    if &x LE Param_xLimit then do;
        &returnVal  = cdf("LOGNORMAL",&x,(Param_1), Param_2) * Param_pnLimit/Param_pLimit;    
	end;
	else do;
		if Param_4 = 0 then do;
            &returnVal = Param_pnLimit + (1-Param_pnLimit)*(1-exp(-(&x-Param_xLimit)/Param_3));
		end;
		else do;
            &returnVal = Param_pnLimit + (1-Param_pnLimit)*(1-(1 + Param_4*(&x-Param_xLimit)/Param_3)**(-1/Param_4));
		end;
	end;
%mend VCdfLOGNGPD;

/*****************************************************************************************
 *   Needed for: %orva_loss_sim    ~line 263
 *
 *   What it does  :  Evaluate the survival distribution function (SDF)
 *
 *   Parameters    :
 *              x     = value at which the SDF is evaluated
 *        valueHolder = name of variable inton which result is stored
 *
 *****************************************************************************************/
%macro SdfLOGNGPD(x,returnVal);
    if &x LE &Param_xLimit then do;
		&returnVal = 1- cdf("LOGNORMAL",&x,(&Param_1), &Param_2) * &Param_pnLimit/ &Param_pLimit;    
	end;
	else do;
		if &Param_4=0 then do;
			&returnVal = (1-&Param_pnLimit) * exp(-(&x-&Param_xLimit)/&Param_3);
		end;
		else do;
			&returnVal = (1-&Param_pnLimit) * ((1 + (&Param_4)*(&x-&Param_xLimit)/&Param_3)**(-1/&Param_4));
		end;
	end;
%mend SdfLOGNGPD;

%macro VSdfLOGNGPD(x,returnVal);
    if &x LE Param_xLimit then do;
        &returnVal = 1 - cdf("LOGNORMAL",&x, (Param_1), Param_2)*Param_pnLimit/Param_pLimit;    
	end;
	else do;
		if Param_4=0 then do;
            &returnVal = (1-Param_pnLimit) * exp(-(&x-Param_xLimit)/Param_3);
		end;
		else do;
            &returnVal = (1-Param_pnLimit)*
                    ((1 + (Param_4)*(&x-Param_xLimit)/Param_3)**(-1/Param_4));
		end;
	end;
%mend VSdfLOGNGPD;

/*****************************************************************************************
 *   Needed for: %orva_histplot    ~line 176
 *
 *   What it does  :  Evaluate the density distribution function (CDF)
 *
 *   Parameters    :
 *              x     = value at which the density is evaluated
 *        valueHolder = name of variable inton which result is stored
 *
 *****************************************************************************************/
%macro PdfLOGNGPd(x,returnVal);
    if &x LE &Param_xLimit then do;
        &returnVal = pdf("LOGNORMAL", &x, (&Param_1), &Param_2) * &Param_pnLimit / &Param_pLimit;    
	end;
	else do;
		if &Param_4 = 0 then do;
			&returnVal = (1-&Param_pnLimit) * exp(-(&x-&Param_xLimit)/&Param_3)/&Param_3;
		end;
		else do;
			&returnVal = (1-&Param_pnLimit) * ((1 + (&Param_4)*(&x-&Param_xLimit)/&Param_3)**(-1/&param_4 -1))/&Param_3;
		end;
	end;
%mend PdfLOGNGPD;

%macro VPdfLOGNGPd(x,returnVal);
    if &x LE Param_xLimit then do;
            &returnVal=pdf("LOGNORMAL",&x, (Param_1), Param_2)*Param_pnLimit/Param_pLimit;    
	end;
	else do;
		if Param_4 = 0 then do;
            &returnVal = (1-Param_pnLimit) * exp(-(&x-Param_xLimit)/Param_3)/Param_3;
		end;
		else do;
            &returnVal = (1-Param_pnLimit)*
                        ((1 + (Param_4)*(&x-Param_xLimit)/Param_3)**(-1/Param_4 -1))/Param_3;
		end;
	end;
%mend VPdfLOGNGPD;

/*****************************************************************************************
 *   Needed for: %orva_qqplot    ~line 118
 *
 *   What it does  :  Generates a dataset named _temp_ containing probability values
 *                    and their corresponding quantiles
 *
 *   No Parameters    
 *
 *****************************************************************************************/
%macro QuantLOGNGPD;
	data _temp_;/* <&VAR><THRESHOLD><TRUNCDF><CDF><QUANT>*/
		set _temp_ nobs=nobs ;/* <&VAR><THRESHOLD>*/
		retain truncdf;

		if _N_ = 1 then do;
			if THRESHOLD=0 then do;
				truncdf=0;
			end;
			else do;
				%CDFLOGNGPD(THRESHOLD,truncdf)
			end;
		end;

        cdf = ecdf*(1-truncdf)+truncdf;

        if (cdf <= &Param_pnLimit) then do;
	        quant = exp( &Param_1 + &Param_2 * probit(cdf * &Param_pLimit / &Param_pnLimit) );
        end;
	run;

    %if %sysevalf(&Param_4 > 0, boolean) %then %do;
    	%let quantTail = &Param_3* ((1/(1-cdf))**(&Param_4)-1)/(&Param_4);
    %end;
    %else %do;
        %let quantTail = - &Param_3 * log(1-cdf);
    %end;

	data _temp_;
		set _temp_ ;

        if (cdf > &Param_pnLimit) then do;
            cdf = (cdf-&Param_pnLimit)/(1-&Param_pnLimit);
/*            quant = &Param_xLimit + (&quantTail)/exp(&KRI_LOG_AVG);*/
            quant = &Param_xLimit + (&quantTail);
			end;
	run;
		

%mend QuantLOGNGPD;

/*****************************************************************************************
 *   Needed for: %orva_loss_sim    ~line 408
 *
 *   What it does  :  Simulates a random value
 *
 *   No Parameters    :  
 *
 *****************************************************************************************/
%macro SimLOGNGPD(loss);
	p=rand("UNIFORM");

    if p LE Param_pnLimit then do;
        &loss = exp(probit(p * Param_pLimit / Param_pnLimit) * Param_2 + (Param_1));
	end;
	else do;
	    if Param_4 > 0 then do;
            &loss    = Param_xLimit + (((1 / rand("UNIFORM"))**(Param_4) -1) * Param_3)/Param_4;        
	    end;
    	else do;
            &loss = Param_xLimit -Param_3 * log(rand("UNIFORM"));
	    end;
	end;
%mend SimLOGNGPD;

/*****************************************************************************************
 *   Needed for: %orva_sfitall    ~line 304
 *
 *   What it does  :  Actually does the minimization using PROC NLP
 *
 *   No Parameters    
 *
 *****************************************************************************************/
%macro InitFitLOGNGPD;
	%let nparms=4;    

	/*- initial guess based on moments -*/
	data _null_;
		avg_uss=&sample_uss / &obsCount;
		sigma_sq=log(avg_uss)-2*log(&sample_mean);
		mu=0.5*log(avg_uss)-sigma_sq;	
		sigma=sqrt(sigma_sq);

		call symput("Param_1_init",mu);
		call symput("Param_2_init",sigma);
		call symput("Param_4_init",.);
		call symput("Param_3_init",.);
	run;
	
	%put **********************************;
	%put Distribution    =  &dist;
	%put Observations    =  &obsCount;
	%put Param_1_init    =  &Param_1_init;
	%put Param_2_init    =  &Param_2_init;
	%put **********************************;

%mend InitFitLOGNGPD;

/*****************************************************************************************
 *   Needed for: %orva_sfitall    ~line 304
 *
 *   What it does  :  Actually does the minimization using PROC NLP
 *
 *   No Parameters    
 *
 *****************************************************************************************/
%macro FitLOGNGPD;
	%if &TailBoundaryChoice = TailBoundaryValue %then %do;
        %let Param_xLimit=&TailBoundaryValue;
	%end;
	%else %do;
        %let Param_pLimit = &TailBoundaryPercent;
		proc univariate data =_workdata_ PCTLDEF=4 noprint;
				output out=__tmp__ pctlpts=&TailBoundaryPercent pctlpre=PCTL;
			var AMOUNT;
		run;
		quit;
		proc transpose data=__tmp__ out =__tmp__;
		run;
		data _null_;
			set __tmp__;
            call symput("Param_xLimit",COL1);
		run;
	%end;

%put ****************************;
%put Param_xLimit = &Param_xLimit;
%put ****************************;


	data _tmpworkdata_;
		set _workdata_ nobs=TotalCount;
		where AMOUNT <= &Param_xLimit;
		call symput("TotalCount",TotalCount);
	run;

	proc summary data=_tmpworkdata_ noprint;
		output out = _body_;
	run;

	data _null_;
		set _body_;
		call symput("BodyCount",_FREQ_);
	run;

    %let Param_pnLimit=%sysevalf(&BodyCount/&TotalCount);

	%let tmpdsid=%sysfunc(open(_tmpworkdata_,I));
	%let tmpnparmobs=%sysfunc(attrn(&tmpdsid,nlobs));
	%let tmprc=%sysfunc(close(&tmpdsid));

	%if &tmpnparmobs=0 %then %do;
   		%put ERROR: No observations below tail threshold.;
		%let status=1;
   		%let reason=No observations below tail threshold;
   		%goto LOCAL_DONE;
	%end;


	/* This piece of code increases Param_pnLimit if the probability of observability is provided	  */
	/* New_pnLimit = Old_pnLimit + (1-Old_pnLimit)*p0 where p0 is the probability below the threshold */
	%if %sysevalf(&THRESHOLDPERCENT >= 0, BOOLEAN) %then %do;
		%let Param_pnLimit = %sysevalf(&Param_pnLimit + (1-&Param_pnLimit)* &THRESHOLDPERCENT); 
	%end;

	ods listing close;
	ods output 
				IterStop          =  _logl_
				ConvergenceStatus =  _cs_ 
				covmat            =  _covmat_
				;

	proc nlp 
		data = _tmpworkdata_ 
		cov  = 4 
		pcov 
		maxit=10000 
		maxfu=10000 
		tech=qn 
		outest=_parmnlp_ ;

		/* The list of parameters */
		parms 
				Param_1 = &Param_1_init,
				Param_2 = &Param_2_init;

		/* The bounds on parameters */
		bounds 
					Param_2 > 0;

		max logf;
			tlcdf = 0;      /*threshold log cdf ------*/
			F0 = 0;
			
			pStar = probnorm( (log(&Param_xLimit)-Param_1) / Param_2);


            /* Handle the probability of observability */
			if &THRESHOLDPERCENT >= 0 then do;
				/* Compute the Cdf at the Threshold point */
				theCdf = cdf("LOGNORMAL", THRESHOLD, param_1, param_2);

				tlcdf =  &THRESHOLDPERCENT/(&Param_pnLimit - &THRESHOLDPERCENT)*
							(
								log(theCdf)
								- log(pStar)
							);
			end; 
			else do;
				if THRESHOLD ne 0 then do;
					F0 = probnorm ((log(threshold)-Param_1)/Param_2);
				end;
			end;

			logf =   -log(pStar - F0)
						+ tlcdf
						-0.5*log(2)
						-lgamma(0.5)
						-log(AMOUNT)
						-log(Param_2)
						-(log(AMOUNT) -Param_1)*(log(AMOUNT)-Param_1)
						/(2*Param_2*Param_2)
						; 
    run;
    quit;
	ods listing;

    %if %sysfunc(exist(_cs_)) %then %do;
    data _null_;
        set _cs_;
        call symput("status",status);
    run;
    %end;
    %else %do;
        %let status=1;
        %let reason=Fit failed;
    %end;


    /* Let's check if the first part of the fit worked  */  
    %if &status=0  %then %do;   
        /* Let's try to fit the tail                */
        data _null_;
            set _parmnlp_;
            where index(_type_,"PARMS")>0;
            /*  This is the body probability of being less than Param_xLimit */
            call symput("Param_pLimit",cdf("NORMAL",log(&Param_xLimit),Param_1,Param_2));                
        run;
    
        data _tmpworkdata_;
            set _workdata_;
            where AMOUNT > &Param_xLimit;
            AMOUNT = AMOUNT-&Param_xLimit;
        run;


	/* For the tail fit the initial guess should be based 
	on the tail distribution moments and not on the full
	distribution moments                                            */

    proc summary data=_tmpworkdata_ noprint;
            var AMOUNT;
            output 
                out  = _stat_(keep=sample_mean2 sample_variance2 sample_size2)
                n    = sample_size2
                mean = sample_mean2 
                var  = sample_variance2
                ; 
        run;
        
        proc transpose 
            data=_stat_ 
            out=_stat_;
        run;
        
        data _null_;
            set _stat_;
            call symput(_name_, col1);
        run;

        %if %sysevalf(&sample_size2 < 2, boolean) %then %do;
            %put ERROR: There are not enough observations above the tail breakpoint.;
            %let status=1;
            %let reason=NOT ENOUGH OBSERVATIONS IN TAIL;
            %goto LOCAL_DONE;
        %end;

        data _null_;
            alpha=(&sample_mean2*&sample_mean2 / &sample_variance2) + 2;
            theta=&sample_mean2*(alpha-1);
            call symput("Param_4_init",1/alpha);
            call symput("Param_3_init",theta/alpha);
        run;



        ods output 
            IterStop           =  _logl2_
            ConvergenceStatus  =  _cs_ 
            covmat             =  _covmat2_
            ;   

		proc nlp 
			data=_tmpworkdata_ 
			cov=4 pcov 
			maxit=10000 
			maxfu=10000 
			tech=qn 
			outest=_parmnlp2_
			;

            /* The list of parameters */
			parms 
				Param_3 = &Param_3_init,
				Param_4 = &Param_4_init;

			/* Bounds on the parameters */
			bounds
				Param_3 > 0,
				Param_4 > 0;

			/* The actual optimization */
			max logf;

			logf =	- log(Param_3) 
					- (1/Param_4 + 1) * 
					log1px( Param_4*AMOUNT/Param_3);
		run;
		quit;

        data _parmnlp_;
            set _parmnlp_;
                 where (index(_type_,"PARMS")>0);
        run;
        
        data _parmnlp2_;
            set _parmnlp2_;
                 where (index(_type_,"PARMS")>0);
        run;

        data _parmnlp_;
            merge _parmnlp_ _parmnlp2_;
            Param_xLimit=&Param_xLimit;
            Param_pLimit=&Param_pLimit;
            Param_pnLimit=&Param_pnLimit;
        run;

        data _CovMat_;
            set _covmat_ _covmat2_;
        run;

		%LLLOGNGPD
        
		data _logl_; 
			set _logl_;        
			if _N_=3 then do;
				nValue1 = &logl;
			end;
		run;
    %end;
    %else %do;
        %let status=1;
        %let reason=DISTRIBUTION BODY FIT FAILURE;
    %end;
%LOCAL_DONE:
%mend FitLOGNGPD;

%macro LLLOGNGPD;
	
	data tmp_parmnlp; 
				set _parmnlp_;
				CCUID=&CCUID;
	run;
	
	/* Compute the log likelihood */
	data _NULL_; 
		merge _workdata_ tmp_parmnlp end=last;
		by CCUID;
		retain logl 0 ;
		tlcdf = 0;     
		F0 = 0;
		
		if &THRESHOLDPERCENT >= 0 then do;
			%VCDFLOGNGPD(THRESHOLD, tlcdf)
			tlcdf =  &THRESHOLDPERCENT/(1 - &THRESHOLDPERCENT) * log(tlcdf);
		end; 
		else if THRESHOLD ne 0 then do;
			%VCDFLOGNGPD(THRESHOLD, F0)
		end;

		%VPDFLOGNGPD(AMOUNT, logf)

		logf = log(logf);
		logl = logl + logf -log(1 - F0) + tlcdf ;

   		if last then do;
			call symput("LOGL", logl);
		end;
	run; 
%mend LLLOGNGPD;

%macro LLKRILOGNGPD;
	data tmp_parmnlp; 
				set _parmnlp_;
				CCUID=&CCUID;
	run;
	
	/* Compute the log likelihood */
	data _NULL_; 
		merge _LogWorkdata_KRI_ tmp_parmnlp end=last;
		by CCUID;
		retain logl 0;

		tlcdf = 0;     
		F0 = 0;
		
		if &THRESHOLDPERCENT >= 0 then do;
			%VCDFLOGNGPD(KRI_ADJ_THRESHOLD, tlcdf)
			tlcdf =  &THRESHOLDPERCENT/(1 - &THRESHOLDPERCENT) * log(tlcdf);
		end; 
		else if THRESHOLD ne 0 then do;
			%VCDFLOGNGPD(KRI_ADJ_THRESHOLD, F0)
		end;

		%VPDFLOGNGPD(KRI_ADJ_AMOUNT  , logf)

		logf = log(logf) - log(KRI_SCALE);
		logl = logl + logf - log(1 - F0) + tlcdf ;

   		if last then do;
			call symput("LOGL", logl);
		end;
	run; 
%mend LLKRILOGNGPD;

/*****************************************************************************************
 *   Needed for: %orva_sfit_kri    ~line 304
 *
 *   What it does  :  Actually does the minimization using PROC NLP
 *
 *   No Parameters    
 *
 *****************************************************************************************/
%macro FitKriLOGNGPD;
	%if &TailBoundaryChoice = TailBoundaryValue %then %do;
		  %let Param_xLimit=&TailBoundaryValue;

		data _NULL_;
			set _LogWorkdata_KRI_ end=last;
			retain InverseScaleSum 0;

			InverseScale = 1/KRI_SCALE;
			InverseScaleSum = InverseScaleSum + InverseScale;

			if last then do;
				call symput("Param_xLimit", &TailBoundaryValue. * InverseScaleSum/_N_);
			end;
		run;
	%end;
	%else %do;
       %let Param_pLimit = &TailBoundaryPercent;
		proc univariate data =_LogWorkdata_KRI_ PCTLDEF=4 noprint;
			output out=__tmp__ pctlpts=&TailBoundaryPercent pctlpre=PCTL;
			var KRI_ADJ_AMOUNT;
		run;
		quit;

		proc transpose data=__tmp__ out =__tmp__;
		run;

		data _null_;
			set __tmp__;
            call symput("Param_xLimit",COL1);
		run;
	%end;


	data _tmpworkdata_KRI_;
		set _LogWorkdata_KRI_ nobs=TotalCount end=TheEnd;
		where KRI_ADJ_AMOUNT <= &Param_xLimit;

		call symput("TotalCount",TotalCount);
	run;

	%let tmpdsid=%sysfunc(open(_tmpworkdata_KRI_,I));
	%let tmpnparmobs=%sysfunc(attrn(&tmpdsid,nlobs));
	%let tmprc=%sysfunc(close(&tmpdsid));

	%if &tmpnparmobs=0 %then %do;
  		%put ERROR: No observations below tail threshold.;
		%let status=1;
 		%let reason=No observations below tail threshold;
 		%goto LOCAL_DONE;
	%end;

	proc summary data=_tmpworkdata_KRI_ noprint;
		output
		out = _body_;
	run;

	data _null_;
		set _body_;
		call symput("BodyCount",_FREQ_);
	run;

   %let Param_pnLimit=%sysevalf(&BodyCount/&TotalCount);

	/* This piece of code increases Param_pnLimit if the probai=bility of observability is provided	*/
	/* New_pnLimit = Old_pnLimit + (1-Old_pnLimit)*p0 where p0 is the probability of being hidden   */
	%if %sysevalf(&THRESHOLDPERCENT >= 0, BOOLEAN) %then %do;
		%let Param_pnLimit = %sysevalf(&Param_pnLimit + (1-&Param_pnLimit)* &THRESHOLDPERCENT); 
	%end;

	ods listing close;
	ods output 
		IterStop           =  _logl_
		ConvergenceStatus  =  _cs_ 
		covmat             =  _covmat_
		;	

	proc nlp 
		data = _tmpworkdata_KRI_ 
		cov  = 4 
		pcov 
		maxit=10000 
		maxfu=10000 
		tech=qn 
		outest=_parmnlp_
		;

		/* The list of parameters */		
		parms 
			Param_1 = &Param_1_init,
			Param_2 = &Param_2_init;
			
		/* The bounds on parameters */		
		bounds 
			Param_2 > 0;
		/* The optimization for the body */
		max logf;
		tlcdf = 0;      /*threshold log cdf ------*/
		F0 = 0;
		
		/* Compute the Cdf at the threshold */
		pStar=probnorm( (log(&Param_xLimit) + &KRI_LOG_AVG - Param_1 - KRI_LOG) / Param_2);
		
		/* Handle the probability of observability */
		if &THRESHOLDPERCENT >= 0 then do;
			theCdf = cdf("LOGNORMAL", THRESHOLD, param_1 + KRI_LOG, param_2);
			
			tlcdf =  &THRESHOLDPERCENT/(&Param_pnLimit  - &THRESHOLDPERCENT)*
						(
							log(theCdf)
							-log(pStar)
						);
		end; 
		else if THRESHOLD ne 0 then do;
			F0 = probnorm ((LogThreshold - Param_1 - KRI_LOG)/Param_2);
		end;

		logf = 	-log(pStar -  F0 )
				+ tlcdf
				-0.5*log(2)
				-lgamma(0.5)
				-LogAMOUNT
				-log(Param_2)
				-(LogAMOUNT - Param_1 - KRI_LOG )*(LogAMOUNT - Param_1 - KRI_LOG)/(2*Param_2*Param_2); 

	run;
	quit;
	ods listing;


	%if %sysfunc(exist(_cs_)) %then %do;
	data _null_;
		set _cs_;
		call symput("status",status);
	run;
	%end;
	%else %do;
		%let status=1;
		%let reason=Fit failed;
	%end;

	/* Let's check if the first part of the fit worked 	*/	
	%if &status=0  %then %do; 	
		/* Let's try to fit the tail				*/
		data _null_;
			set _parmnlp_;
			where index(_type_,"PARMS") > 0;
			/*  This is the body probability of being less than Param_xLimit */
			call symput("Param_pLimit",cdf("NORMAL",log(&Param_xLimit),Param_1,Param_2)); 
		run;
	
		data _tmpworkdata_;
			set _LogWorkdata_KRI_;
			where KRI_ADJ_AMOUNT > &Param_xLimit;
/*			KRI_ADJ_AMOUNT = (KRI_ADJ_AMOUNT- &Param_xLimit)*KRI_SCALE;*/
			KRI_ADJ_AMOUNT = (KRI_ADJ_AMOUNT- &Param_xLimit);
		run;


		%let tmpdsid=%sysfunc(open(_tmpworkdata_,I));
		%let tmpnparmobs=%sysfunc(attrn(&tmpdsid,nlobs));
		%let tmprc=%sysfunc(close(&tmpdsid));

		%if &tmpnparmobs=0 %then %do;
			%put ERROR: No observations above tail threshold.;
			%let status=1;
			%let reason=No observations above tail threshold;
			%goto LOCAL_DONE;
		%end;

		/* For the tail fit the initial guess should be based 
		on the tail distribution moments and not on the full
		distribution moments                                            */

		proc summary data=_tmpworkdata_ noprint;
			var KRI_ADJ_AMOUNT;
			output 
				out  = _stat_(keep=sample_mean2 sample_variance2 sample_size2)
				n    = sample_size2
				mean = sample_mean2 
				var  = sample_variance2 
				; 
		run;

        proc transpose 
            data=_stat_ 
            out=_stat_;
        run;
        
		data _null_;
            set _stat_;
            call symput(_name_, col1);
        run;

        %if %sysevalf(&sample_size2 < 2, boolean) %then %do;
            %put ERROR: There are not enough observations above the tail breakpoint.;
            %let status=1;
            %let reason=NOT ENOUGH OBSERVATIONS IN TAIL;
            %goto LOCAL_DONE;
        %end;
    
     data _null_;
         alpha=(&sample_mean2*&sample_mean2 / &sample_variance2) + 2;
         theta=&sample_mean2*(alpha-1);
         call symput("Param_4_init", 1/alpha);
         call symput("Param_3_init", theta/alpha);
     run;
        



		  	data _NULL_;
				set _tmpworkdata_ ;
				retain logl 0;
				logl = logl + log(kri_SCALE);
				call symput("LOGL_KRI",logl);
			run;

        ods output 
            IterStop           =  _logl2_
            ConvergenceStatus  =  _cs_ 
            covmat             =  _covmat2_
            ;   

        proc nlp 
                data=_tmpworkdata_ 
	            cov=4 pcov 
                maxit=10000 
                maxfu=10000 
                tech=qn 
                outest=_parmnlp2_
                ;

			/* List of the parameters */
			parms 
				Param_3 = &Param_3_init,
				Param_4 = &Param_4_init;

			/* Bounds on the parameters */				
			bounds 
				Param_3 > 0,
				Param_4 > 0;

			/* Do the optimization */
			max logf;
			
			logf =	-log(Param_3) 
					- (1/Param_4 + 1) * 
					log1px( Param_4*KRI_ADJ_AMOUNT/Param_3);
		run;
		quit;

		data _parmnlp_;
			set _parmnlp_;
			where (index(_type_,"PARMS")>0);
		run;

		data _parmnlp2_;
			set _parmnlp2_;
			where (index(_type_,"PARMS")>0);
		run;

		data _parmnlp_;
			merge _parmnlp_ _parmnlp2_;
			Param_xLimit = &Param_xLimit;
			Param_pLimit = &Param_pLimit;
			Param_pnLimit = &Param_pnLimit;
		run;

		data _CovMat_;
			set _covmat_ _covmat2_;
		run;

		%LLKRILOGNGPD;
        
        data _logl_;
            set _logl2_;

			if _N_ = 3 then do;
/*				nValue1 = &logl + &LOGL_KRI;*/
				nValue1 = &logl;
			end;
		run;
	%end;
	%else %do;
		%let status=1;
		%let reason=DISTRIBUTION BODY FIT FAILURE;
	%end;
%LOCAL_DONE:

%mend FitKriLOGNGPD;

/*****************************************************************************************
 *   Needed for: %orva_sfitall    ~line 320
 *
 *   What it does  :  Modify loglikelihood for cases when we had to take the log of the 
 *                    observations first. Here, it does nothing.
 *
 *   NO Parameters    
 *
 *****************************************************************************************/
%macro MakeLLGoodLOGNGPD;
    %let sumlog=0;
%mend MakeLLGoodLOGNGPD;

%macro MakeLLGoodKRILOGNGPD;
    %let sumlog=0;
%mend MakeLLGoodKRILOGNGPD;

/*****************************************************************************************
 *   Needed for: %orva_sfitall    ~line 379
 *
 *   What it does  :  Checks whether some moments are missing.
 *                    Here does nothing because all moments exist for Exponential distribution
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro NoMissingMOMLOGNGPD(parmdata=);
    data &parmdata;
        set &parmdata;

        if mean=. then do;
            status=1;
            reason="Bad Parameter Combination: Mean does not exist.";
        end;
        else if std=. then do;
            status=2;
            reason="Bad Parameter Combination: Variance does not exist.";
        end;

        call symput("status",status);
	run;
%mend NoMissingMOMLOGNGPD;

/*- actually, the moments of logarithm of losses -*/
%macro MomentsLOGNGPD;

	/* Part of the moment due to the Lognormal body                   */
	/* Need to use limited expectation of lognormal                   */
	/* See formula in "Loss Models" by Klugman on page 583 at the top */         
    %do i=1 %to 4;
        m&i=exp(&i*Param_1 + &i*&i*Param_2*Param_2/2)*probnorm((log(Param_xLimit)+ &KRI_LOG_AVG-Param_1-&i*Param_2*Param_2)/Param_2) ;
    %end;

	/*	This piece is so that the contribution of the lognormal has the right weight,*/
    /*  i.e Param_pnLimit=empirical probability*/
	%do i=1 %to 4;
        m&i = m&i * Param_pnLimit/Param_pLimit;
    %end;


	/* To each of the moments, add the part due to the GPD tail */
   if 1 < 1/Param_4 then do;
        mGPD1 = exp(&KRI_LOG_AVG) * (Param_3/Param_4)/(1/Param_4-1);
        m1    = m1 + (1 - Param_pnLimit)*(mGPD1 + Param_xLimit * exp(&KRI_LOG_AVG));
		end;
		else do;
        m1 = .;
		end;
    
   if 2 < 1/Param_4 then do;
        mGPD2 = exp(2*&KRI_LOG_AVG) * ((Param_3/Param_4)**2)*2/((1/Param_4-1)*(1/Param_4-2));
        m2    = m2 + (1 - Param_pnLimit) * (mGPD2 + 2 * Param_xLimit * exp(&KRI_LOG_AVG) * mGPD1 + (Param_xLimit * exp(&KRI_LOG_AVG))**2 );
	end;
	else do;
        m2 = .;
	end;
    
    if 3 < 1/Param_4 then do;
        mGPD3 = exp(3*&KRI_LOG_AVG) * ((Param_3/Param_4)**3)*6/((1/Param_4-1)*(1/Param_4-2)*(1/Param_4-3));
        m3    = m3 + (1 - Param_pnLimit)*(mGPD3 + 3 * Param_xLimit * exp(&KRI_LOG_AVG) * mGPD2 + 3 * (Param_xLimit * exp(&KRI_LOG_AVG))**2 * mGPD1  + (Param_xLimit * exp(&KRI_LOG_AVG))**3 );
	end;
	else do;
        m3 = .;
	end;
	
    
   	if 4 < 1/Param_4 then do;
        mGPD4 = exp(4*&KRI_LOG_AVG) * ((Param_3/Param_4)**4)*24/((1/Param_4-1)*(1/Param_4-2)*(1/Param_4-3)*(1/Param_4-4));
        m4    = m4 + (1 - Param_pnLimit) * (mGPD4 + 4 * Param_xLimit * exp(&KRI_LOG_AVG) * mGPD3 + 6 * (Param_xLimit * exp(&KRI_LOG_AVG))**2 * mGPD2 + 4 * (Param_xLimit * exp(&KRI_LOG_AVG))**3 * mGPD1  + (Param_xLimit * exp(&KRI_LOG_AVG))**4 );
	end;
	else do;
        m4 = .;
	end;

    mean=m1;
    std=sqrt(m2-m1*m1);
    skewness=(m3-3*m1*m2+2*(m1**3))/(std**3);
    kurtosis=(m4-4*m1*m3+6*(m1**2)*m2-3*(m1**4))/(std**4);
%mend MomentsLOGNGPD;

%macro RealMomentsLOGNGPD;
	%MomentsLOGNGPD
%mend RealMomentsLOGNGPD;

/*****************************************************************************************
 *   Needed for: %orva_sfitall_mom.sas      ~line 78
 *
 *   What it does  :  Does method of moments estimation and assigns the corresponding 
 *                    values to the param_1, param_2, param_4, and param_3 macros
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro MomFitLOGNGPD;
	/*- expect  the mean, variance, and kurtosis of logarithm of losses -*/    
	data _null_;
        Param_2=sqrt(&variance);
        call symput("Param_1",&mean);
	    call symput("Param_2",Param_2);
	run;
%mend MomFitLOGNGPD;
/*****************************************************************************************
 *   Needed for: %orvr_sfitstats.sas      ~line 78
 *
 *   What it does  :  Displays the estimated parameters
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro ParamsLOGNGPD;
	data foo (keep=Variable Estimate StdErr TValue probt);
		format Variable $12.;
		set &sparm;
		Variable="LocationBody";
		Estimate=Param_1;
		StdErr=sqrt(cov_1);
		TValue=estimate/StdErr;
		probt=(1-probt(abs(tvalue),obscount -&KRI_ADJUST-&Param_count))*2;
		output;
		Variable="ScaleBody";
		Estimate =Param_2;
		StdErr=sqrt(cov_3);
		TValue=estimate/StdErr;
		probt=(1-probt(abs(tvalue),obscount -&KRI_ADJUST-&Param_count))*2;
		output;
		Variable="ScaleTail";
		Estimate=Param_3;
		StdErr=sqrt(cov_6);
		TValue=estimate/StdErr;
		probt=(1-probt(abs(tvalue),obscount -&KRI_ADJUST-&Param_count))*2;
		output;
		Variable="ShapeTail";
		Estimate=Param_4;
		StdErr=sqrt(cov_10);
		TValue=estimate/StdErr;
		probt=(1-probt(abs(tvalue),obscount -&KRI_ADJUST-&Param_count))*2;
		output;
    run;

	data _NULL_;set foo;
          file print ods=(template='Stat.Reg.ParameterEstimates');
          put _ods_;
	run;
%mend ParamsLOGNGPD;

/*****************************************************************************************
 *   Needed for: %orvr_sfitstats.sas      ~line 78
 *
 *   What it does  :  Displays the estimated parameters
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro CovmatLOGNGPD;
	data foo (keep=_name_ Param_1 Param_2 Param_3 Param_4);
		format _name_ $12.;
		set &sparm;
		_name_="LocationBody";
		param_1 =  cov_1;
		param_2    =  cov_2;
		param_3	=	0;
		param_4	=	0;
		output;
		_name_="ScaleBody";
		param_1 =  cov_2;
		param_2    =  cov_3;
		param_3	 =	0;
		param_4	 =	0;
		output;
		_name_="ScaleTail";
		param_1 =  0;
		param_2    =  0;
		param_3	 =	cov_6;
		param_4	 =	cov_9;
		output;
		_name_="ShapeTail";
		param_1 =  0;
		param_2    =  0;
		param_3	 =	cov_9;
		param_4	 =	cov_10;
		output;
	run;

	proc print data=foo noobs label;
		var _name_ Param_1 Param_2 Param_3 Param_4 ;
		label _name_='00'x Param_1="LocationBody" Param_2="ScaleBody"  Param_3="ScaleTail"  Param_4="ShapeTail"  ;
	run;

%mend CovmatLOGNGPD;

/*****************************************************************************************
 *   Needed to be called before anything else is available
 *****************************************************************************************/
%macro ORVD_LOGNGPD;	

    %let EXTRA_PARAMS=  ;
    %let NEEDED_INPUT= TailBoundaryChoice TailBoundaryValue TailBoundaryPercent;
    %let SCALE_TRANSFORM= LOG;
    %let DIST_NAME= Log Normal-GPD;
    %let PARAM_COUNT= 4;

	data LOGNGPD_STATUS;
		status          = 0;
	run;
	
%mend ORVD_LOGNGPD;
