/******************************************************************************
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *   Title: orvd_LOGNGPD_Hill.sas
 *   
 *   -Provides support for the mixture of a LogNormal distribution
 *   -to represent the body and a Generalized Pareto to represent the tail
 *   -The tail cut off is calculated for the user using a Hill based algorithm
 *
 *	This file contains macro fragments to implement calculation specific to the 
 *	LogNormal distribution
 *
 *  The macro %ORVD_LOGNGPD_HILL is at the end of this file. It is an empty macro that takes
 *  no parameter and returns no feedback. In normal usage, this macro needs to be 
 *  called before trying to call any other macros depending on the fragments in this file.
 *
 *  Calling the %ORVD_LOGN_HILL macro makes the SAS system parse and compile the whole file 
 *  so that the other macro fragment then become defined and available for other 
 *  macros to call. This uses the %ORVD_LOGN macro extensively.
 *
 *  Of course this is only guaranteed to work if there is no syntax error in any of 
 *  the fragments. 
 *
 *  The list of other macro files that rely on fragments in this file:
 *
 *   File                    Fragments it depends on (Approx Line # where fragement is called)
 *   orva_cdfplot.sas        CdfLOGNGPD(170)
 *   orva_gfit.sas           CdfLOGNGPD(188)
 *   orva_histplot.sas       CdfLOGNGPD(176), PdfLOGN(177)
 *   orva_los_sim.sas        SdfLOGNGPD(263), SimLOGN(408)
 *   orva_probplot.sas       CdfLOGNGPD(160)
 *   orva_qqplot.sas         QuantLOGNGPD(118)
 *   orva_sfitall.sas        FitLOGNGPD(304), MakeLLGoodLOGNGPD(320), NoMissingMOMLOGNGPD(379)
 *   orva_sfitall_model.sas  ModelFitLOGNGPD(287), MomentsLOGNGPD(391)
 *   orva_sfitall_mom.sas    MomFitLOGNGPD(78)
 *
 ******************************************************************************/

/*****************************************************************************************
 *   Needed for: %orva_cdfplot     ~line 170
 *               %orva_gfit        ~line 188
 *               %orva_histplot    ~line 176
 *               %orva_probplot    ~line 160
 *
 *   What it does  :  Evaluate the cumulative distribution function (CDF)
 *
 *   Parameters    :
 *              x     = value at which the CDF is evaluated
 *        valueHolder = name of variable inton which result is stored
 *
 *****************************************************************************************/

/* 
	Old Parameters ----------- New Parameters
   ++++++++++++++++++++++++++++++++++++++++++++
    	location	----------> param_1 (LOGN's location parameter)
        scale	    ----------> param_2 (LOGN's scale parameter)
	    shape1      ----------> param_4 (GPD's shape1 parameter)
        shape2      ----------> param_3 (GPD's scale parameter)
*/

%macro CdfLOGNGPD_HILL(x,returnVal);
        %CdfLOGNGPD1(&x,&returnVal);
%mend CdfLOGNGPD_HILL;



/*****************************************************************************************
 *   Needed for: %orva_loss_sim    ~line 263
 *
 *   What it does  :  Evaluate the survival distribution function (SDF)
 *
 *   Parameters    :
 *              x     = value at which the SDF is evaluated
 *        valueHolder = name of variable inton which result is stored
 *
 *****************************************************************************************/
%macro SdfLOGNGPD_HILL(x,returnVal);
        %SdfLOGNGPD1(&x,&returnVal);
%mend SdfLOGNGPD_HILL;



/*****************************************************************************************
 *   Needed for: %orva_histplot    ~line 176
 *
 *   What it does  :  Evaluate the density distribution function (CDF)
 *
 *   Parameters    :
 *              x     = value at which the density is evaluated
 *        valueHolder = name of variable inton which result is stored
 *
 *****************************************************************************************/
%macro PdfLOGNGPd_HILL(x,returnVal);
       %PdfLOGNGPd1(&x,&returnVal);
%mend PdfLOGNGPD_HILL;



/*****************************************************************************************
 *   Needed for: %orva_qqplot    ~line 118
 *
 *   What it does  :  Generates a dataset named _temp_ containing probability values
 *                    and their corresponding quantiles
 *
 *   No Parameters    
 *
 *****************************************************************************************/
%macro QuantLOGNGPD_HILL;
       %QuantLOGNGPD1;
%mend QuantLOGNGPD_HILL;

/*****************************************************************************************
 *   Needed for: %orva_loss_sim    ~line 408
 *
 *   What it does  :  Simulates a random value
 *
 *   No Parameters    :  
 *
 *****************************************************************************************/
%macro SimLOGNGPD_HILL(loss);
       %SimLOGNGPD1(&loss);
%mend SimLOGNGPD_HILL;

/*****************************************************************************************
 *   Needed for: %orva_sfitall    ~line 304
 *
 *   What it does  :  Actually does the minimization using PROC NLP
 *
 *   No Parameters    
 *
 *****************************************************************************************/
%macro InitFitLOGNGPD_HILL;
       %InitFitLOGNGPD1;
%mend InitFitLOGNGPD_HILL;

/*****************************************************************************************
 *   Needed for: %orva_sfitall    ~line 304
 *
 *   What it does  :  Actually does the minimization using PROC NLP
 *
 *   No Parameters    
 *
 *****************************************************************************************/
%macro FitLOGNGPD_HILL;
    %let Param_xLimit=.;
    /* get the tail index  ------------------*/
    %orva_get_hill_estimate( data = _workdata_ , useKRI = NO);

	data _tmpworkdata_;
		set _workdata_ nobs=TotalCount;
		where AMOUNT <= &Param_xLimit;
		call symput("TotalCount",TotalCount);
	run;

	proc summary data=_tmpworkdata_ noprint;
		output out = _body_;
	run;

	data _null_;
		set _body_;
		call symput("BodyCount",_FREQ_);
	run;

    %let Param_pnLimit=%sysevalf(&BodyCount/&TotalCount);

	%let tmpdsid=%sysfunc(open(_tmpworkdata_,I));
	%let tmpnparmobs=%sysfunc(attrn(&tmpdsid,nlobs));
	%let tmprc=%sysfunc(close(&tmpdsid));

	%if &tmpnparmobs=0 %then %do;
   		%put ERROR: No observations below tail threshold.;
		%let status=1;
   		%let reason=No observations below tail threshold;
   		%goto LOCAL_DONE;
	%end;


	/* This piece of code increases Param_pnLimit if the probability of observability is provided	  */
	/* New_pnLimit = Old_pnLimit + (1-Old_pnLimit)*p0 where p0 is the probability below the threshold */
	%if %sysevalf(&THRESHOLDPERCENT >= 0, BOOLEAN) %then %do;
		%let Param_pnLimit = %sysevalf(&Param_pnLimit + (1-&Param_pnLimit)* &THRESHOLDPERCENT); 
	%end;

	ods listing close;
	ods output 
				IterStop          =  _logl_
				ConvergenceStatus =  _cs_ 
				covmat            =  _covmat_
				;

	proc nlp 
		data = _tmpworkdata_ 
		cov  = 4 
		pcov 
		maxit=10000 
		maxfu=10000 
		tech=qn 
		outest=_parmnlp_ ;

		parms 
				Param_1 = &Param_1_init,
				Param_2 = &Param_2_init;

		bounds 
					Param_2 > 0;

		max logf;

			tlcdf = 0;      /*threshold log cdf ------*/
			F0 = 0;
			pStar = probnorm( (log(&Param_xLimit)-Param_1) / Param_2);
            
			if &THRESHOLDPERCENT >= 0 then do;
				tlcdf =  &THRESHOLDPERCENT/(&Param_pnLimit - &THRESHOLDPERCENT)*
							(
								logcdf("LOGNORMAL", THRESHOLD, param_1, param_2)
								- log(pStar)
							);
			end; 
			else do;
				if THRESHOLD ne 0 then do;
					F0 = probnorm ((log(threshold)-Param_1)/Param_2);
				end;
			end;

			logf =   -log(pStar -F0)
						+ tlcdf
						-0.5*log(2)
						-lgamma(0.5)
						-log(AMOUNT)
						-log(Param_2)
						-(log(AMOUNT) -Param_1)*(log(AMOUNT)-Param_1)
						/(2*Param_2*Param_2)
						; 
    run;
    quit;
	ods listing;

	%if %sysfunc(exist(_cs_)) %then %do;
	data _null_;
		set _cs_;
		call symput("status",status);
	run;
	%end;
	%else %do;
		%let status=1;
		%let reason=Fit failed;
	%end;

	/* Let's check if the first part of the fit worked 	*/	
	%if &status=0  %then %do; 	
		/* Let's try to fit the tail				*/
		data _null_;
			set _parmnlp_;
			where index(_type_,"PARMS")>0;
			/*  This is the LogNormal probability of being less than Param_xLimit */
			call symput("Param_pLimit",cdf("NORMAL",log(&Param_xLimit),Param_1,Param_2));
		run;
	
		data _tmpworkdata_;
			set _workdata_;
            where AMOUNT > &Param_xLimit;
            AMOUNT = AMOUNT-&Param_xLimit;
		run;


	/*--INSERTED BY R.TRAVERSINI SAS ITALY 27APR2004-------------------*/
	/* For the tail fit the initial guess should be based 
	   on the tail distribution moments and not on the full
	   distribution moments                                            */

	proc summary data=_tmpworkdata_ noprint;
			var AMOUNT;
			output 
				out  = _stat_(keep=sample_mean2 sample_variance2 sample_size2)
				n    = sample_size2
				mean = sample_mean2 
				var  = sample_variance2
				; 
	run;

	proc transpose 
		data=_stat_ 
		out=_stat_;
	run;

	data _null_;
		set _stat_;
		call symput(_name_, col1);
	run;

     %if %sysevalf(&sample_size2 < 2, boolean) %then %do;
			%put ERROR: There are not enough observations above the tail breakpoint.;
			%let status=1;
			%let reason=NOT ENOUGH OBSERVATIONS IN TAIL;
			%goto LOCAL_DONE;
		%end;

		data _null_;
			alpha=(&sample_mean2*&sample_mean2 / &sample_variance2) + 2;
			theta=&sample_mean2*(alpha-1);
			call symput("Param_4_init",alpha);
			call symput("Param_3_init",theta/alpha);
		run;

		
	/*- END 27APR2004-------------------------*/


	ods output 
		IterStop           =  _logl2_
		ConvergenceStatus  =  _cs_ 
		covmat             =  _covmat2_
		;	

		proc nlp 
			data=_tmpworkdata_ 
			cov=4 pcov 
			maxit=10000 
			maxfu=10000 
			tech=qn 
			outest=_parmnlp2_
			;
			
			parms 
				Param_3 = &Param_3_init,
				Param_4 = &Param_4_init;

			bounds 
				Param_3 > 0,
				Param_4 > 0;

			max logf;

			logf =	-log(Param_3) 
					- (Param_4 + 1) * 
					log1px( AMOUNT/(Param_3*Param_4));
		run;
		quit;

        data _parmnlp_;
			set _parmnlp_;
			where (index(_type_,"PARMS")>0);
		run;
		
        data _parmnlp2_;
			set _parmnlp2_;
			where (index(_type_,"PARMS")>0);
		run;

		data _parmnlp_;
			merge _parmnlp_ _parmnlp2_;
            Param_xLimit=&Param_xLimit;
            Param_pLimit=&Param_pLimit;
            Param_pnLimit=&Param_pnLimit;
		run;

		data _CovMat_;
			set _covmat_ _covmat2_;
		run;

	%LLLOGNGPD1
	
	data _logl_; 
		set _logl_;        
		if _N_=3 then do;
			nValue1 = &logl;
		end;
	run;

	%end;
	%else %do;
/*		%put ERROR: The fit of the body of the distribution failed.;*/
		%let status=1;
		%let reason=DISTRIBUTION BODY FIT FAILURE;
	%end;
%LOCAL_DONE:
%mend FitLOGNGPD_HILL;

/*****************************************************************************************
 *   Needed for: %orva_sfit_kri    ~line 304
 *
 *   What it does  :  Actually does the minimization using PROC NLP
 *
 *   No Parameters    
 *
 *****************************************************************************************/
%macro FitKriLOGNGPD_HILL;
    %let Param_xLimit=.;
        /* get the tail index  ------------------*/
        %orva_get_hill_estimate( data = _LogWorkdata_KRI_ , useKRI = YES);

	data _tmpworkdata_KRI_;
		set _LogWorkdata_KRI_ nobs=TotalCount end=TheEnd;
		where KRI_ADJ_AMOUNT <= &Param_xLimit;

		call symput("TotalCount",TotalCount);
	run;

	%let tmpdsid=%sysfunc(open(_tmpworkdata_KRI_,I));
	%let tmpnparmobs=%sysfunc(attrn(&tmpdsid,nlobs));
	%let tmprc=%sysfunc(close(&tmpdsid));

	%if &tmpnparmobs=0 %then %do;
  		%put ERROR: No observations below tail threshold.;
		%let status=1;
 		%let reason=No observations below tail threshold;
 		%goto LOCAL_DONE;
	%end;

	proc summary data=_tmpworkdata_KRI_ noprint;
		output
		out = _body_;
	run;

	data _null_;
		set _body_;
		call symput("BodyCount",_FREQ_);
	run;

   %let Param_pnLimit=%sysevalf(&BodyCount/&TotalCount);

	/* This piece of code increases Param_pnLimit if the probai=bility of observability is provided	*/
	/* New_pnLimit = Old_pnLimit + (1-Old_pnLimit)*p0 where p0 is the probability of being hidden   */
	%if %sysevalf(&THRESHOLDPERCENT >= 0, BOOLEAN) %then %do;
		%let Param_pnLimit = %sysevalf(&Param_pnLimit + (1-&Param_pnLimit)* &THRESHOLDPERCENT); 
	%end;

	ods listing close;
	ods output 
		IterStop           =  _logl_
		ConvergenceStatus  =  _cs_ 
		covmat             =  _covmat_
		;	

	proc nlp 
		data = _tmpworkdata_KRI_ 
		cov  = 4 
		pcov 
		maxit=10000 
		maxfu=10000 
		tech=qn 
		outest=_parmnlp_
		;
		
		parms 
			Param_1 = &Param_1_init,
			Param_2 = &Param_2_init;

		bounds 
			Param_2 > 0;

		max logf;

		tlcdf = 0;      /*threshold log cdf ------*/
		F0 = 0;
		pStar=probnorm( (log(&Param_xLimit) + &KRI_LOG_AVG - Param_1 - KRI_LOG) / Param_2);

		if &THRESHOLDPERCENT >= 0 then do;
			tlcdf =  &THRESHOLDPERCENT/
						(&Param_pnLimit  - &THRESHOLDPERCENT)*
						(
							logcdf("LOGNORMAL", THRESHOLD, param_1 + KRI_LOG, param_2)
							-log(pStar)
						);
		end; 
		else if THRESHOLD ne 0 then do;
			F0 = probnorm ((LogThreshold - Param_1 - KRI_LOG)/Param_2);
		end;

		logf = 	-log(pStar -  F0 )
				+ tlcdf
				-0.5*log(2)
				-lgamma(0.5)
				-LogAMOUNT
				-log(Param_2)
				-(LogAMOUNT - Param_1 - KRI_LOG )*(LogAMOUNT - Param_1 - KRI_LOG)/(2*Param_2*Param_2); 
	run;
	quit;
	ods listing;


	%if %sysfunc(exist(_cs_)) %then %do;
	data _null_;
		set _cs_;
		call symput("status",status);
	run;
	%end;
	%else %do;
		%let status=1;
		%let reason=Fit failed;
	%end;

	/* Let's check if the first part of the fit worked 	*/	
	%if &status=0  %then %do; 	
		/* Let's try to fit the tail				*/
		data _null_;
			set _parmnlp_;
			where index(_type_,"PARMS") > 0;
			/*  This is the LogNormal probability of being less than Param_xLimit */
			call symput("Param_pLimit",cdf("NORMAL",log(&Param_xLimit),Param_1,Param_2)); 
		run;
	
		data _tmpworkdata_;
			set _LogWorkdata_KRI_;
			where KRI_ADJ_AMOUNT > &Param_xLimit;
/*			KRI_ADJ_AMOUNT = (KRI_ADJ_AMOUNT- &Param_xLimit)*KRI_SCALE;*/
			KRI_ADJ_AMOUNT = (KRI_ADJ_AMOUNT- &Param_xLimit);
		run;

	

		%let tmpdsid=%sysfunc(open(_tmpworkdata_,I));
		%let tmpnparmobs=%sysfunc(attrn(&tmpdsid,nlobs));
		%let tmprc=%sysfunc(close(&tmpdsid));

		%if &tmpnparmobs=0 %then %do;
			%put ERROR: No observations above tail threshold.;
			%let status=1;
			%let reason=No observations above tail threshold;
			%goto LOCAL_DONE;
		%end;

		/*--INSERTED BY R.TRAVERSINI SAS ITALY 27APR2004-------------------*/
		/* For the tail fit the initial guess should be based 
		on the tail distribution moments and not on the full
		distribution moments                                            */


	proc summary data=_tmpworkdata_ noprint;
		var KRI_ADJ_AMOUNT;
		output 
			out  = _stat_(keep=sample_mean2 sample_variance2 sample_size2)
			n    = sample_size2
			mean = sample_mean2 
			var  = sample_variance2 
			; 
	run;

		proc transpose 
			data=_stat_ 
			out=_stat_;
		run;
		
		data _null_;
			set _stat_;
			call symput(_name_, col1);
		run;

     %if %sysevalf(&sample_size2 < 2, boolean) %then %do;
         %put ERROR: There are not enough observations above the tail breakpoint.;
         %let status=1;
         %let reason=NOT ENOUGH OBSERVATIONS IN TAIL;
         %goto LOCAL_DONE;
     %end;
    
        data _null_;
            alpha=(&sample_mean2*&sample_mean2 / &sample_variance2) + 2;
            theta=&sample_mean2*(alpha-1);
            call symput("Param_4_init", alpha);
            call symput("Param_3_init", theta/alpha);
        run;
        

    /*- END 27APR2004-------------------------*/


		  	data _NULL_;
				set _tmpworkdata_ ;
				retain logl 0;
				logl = logl + log(kri_SCALE);
				call symput("LOGL_KRI",logl);
			run;

	ods output 
		IterStop           =  _logl2_
		ConvergenceStatus  =  _cs_ 
		covmat             =  _covmat2_
		;	

		proc nlp 
			data=_tmpworkdata_ 
			cov=4 pcov 
			maxit=10000 
			maxfu=10000 
			tech=qn 
			outest=_parmnlp2_
			;

			parms 
				Param_3 = &Param_3_init,
				Param_4 = &Param_4_init;
				
			bounds 
				Param_3 > 0,
				Param_4 > 0;

			max logf;
				logf =	-log(Param_3) 
						- (Param_4 + 1) * 
						log1px( KRI_ADJ_AMOUNT/(Param_3*Param_4));
		run;
		quit;

		data _parmnlp_;
			set _parmnlp_;
			where (index(_type_,"PARMS")>0);
		run;

		data _parmnlp2_;
			set _parmnlp2_;
			where (index(_type_,"PARMS")>0);
		run;

		data _parmnlp_;
			merge _parmnlp_ _parmnlp2_;
			Param_xLimit = &Param_xLimit;
			Param_pLimit = &Param_pLimit;
			Param_pnLimit = &Param_pnLimit;
		run;

		data _CovMat_;
			set _covmat_ _covmat2_;
		run;

		%LLKRILOGNGPD1;
		
		data _logl_;
			set _logl2_;

			if _N_ = 3 then do;
/*				nValue1 = &logl + &LOGL_KRI;*/
				nValue1 = &logl;
			end;
		run;
	%end;
	%else %do;
		%let status=1;
		%let reason=DISTRIBUTION BODY FIT FAILURE;
	%end;
%LOCAL_DONE:

%mend FitKriLOGNGPD_HILL;

/*****************************************************************************************
 *   Needed for: %orva_sfitall    ~line 320
 *
 *   What it does  :  Modify loglikelihood for cases when we had to take the log of the 
 *					  observations first. Here, it does nothing.
 *
 *   NO Parameters    
 *
 *****************************************************************************************/
%macro MakeLLGoodLOGNGPD_HILL;
    %let sumlog=0;
%mend MakeLLGoodLOGNGPD_HILL;

%macro MakeLLGoodKRILOGNGPD_HILL;
     %let sumlog=0;
%mend MakeLLGoodKRILOGNGPD_HILL;

/*****************************************************************************************
 *   Needed for: %orva_sfitall    ~line 379
 *
 *   What it does  :  Checks whether some moments are missing.
 *					  Here does nothing because all moments exist for Exponential distribution
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro NoMissingMOMLOGNGPD_HILL(parmdata=);
data &parmdata;
		set &parmdata;

		if mean=. then do;
			status=1;
			reason="Bad Parameter Combination: Mean does not exist.";
		end;
		else if std=. then do;
			status=2;
			reason="Bad Parameter Combination: Variance does not exist.";
		end;

        call symput("status",status);
	run;
%mend NoMissingMOMLOGNGPD_HILL;

/*- actually, the moments of logarithm of losses -*/
%macro MomentsLOGNGPD_HILL;
       %MomentsLOGNGPD1;
%mend MomentsLOGNGPD_HILL;

%macro RealMomentsLOGNGPD_HILL;
    %MomentsLOGNGPD1;
%mend RealMomentsLOGNGPD_HILL;

/*****************************************************************************************
 *   Needed for: %orva_sfitall_mom.sas      ~line 78
 *
 *   What it does  :  Does method of moments estimation and assigns the corresponding 
 *                    values to the param_1, param_2, param_4, and param_3 macros
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro MomFitLOGNGPD_HILL;
	/*- expect  the mean, variance, and kurtosis of logarithm of losses -*/    
	data _null_;
        Param_2=sqrt(&variance);
        call symput("Param_1",&mean);
	    call symput("Param_2",Param_2);
	run;
%mend MomFitLOGNGPD_HILL;

/*****************************************************************************************
 *   Needed for: %orvr_sfitstats.sas      ~line 78
 *
 *   What it does  :  Displays the estimated parameters
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro ParamsLOGNGPD_HILL;
       %ParamsLOGNGPD1;
%mend ParamsLOGNGPD_HILL;

/*****************************************************************************************
 *   Needed for: %orvr_sfitstats.sas      ~line 78
 *
 *   What it does  :  Displays the estimated parameters
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro CovmatLOGNGPD_HILL;
      %CovmatLOGNGPD1;
%mend CovmatLOGNGPD_HILL;

/*****************************************************************************************
 *   What it does  :  Calculates the loglikelihood contribution for each observation
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro LLLOGNGPD_HILL(var);
      %LLLOGNGPD1(&var);
%mend LLLOGNGPD_HILL;


/*****************************************************************************************
 *   Needed to be called before anything else is available
 *****************************************************************************************/
%macro ORVD_LOGNGPD_HILL;
	/* include ORVD_LOGNGPD since we will use some of its macros */
    %ORVD_LOGNGPD1;
    %let EXTRA_PARAMS= ;
    %let NEEDED_INPUT= ;
    %let SCALE_TRANSFORM= LOG;
    %let DIST_NAME= Log Normal-GPD Hill;
    %let PARAM_COUNT= 4;
	
	data LOGNGPD_HILL_STATUS;
		status          = 0;
	run;
	
%mend ORVD_LOGNGPD_HILL;
