/*-------------------------------------------------------------------
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 * NAME:    ORVR_Empirical                                             
 * PURPOSE: Produces the OpRiskVaR Empirical Statistics report.       
 * SUPPORT: stchri
 *
 * PARAMETERS:
 *    fileref - fileref for HTML output.
 *    model_uid - specified what project we're dealing with.
 *    options - specifies which parts of the report to produce, 
 *              separated by spaces:
 *       		SUMMARY_I -
 *       		DETAILS_I - 
 *       		SUMMARY_C -
 *       		DETAILS_C - 
 *       		SUMMARY_P -
 *       		DETAILS_P - 
 *              SCENARIOS
 *
 *    outputType - file output type (html, rtf, etc.)
 *                 defaults to HTML.
 *    style - ODS style definition,
 *            defaults to DEFAULT.
 *
 * NOTES: Assumes caller has set the following librefs:
 *     varprj, varuser, varcomp, varorg
 *-------------------------------------------------------------------*/
 %macro ORVR_Empirical( fileref     =, 
                        model_uid   =,
                        options     =, 
                        outputType  =HTML,
                        style       =DEFAULT);

%orvr_report_setup(fileref=&fileref,
                   model_uid=&model_uid,
                   outputType=&outputType,
                   style=&style);
%let parts=%upcase(&options);
title2 &titleSpec2 "OpRiskVaR Empiricial Statistics Report";
title3 &titleSpec3 "(Model=&MODEL_NAME Model Author=&FULL_NAME)";

/*- get list of Scenarios in this project -*/
%let SCENARIO_UIDS=;
%let SCENARIO_COUNT=0;
data _NULL_; set varcomp.PROJECT_SCENARIO_REFS end=last;
      where MODEL_UID=&model_uid;
      length scenarios $ 1000;
      retain scenarios;
	  retain scenario_count;
	  if scenario_count eq . then scenario_count = 0;
      scenario_count = scenario_count+1;
      scenarios = trim(left(scenarios)) || " "|| trim(left(SCENARIO_UID));
      if last then do;
         call symput( "SCENARIO_UIDS", trim(scenarios));
         call symput( "SCENARIO_COUNT", trim(scenario_count));
      end;
run;


/*- print the Model Table itself -*/
proc report data=varprj.MODEL nowd; 
   column MODEL_NAME DATE_CREATED DATE_LAST_MODIFIED;
   define MODEL_NAME / 'Name';
   define DATE_CREATED / 'Created (GMT)' format=&DATETIMEFMT;
   define DATE_LAST_MODIFIED / 'Last Modified (GMT)' format=&DATETIMEFMT;
run;


%if %index(&parts,SUMMARY_I)^=0 %then %do;
  title4 &titleSpec4 "Summary Report: Internal Data";
  footnote1 &footnoteSpec1;
  %mom(sdata=sevmomts_i,fdata=freqmomts_i,where=(ccvuid_1 eq 0),summary=true);
%end;
%if %index(&parts,SUMMARY_C)^=0 %then %do;
  title4 &titleSpec4 "Summary Report: Consortium Data";
  footnote1 &footnoteSpec1;
  %mom(sdata=sevmomts_c,fdata=freqmomts_c,where=(ccvuid_1 eq 0),summary=true);
%end;
%if %index(&parts,SUMMARY_P)^=0 %then %do;
  title4 &titleSpec4 "Summary Report: Public Data";
  footnote1 &footnoteSpec1;
  %mom(sdata=sevmomts_p,fdata=freqmomts_p,where=(ccvuid_1 eq 0),summary=true);
%end;

%if %index(&parts,DETAILS_I)^=0 %then %do;
  title4 &titleSpec4  "Details Report: Internal Data";
  footnote1 &footnoteSpec1;
  %mom(sdata=sevmomts_i,fdata=freqmomts_i);
%end;
%if %index(&parts,DETAILS_C)^=0 %then %do;
  title4 &titleSpec4 "Details Report: Consortium Data";
  footnote1 &footnoteSpec1;
  %mom(sdata=sevmomts_c,fdata=freqmomts_c);
%end;
%if %index(&parts,DETAILS_P)^=0 %then %do;
  title4 &titleSpec4 "Details Report: Public Data";
  footnote1 &footnoteSpec1;
  %mom(sdata=sevmomts_p,fdata=freqmomts_p);
%end;
%if %index(&parts,SCENARIO)^=0 %then %do;
  %if &SCENARIO_COUNT = 0 %then %do;
     %emptyrpt(No Scenarios assigned to project);
  %end;
  %let nScenario = 1;
  %let scenarioUid = %scan(&SCENARIO_UIDS, &nScenario, ' ');
  %do %while(&scenarioUid ne ); 
     proc sql noprint; 
        select SCENARIO_NAME into :SCENARIO_NAME
        from varorg.SCENARIO  
        where SCENARIO_UID=&scenarioUid and VERSION_UID=&version_uid;
	 quit;
     title4 &titleSpec4 "Details Report: Scenario &SCENARIO_NAME";
     footnote1 &footnoteSpec1;
     %mom(sdata=sevmomts_scenario&scenarioUid,fdata=freqmomts_scenario&scenarioUid,scenario=true);	          
	 %let nScenario=%eval(&nScenario+1);
	 %let scenarioUid=%scan(&SCENARIO_UIDS, &nScenario, ' ');
  %end;

%end;


/* close OUTPUT file */
%if &fileref ^= "" %then %do;
  ods &outputType close;
  ods listing;
%end;

%mend ORVR_Empirical;


%macro mom(sdata=,fdata=,where=,summary=false,scenario=false);
    /*- merge frequency and severity statistics together by CCUID -*/
	%let snobs=0;
	%let dsid=%sysfunc(open(varprj.&sdata,I));
	%if &dsid ^= 0 %then %do;
	    %let snobs=%sysfunc(attrn(&dsid,nlobs));
	    %let rc=%sysfunc(close(&dsid));
	%end;
	%let fnobs=0;
	%let dsid=%sysfunc(open(varprj.&fdata,I));
	%if &dsid ^= 0 %then %do;
	    %let fnobs=%sysfunc(attrn(&dsid,nlobs));
	    %let rc=%sysfunc(close(&dsid));
	%end;
    %if &snobs ^= 0 and &fnobs ^= 0 %then %do;
      proc sort data=varprj.&sdata; by CCUID; run;
      proc sort data=varprj.&fdata; by CCUID; run;
      data _workf_; set varprj.&fdata; 
         FMEAN = MEAN; 
         drop MEAN %if &scenario=true %then %do; VARIANCE %end;;
      run;
      data _work0_;
	      merge varprj.&sdata _workf_; 
	      by ccuid;
      run;
    %end;
    %else %if &snobs ^= 0 %then %do;
       proc sort data=varprj.&sdata; by CCUID; run;
       data _work0_; set varprj.&sdata; run;
    %end;
    %else %if &fnobs ^= 0 %then %do;
       proc sort data=varprj.&fdata; by CCUID; run;
       data _work0_; set varprj.&fdata; 
         FMEAN = MEAN; 
         drop MEAN;
       run;
    %end;
    %else %do;
      %emptyrpt(No data found);
      %goto DONE;
    %end;
    
    /*- merge statistics with CC dataset -*/
	proc sql;
	  create table _work1_ as
	  select * from 
	       _work0_ a, 
	       varprj.cc c
	  where a.CCUID = c.CCUID;
	quit;

    /*- get names for CCVUIDs -*/
	proc sql;
	  create table _moments as
	  select * from 
	       _work1_ a, 
	       varprj.ccmapval_0 ccv0,
	       varprj.ccmapval_1 ccv1
	  where a.CCVUID_0 = ccv0.CCVUID_0 and
	        a.CCVUID_1 = ccv1.CCVUID_1;
	quit;
	
    
    /*- fix up cc strings -*/
 	%if &summary=false %then %do;
      proc sort data=_moments; by CCUID; run;
      data _moments; set _moments;
        retain foo;
        length foo $ 128;
        if trim(left(foo)) ne "" and trim(left(foo)) eq trim(left(CCVVALUE_0)) then do;
           foo = CCVVALUE_0;
           CCVVALUE_0="";
        end;
        else do; 
           foo=CCVVALUE_0; 
        end;
      run;
    %end;
    /*- run proc report -*/
    proc report data=_moments nowd SPLIT="*"  
			   headline headskip CENTER contents="Empirical Statistics"; 
	    %if &summary=false %then %do;
			column  ('Classifications' CCVVALUE_0 CCVVALUE_1 ) 
                    %if &scenario=false %then %do;
			           COUNT MINLOSS MAXLOSS
					%end;
			        ('Frequency' FMEAN VARIANCE)
			        ('Severity' MEAN STD KURTOSIS)
			        ;		
			define CCVVALUE_0 / left "&MAPNAME0";
			define CCVVALUE_1 / display left "&MAPNAME1";
		%end;
		%else %do;
			column CCVVALUE_0 			      
                    %if &scenario=false %then %do;
			           COUNT MINLOSS MAXLOSS
					%end;
			       ('Frequency' FMEAN VARIANCE)
			       ('Severity' MEAN STD KURTOSIS);
			define CCVVALUE_0 / display left "&MAPNAME0";
		%end;
		define FMEAN / display format=&deffmt RIGHT "Mean";
		define VARIANCE / display format=&deffmt RIGHT "Variance";
		define MEAN / display format=&deffmt RIGHT "Mean";
		define STD / display format=&deffmt RIGHT "Std";
		define KURTOSIS / display format=&deffmt RIGHT "Kurt";
		%if &scenario=false %then %do;
		  define COUNT / display format=COMMA10. RIGHT "Count";
		  define MINLOSS / display format=&curfmt RIGHT "MinLoss";
		  define MAXLOSS / display format=&curfmt RIGHT "MaxLoss";
		%end;
		%if &where ^= "" %then %do;
		  where &where;
		%end;
	   
	run;  
%DONE:	
	proc datasets library=work memtype=data nolist nodetails;
      delete _moments _workf_ _work0_ _work1_;
    run;
    quit;
   
%mend mom;

%macro emptyrpt(phrase);
	data fake;												
	   length fakevar $10; label fakevar = '00'X;
	   fakevar= ' ';
	run;
	proc report data=fake nowd SPLIT="." CENTER;	
		column fakevar;
		define fakevar / display "&phrase";
	run;
%mend emptyrpt;

