/*-------------------------------------------------------------------
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 * NAME    : orva_var_calc.sas  
 * PURPOSE : Produces report for VaR Criteria, VaR Final Matrix and VaR Results     
 * SUPPORT : stchri 
 *
 * Input:
 *    fileref - fileref for HTML output, ignored if blank
 *    model_uid - UID for model.
 *    options - specifies which parts of the report to produce, 
 *              separated by spaces:
 *       VINS - Final Insurance Values. Both Primary and Secondary.
 *       VCRIT - VaR Calculation Criteria (options)
 *       VMAT - Final Matrix for VaR
 *       VFEFFECTS - Frequency Effects 
 *       VRES_WITHOUT  - VaR Result Matrix without insurance
 *       VRES_INSURANCE  - VaR Result Matrix with insurance
 *       VRESBASEL_WITHOUT  - Basel VaR Result Matrix without insurance
 *       VRESBASEL_INSURANCE - Basel VaR Result Matrix with insurance
 *    outputtype - html, pdf, etc.
 *    currency - USD or EUR
 *    style - template style
 *
 * Required tables:
 *   varuser.VAR_USER
 *   varorg.INSURANCE
 *   varprj.MODEL
 *   varprj.VAR_CRITERIA
 *   varprj.VAR_MATRIX
 *   varprj.VAR_RESULT_MATRIX
 *   varprj.ccmapval_0
 *   varprj.ccmapval_1
 *   varprj.cc
 *   varprj.aggstat_naked
 *   varprj.aggstat_insured
 *   varprj.varstat_naked
 *   varprj.varstat_insured
 *   varprj.aggstatbasel_naked
 *   varprj.aggstatbasel_insured
 *
 * Notes: Assumes the caller has set the macro varaible _VARLIB to point
 *        to the client install area's admin folder.  This is where
 *        various sas graphics jars are installed and must be set as the
 *        codebase for graphic reports.
 *-------------------------------------------------------------------*/ 
%macro ORVR_var_calc(fileref      =,
                     model_uid    =,
                     options      =,
                     outputType   =HTML,
                     style        =DEFAULT);
                     
	
    %orvr_report_setup(fileref=&fileref,
                       model_uid=&model_uid,
                       outputType=&outputType,
                       style=&style,
                       graphics=true);
	

	/******************************************/
	/********** formats needed below **********/
	/******************************************/
	proc format;
	  value BooleanYN   1='Yes'
	                    0='No';
	run;
	
	proc format;
	  value VarMixingSelection   1='Internal Only'
	                             2='Relative Relations'
	                             3='Consortium Only'
	                             5='Simulation Mixing'
	                             ;
    run;
    	
	proc format;
	  value ConfLevel  low-high=[5.1] '%';
	run;

	proc format;
       value $CorrType 'U'='User Defined'
                       'T'='Internal'
                       'I'='Shared Internal'
                       'C'='Shared Consortium'
                       'O'='Identity'
                       ;
    run;

	proc format;
       value $DataType 'I'='Internal'
                       'M'='Mixed'
                       'S'='Scenario'
                       'P'='Public'
                       'C'='Consortium'
                       ;
    run;

/********************************************************/
/******************** Criteria Data *********************/
/********************************************************/

    data Crit;
      set varprj.VAR_CRITERIA(where=(MODEL_UID=&model_uid));
      call symput("CONF1",CONFIDENCE_LEVEL_1);
      call symput("CONF2",CONFIDENCE_LEVEL_2);
      call symput("CONF3",CONFIDENCE_LEVEL_3);
      
 	  call symput("SIMULATION_MIXING",MIXING_SELECTION);
 	  call symput("USE_RATIO", USE_RATIO);
	run;

 
/****************************************************/
/* VaR Insurance    *********************************/
/****************************************************/

%if %index(&options, VINS)^=0 %then %do;

    /********** Primary Insurance **********/
    footnote1; %cleartitles; 
	title2 &titleSpec2 "VaR Final Matrix Primary Insurance Report";
	title3 &titleSpec3 "This report contains the primary insurance probabilities for VaR calculations";
	title4 &titleSpec4 "(Model=&MODEL_NAME Model Author=&FULL_NAME)";
	
	
	%combine_data_with_ccv_names(dataIn=varprj.insure_1, dataOut=_work0);
	
	*** Get Insurance Policy names;
    proc sql noprint;
      create table _work as
        select * 
        from _work0 as I left join varorg.INSURANCE as N          
        on  I.INSURANCE_1 eq N.INSURANCE_UID ; 
    quit;
    *** dump the Nones!;
    data _work; set _work;
      where INSURANCE_1 ne 0;
    run;
    proc sort data=_work; by CCVVALUE_0 CCVVALUE_1; run;
    	
    *** run the report;
    proc template;
	  define table orvar_insurance;
         header h1;
	     
	     column (CCVVALUE_0) CCVVALUE_1 
		         INSURANCE_NAME PIns1 PInsCurrent1;
                
         define h1;
             text 'Classifications'; 
             start=CCVVALUE_0  space=1 expand='-' end=CCVVALUE_1;
         end;                   

         define CCVVALUE_0;
             header="&MAPNAME0" blank_dups=on id;
         end; 
	     define CCVVALUE_1;
             header="&MAPNAME1";
         end; 
		 
         define INSURANCE_NAME;
             header="Insurance Policy";
         end; 
         define PIns1;
             header="Default";
			 print=false;
         end;  
         define PInsCurrent1;
             header="Probability of Insurance";
			 cellstyle PIns1 ^= PInsCurrent1 as data
                {foreground=red font_style=italic font_weight=bold},
                1 as data;
         end;
		 define footer table_footer;
             text "RED indicates values modified by user";
             style=header{foreground=red font_style=italic font_weight=bold};
             ;
	     end;
      end;  
    run;

	data _NULL_;set _work;
       file print ods=(template='orvar_insurance');
       put _ods_;
	run;  
   
    /********** Secondary Insurance **********/
    footnote1; %cleartitles; 
	title2 &titleSpec2 "VaR Final Matrix Secondary Insurance Report";
	title3 &titleSpec3 "This report contains the secondary insurance probabilities for VaR calculations";
	title4 &titleSpec4 "(Model=&MODEL_NAME Model Author=&FULL_NAME)";
	
	%combine_data_with_ccv_names(dataIn=varprj.insure_2, dataOut=_work0);
	
	*** Get Insurance Policy Names;
    proc sql noprint;
      create table _work as
        select I.*, N.INSURANCE_NAME 
        from _work0 as I left join varorg.INSURANCE as N          
        on  I.INSURANCE_1 eq N.INSURANCE_UID; 
    quit;
    data _work0; set _work (rename=(INSURANCE_NAME= INSURANCE_NAME_1)); run;
    proc sql noprint;
      create table _work as
        select I.*, N.INSURANCE_NAME 
        from _work0 as I left join varorg.INSURANCE as N          
        on  I.INSURANCE_2 eq N.INSURANCE_UID; 
    quit;
	*** Dump the Nones;
    data _work; set _work;
      where INSURANCE_1 ne 0 and INSURANCE_2 ne 0;
    run;
    proc sort data=_work; by CCVVALUE_0 CCVVALUE_1; run;	  
	
	*** Run Report;
    proc template;
	  define table orvar_insurance_2;
         header h1;
         header h2;
	     
	     column  (CCVVALUE_0) CCVVALUE_1
		         INSURANCE_NAME_1 INSURANCE_NAME
		         PIns2 PInsCurrent2;
                
         define h1;
             text 'Classifications'; 
             start=CCVVALUE_0  space=1 expand='-' end=CCVVALUE_1;
         end;                           
         define CCVVALUE_0;
             header="&MAPNAME0" blank_dups=on id;
         end; 
	     define CCVVALUE_1;
             header="&MAPNAME1";
         end; 
         
         define h2;
             text 'Insurance Policies'; 
             start=INSURANCE_NAME_1  space=1 expand='-' end=INSURANCE_NAME;
         end;     
         define INSURANCE_NAME_1;
             header="Primary";
         end; 
         define INSURANCE_NAME;
             header="Secondary";
         end; 
         
         define PIns2;
             header="Default";
			 print=false;
         end;           
         define PInsCurrent2;
             header="Probability of Insurance";
			 cellstyle PIns2 ^= PInsCurrent2 as data
                {foreground=red font_style=italic font_weight=bold},
                1 as data;
         end;
         
		 define footer table_footer;
             text "RED indicates values modified by user";
             style=header{foreground=red font_style=italic font_weight=bold};
             ;
	     end;
      end;  
    run;
	data _NULL_; set _work;
       file print ods=(template='orvar_insurance_2');
       put _ods_;
	run;  
    
%end;
/****************************************************/
/* END VaR Insurance Matrix *************************/
/****************************************************/




/****************************************************/
/* VaR FINAL Matrix *********************************/
/****************************************************/
%if %index(&options, VMAT)^=0 %then %do;

    /*** Internal Only ***/                       
    %if &SIMULATION_MIXING eq 1 %then %do;
       data _final_sparm;  set varprj.selsparmi; run;
       data _final_fparm;  set varprj.selfparmi; run;
       %let MIXED = false;
	   %let SIMMIX = false;
    %end;
    /*** Relative Relations ***/                       
    %else %if &SIMULATION_MIXING eq 2 %then %do;
       data _final_sparm;  set varprj.selsparmi varprj.mixsparm; run;
       data _final_fparm;  set varprj.selfparmi varprj.mixfparm; run;
       %let MIXED = true;
	   %let SIMMIX = false;
    %end;
    /*** Consortium Only ***/                       
    %else %if &SIMULATION_MIXING = 3 %then %do;
       data _final_sparm;  set varprj.selsparmc; run;
       data _final_fparm;  set varprj.selfparmc; run;
       %let MIXED = false;
	   %let SIMMIX = false;
    %end;
    /*** Simulation Mixing ***/                       
    %else %if &SIMULATION_MIXING eq 5 %then %do;
       data _final_sparm;  set varprj.simpriority_sev; run;
       data _final_fparm;  set varprj.simpriority_freq; run;
	   %let SIMMIX = true;
    %end;


    /************************* NOT SIMMIX ******************************/
    %if &SIMMIX ne true %then %do;
	    *** Frequency ***;
		%combine_data_with_ccv_names(dataIn=_final_fparm, dataOut=_final);
		
	    footnote1; %cleartitles; 
	    title2 &titleSpec2 "VaR Final Matrix Frequency Distribution Report";
		title3 &titleSpec3 "This report contains the frequency data used for VaR calculations";
		title4 &titleSpec4 "(Model=&MODEL_NAME Model Author=&FULL_NAME)";		
		title5 &titleSpec5 'Executive Summary Graph';
		%var_calc_graph_report(_final,(ccvuid_0 ne 0) and (ccvuid_1 ne 0),CCVVALUE_0,&MAPNAME0,MEAN,mean,HBAR3D);
		%if &empty=0 %then %do; 					
			%emptyrpt(No data to generate graph.);
		%end;
	
	    footnote1 &footnoteSpec1;
		title5 &titleSpec5 "Frequency Distribution Detailed Report";	      
		proc template;
		  define table orvar_final_freq;	     
		     column (CCVVALUE_0) CCVVALUE_1 DISTRIBUTION MEAN VARIANCE;                
	         header h1;
	         define h1;
	             text "Fitted";
	             start=MEAN  space=1 expand='-' end=VARIANCE;
	         end;                         
	         define CCVVALUE_0;
	             header="&MAPNAME0" blank_dups=on id;
	         end; 
		     define CCVVALUE_1;
	             header="&MAPNAME1";
	         end;         
	         define DISTRIBUTION;
	             header="Distribution";
	         end; 
	      end;  
	    run;
	
		data _NULL_; set _final;	   
	       file print ods=(template='orvar_final_freq');
	       put _ods_;
		run;
	
	    *** Severity ***; 
		%combine_data_with_ccv_names(dataIn=_final_sparm, dataOut=_final);
	    footnote1; %cleartitles;
		title2 &titleSpec2 "VaR Final Matrix Severity Distribution Report";
		title3 &titleSpec3 "This report contains the severity data used for VaR calculations";
		title4 &titleSpec4 "(Model=&MODEL_NAME Model Author=&FULL_NAME)";
		
		title5 &titleSpec5 'Executive Summary Graph';
		%var_calc_graph_report(_final,(ccvuid_0 ne 0) and (ccvuid_1 ne 0),CCVVALUE_0,&MAPNAME0,MEAN,mean,HBAR3D);
		%if &empty=0 %then %do; 
		   %emptyrpt(No data to generate graph.);
		%end;
		
		footnote1 &footnoteSpec1;
		title5 &titleSpec5 "Severity Distribution Detailed Report";
		proc template;
			 define table orvar_final_sev;
		         header h1;	     
			     column (CCVVALUE_0) CCVVALUE_1 DISTRIBUTION MEAN STD KURTOSIS ;               
		         define h1;
		             text "Fitted";
		             start=MEAN  space=1 expand='-' end=KURTOSIS;
		         end;          
		         define CCVVALUE_0;
		             header="&MAPNAME0" blank_dups=on id;
		         end; 
			     define CCVVALUE_1;
		             header="&MAPNAME1";
		         end; 		 
		         define DISTRIBUTION;
		             header="Distribution";
		         end;        
		         define MEAN;
		             header="Mean" format=&deffmt;
		         end;		 
		         define STD;
		             header="Standard Deviation" format=&deffmt;
		         end;          
		         define KURTOSIS;
		             header="Kurtosis" format=&deffmt;
		         end;
		     end;  
	    run;
	
		data _NULL_;set _final;
	       file print ods=(template='orvar_final_sev');
	       put _ods_;
		run;
    %end; /*** SIMMIX ne true ***/
    
    /************************* SIMMIX ******************************/
    %else %do;
	    *** Frequency ***;
		%combine_data_with_ccv_names(dataIn=_final_fparm, dataOut=_final);
		
	    footnote1; %cleartitles; 
	    title2 &titleSpec2 "VaR Final Matrix - Frequency Simulaton Mixing Priority Report";
		title3 &titleSpec3 "This report contains the frequency simulation mixing choices used for VaR calculations";
		title4 &titleSpec4 "(Model=&MODEL_NAME Model Author=&FULL_NAME)";
	
	    footnote1 &footnoteSpec1;
		proc template;
		  define table orvar_freq_priority;	     
		         header h1;	     
			     column (CCVVALUE_0) CCVVALUE_1 SOURCE_NAME;                       
		         define CCVVALUE_0;
		             header="&MAPNAME0" blank_dups=on id;
		         end; 
			     define CCVVALUE_1;
		             header="&MAPNAME1";
		         end; 		 
		         define SOURCE_NAME;
		             header="Data Source" ;
		         end;
	      end;  
	    run;
	
		data _NULL_; set _final;	   
	       file print ods=(template='orvar_freq_priority');
	       put _ods_;
		run;
	
	    *** Severity ***; 
		%combine_data_with_ccv_names(dataIn=_final_sparm, dataOut=_final);
	    footnote1; %cleartitles;
		title2 &titleSpec2 "VaR Final Matrix - Severity Simulaton Mixing Priority Report";
		title3 &titleSpec3 "This report contains the severity simulation mixing choices used for VaR calculations";
		title4 &titleSpec4 "(Model=&MODEL_NAME Model Author=&FULL_NAME)";
		
		footnote1 &footnoteSpec1;
		proc template;
			 define table orvar_sev_priority;
		         header h1;	     
			     column (CCVVALUE_0) CCVVALUE_1 SOURCE_NAME LOWER_BOUND UPPER_BOUND ;               
		         define h1;
		             text "Bounds";
		             start=LOWER_BOUND  space=1 expand='-' end=UPPER_BOUND;
		         end;          
		         define CCVVALUE_0;
		             header="&MAPNAME0" blank_dups=on id;
		         end; 
			     define CCVVALUE_1;
		             header="&MAPNAME1";
		         end; 		 
		         define SOURCE_NAME;
		             header="Data Source" ;
		         end;        
		         define LOWER_BOUND;
		             header="Lower" format=ConfLevel.;
		         end;		 
		         define UPPER_BOUND;
		             header="Upper" format=ConfLevel.;
		         end;		         
		     end;  
	    run;
	
		data _NULL_;set _final;
	       file print ods=(template='orvar_sev_priority');
	       put _ods_;
		run;
    
    %end;
    
%end;
/****************************************************/
/* END VaR Final Matrix *****************************/
/****************************************************/


/****************************************************/
/* VaR Calculation Criteria *************************/
/****************************************************/

%if %index(&options, VCRIT)^=0 %then %do;
   footnote1; %cleartitles;	
   title2 &titleSpec2 "VaR Criteria Report" ;
   title3 &titleSpec3 "This report contains the options used for Simulation";
   title4 &titleSpec4 "(Model=&MODEL_NAME Model Author=&FULL_NAME)";
   footnote1 &footnoteSpec1 ;

   proc report data=crit style(column)=[font_size=2] nowd;
	 column 
	     ( 'Confidence Levels'  CONFIDENCE_LEVEL_1 CONFIDENCE_LEVEL_2 CONFIDENCE_LEVEL_3)
	      SIM_INTERVAL ITERATIONS SEED MAX_NUM_HOSTS      
	      %if &USE_RATIO eq 1 %then %do; 
	         MAX_SCALE_FACTOR 
	      %end;
	      %else %do; 
	         DEFAULT_MAXIMUM_LOSS 
	      %end; 
	      MIXING_SELECTION CORRELATION_TYPE 
	      ;
	      
         define CONFIDENCE_LEVEL_1 / "1" format=ConfLevel.;
	     define CONFIDENCE_LEVEL_2 / "2" format=ConfLevel.;
         define CONFIDENCE_LEVEL_3 / "3" format=ConfLevel.;
         define ITERATIONS / "Iterations";
	     define SEED / "Random Seed";
	     define MIXING_SELECTION / "Mixing Selection" format =VarMixingSelection.;
	     define CORRELATION_TYPE / "Freq Correlation" format =$CorrType.;
	     define SIM_INTERVAL / "Interval";
	     define MAX_NUM_HOSTS / "Parallel Hosts";
         %if &USE_RATIO eq 1 %then %do;
	        define MAX_SCALE_FACTOR / "Worst Case Loss Ratio";
	     %end;
	     %else %do; 
	        define DEFAULT_MAXIMUM_LOSS / "Default Maximum Loss"
	     %end;
    run;
    
   %if &SIMULATION_MIXING eq 5 %then %do;
      /* get macro var for simmiz ranges */
      data _NULL_;
        set varprj.SIMMIX_RANGES(where=(MODEL_UID=&model_uid));
        call symput("NUM_RANGES", NUM_RANGES);
      run;
      data _NULL_;
        set varprj.SIMMIX_RANGES(where=(MODEL_UID=&model_uid));
        %do i=1 %to &NUM_RANGES;
           call symput("LOWER_&i", LOWER_&i);
           call symput("UPPER_&i", UPPER_&i);
        %end;
      run;
      /* add better range names to simmix_priority dataset */
      data _priority0; set varprj.SIMMIX_PRIORITY(where=(MODEL_UID=&model_uid));
         length RANGE_NAME $100;
         %do i = 1 %to &NUM_RANGES;
            if RANGE_NUM = &i then do;
               RANGE_NAME = "&&LOWER_&i. - &&UPPER_&i.%";
            end;
         %end;
      run;
      /* add Scenario names */
      proc sql noprint;
      create table _priority as
        select p.*, n.SCENARIO_NAME 
        from _priority0 as p left join varorg.SCENARIO as n          
        on p.SCENARIO_UID eq n.SCENARIO_UID; 
      quit;      
      /* combine with ccv names */
      %combine_data_with_ccv_names(dataIn=_priority, dataOut=_priority0, defaults=true);
      proc sort data=_priority0; by RANGE_NUM PRIORITY; run;
      
      /*** severity simmix priorities ***/
      footnote1; %cleartitles;	
      title2 &titleSpec2 "Simulation Mixing Priorities (Severity)" ;
      title3 &titleSpec3 "Severity Simulation Mixing Priorities used for VaR Calculations";
      title4 &titleSpec4 "(Model=&MODEL_NAME Model Author=&FULL_NAME)";
      footnote1 &footnoteSpec1 ;
      proc report data= _priority0 style(column)=[font_size=2] nowd;
        where RANGE_NUM ne 0;
        column ('Classifications' CCVVALUE_0 CCVVALUE_1)
               RANGE_NAME PRIORITY DATA_TYPE SCENARIO_NAME;
           define CCVVALUE_0 / "&MAPNAME0";
           define CCVVALUE_1 / "&MAPNAME1";
           define RANGE_NAME / "Range";
           define PRIORITY / "Priority";
           define DATA_TYPE / "Data Source" format= $DataType.;
           define SCENARIO_NAME / "(Name)";
      run;

      /*** frequency simmix priorities ***/
      footnote1; %cleartitles;	
      title2 &titleSpec2 "Simulation Mixing Priorities (Frequency)" ;
      title3 &titleSpec3 "Frequency Simulation Mixing Priorities used for VaR Calculations";
      title4 &titleSpec4 "(Model=&MODEL_NAME Model Author=&FULL_NAME)";
      footnote1 &footnoteSpec1 ;
      proc report data= _priority0 style(column)=[font_size=2] nowd;
        where RANGE_NUM eq 0;
        column ('Classifications' CCVVALUE_0 CCVVALUE_1)
               PRIORITY DATA_TYPE SCENARIO_NAME;
           define CCVVALUE_0 / "&MAPNAME0";
           define CCVVALUE_1 / "&MAPNAME1";
           define PRIORITY / "Priority";
           define DATA_TYPE / "Data Source" format= $DataType.;
           define SCENARIO_NAME / "(Name)";
      run;
   %end;
%end;

/****************************************************/
/* VaR Frequency Effects ****************************/
/****************************************************/

%if %index(&options, VFEFFECTS)^=0 %then %do;
    footnote1; %cleartitles;	
    title2 &titleSpec2 "Simulation Frequency Effects Report" ;
    title3 &titleSpec3 "This report contains the frequency effects used in Simulation";
    title4 &titleSpec4 "(Model=&MODEL_NAME Model Author=&FULL_NAME)";
    footnote1 &footnoteSpec1;

    /*** get appropriate CCVs ***/
	%combine_data_with_ccv_names(dataIn=VARPRJ.feffects, dataOut=_feffects);
	proc sort data=_feffects; by CCUID; run;

  	/*** create Freq Effects Template ***/
    proc template;
	  define table var_feffects;
         header ccHeader;
	     
	     column (CCVVALUE_0) CCVVALUE_1 
	        DISTRIBUTION FPARAM_1 FPARAM_2;
                
         define ccHeader;
             text 'Classifications'; 
             start=CCVVALUE_0  space=1 expand='-' end=CCVVALUE_1;
         end;                   
         define CCVVALUE_0;
             header="&MAPNAME0" blank_dups=on id;
         end; 
	     define CCVVALUE_1;
             header="&MAPNAME1";
         end; 
		 
		 define DISTRIBUTION; header="Frequency Distribution"; end; 			     
		 define FPARAM_1; header="Parameter 1"; end; 			     
		 define FPARAM_2; header="Parameter 2"; end; 			     
      end;  
	run;
	
	/*** print report ***/
    data _NULL_; set _feffects;	   
	   file print ods=(template='var_feffects');
	   put _ods_;
	run;

	/*** delete temporary data sets ***/
	proc datasets library=work memtype=data nolist nodetails;
		delete _feffects;
	quit;
	
%end;


/****************************************************/
/* VaR Result Matrices ******************************/
/****************************************************/
%if %index(&options, VRES_INSURANCE) or 
    %index(&options, VRESBASEL_INSURANCE) or
    %index(&options, VRESBASEL_WITHOUT) or 
    %index(&options, VRES_WITHOUT) %then %do;

 	/*** create VaR Report Template ***/
    proc template;
	  define table var_results;
         header ccHeader %do i=1 %to 3; confHeader&i. %end;;
	     
	     column (CCVVALUE_0) CCVVALUE_1 
	        LCL1 P_1 UCL1 LCL2 P_2 UCL2 LCL3 P_3 UCL3 MEAN;
                
         define ccHeader;
             text 'Classifications'; 
             start=CCVVALUE_0  space=1 expand='-' end=CCVVALUE_1;
         end;                   
         define CCVVALUE_0;
             header="&MAPNAME0" blank_dups=on id;
         end; 
	     define CCVVALUE_1;
             header="&MAPNAME1";
         end; 
		 		
		 %do i=1 %to 3;
             define confHeader&i.;
	             text "&&CONF&i..th Percentile"; 
	             start=LCL&i  space=1 expand='-' end=UCL&i;
	         end;  	
         	 define LCL&i;  header= "Lower CL"       format= &curfmt;     end; 
		     define P_&i;   header= "Quantile VaR"   format= &curfmt;     end; 
			 define UCL&i;  header= "Upper CL"       format= &curfmt;     end; 
		 %end;
		 
		 define MEAN; header="Mean"  format= &curfmt;  end; 			     
      end;  
	run;
    
	%if %index(&options, VRES_INSURANCE) %then %do;
	    %let VARDATA= varprj.varstat_insured;
	    %let TITLETEXT1= "VaR Results With Insurance";
	    %let TITLETEXT3= "This report contains VaR results with insurance.";
	    %let AGGDATA= varprj.aggstat_insured;
	    %let TITLETEXT4= "SAS method used for aggregate cells.";
	    %var_result_report(&TITLETEXT1, &TITLETEXT3, &TITLETEXT4, var_results, &VARDATA, &AGGDATA);
	%end;
	
	%if %index(&options, VRESBASEL_INSURANCE) %then %do;
	    %let VARDATA= varprj.varstat_insured;
	    %let TITLETEXT1= "VaR Results With Insurance";
	    %let TITLETEXT3= "This report contains VaR results with insurance.";
	    %let AGGDATA= varprj.aggstatbasel_insured;
	    %let TITLETEXT4= "Standard method used for aggregate cells.";
	    %var_result_report(&TITLETEXT1, &TITLETEXT3, &TITLETEXT4, var_results, &VARDATA, &AGGDATA);
	%end;
	
	
	%if %index(&options, VRESBASEL_WITHOUT) %then %do;
	    %let VARDATA= varprj.varstat_naked;
	    %let TITLETEXT1= "VaR Results Without Insurance";
	    %let TITLETEXT3= "This report contains VaR results without insurance.";
	    %let AGGDATA= varprj.aggstatbasel_naked;
	    %let TITLETEXT4= "Standard method used for aggregate cells.";
	    %var_result_report(&TITLETEXT1, &TITLETEXT3, &TITLETEXT4, var_results, &VARDATA, &AGGDATA);
	%end; 
	
	%if %index(&options, VRES_WITHOUT) %then %do;
	    %let VARDATA= varprj.varstat_naked;
	    %let TITLETEXT1= "VaR Results Without Insurance";
	    %let TITLETEXT3= "This report contains VaR results without insurance.";
	    %let AGGDATA= varprj.aggstat_naked;
	    %let TITLETEXT4= "SAS method used for aggregate cells.";
	    %var_result_report(&TITLETEXT1, &TITLETEXT3, &TITLETEXT4, var_results, &VARDATA, &AGGDATA);
	%end;        
			
%end;



/****************************************************/
/* END of VaR Result Matrices ***********************/
/****************************************************/


/*- delete unwanted data sets -*/
proc datasets library=work memtype=data nolist nodetails;
  delete ResultInsurance0 ResultInsurance1 ResultInsurance 
       Result0 Result1 Result _work_ _fake_ var_user
       Crit Final;
quit;

  ods _all_ close;
  ods listing;


%mend ORVR_var_calc;





/***********************************************************************************/
/******************************** Report Macro #1 **********************************/
/***********************************************************************************/

%macro var_result_report(TITLETEXT1, TITLETEXT3, TITLETEXT4, TEMPLATE, VARDATA, AGGDATA);
    footnote1; %cleartitles;
  	title2 &titleSpec2 &TITLETEXT1;
	title3 &titleSpec3 &TITLETEXT3;
	title4 &titleSpec4 &TITLETEXT4;
	title5 &titleSpec5 "(Model=&MODEL_NAME Model Author=&FULL_NAME)";
    footnote1 &footnoteSpec1;

    %let nobs=0;
    %let dsid=%sysfunc(open(&VARDATA,I));
    %if &dsid ^= 0 %then %do;
        %let nobs=%sysfunc(attrn(&dsid,nlobs));
        %let rc=%sysfunc(close(&dsid));
    %end;
    %if &nobs eq 0 %then %do;
       %emptyrpt(No VaR results in project.);
    %end;
    %else %do;
        /*** combine cells with aggregates ***/
	    data ResultInsurance0;
	       set &VARDATA (where=(STATUS=0)) &AGGDATA ;
	    run;
	    /*** get Cross-classification names ***/
	    %combine_data_with_ccv_names(dataIn=ResultInsurance0, dataOut=ResultInsurance);
	    proc sort data=ResultInsurance; by CCUID; run;
	    
	    /*** graphical summary ***/
		%var_calc_graph_report(ResultInsurance,(ccvuid_0 ne 0) and (ccvuid_1 ne 0),CCVVALUE_0,&MAPNAME0,MEAN,mean,HBAR3d);
		%if &empty=0 %then %do; 
		   %emptyrpt(No data to generate graph. Check results in aggregate totals);
		%end;

	    /*** detailed report ***/
		data _NULL_; set ResultInsurance;	   
	       file print ods=(template='var_results');
	       put _ods_;
		run;
    %end;
%mend var_result_report;


/***********************************************************************************/
/******************************** Graph Macro #1 ***********************************/
/***********************************************************************************/

%macro var_calc_graph_report(indata,where,ccvar,ccvarlabel,sumvar,sumvarlabel,charttype);
    /* Check to see if dataset is empty */
	%global empty;
	%let empty=0;								
	data _NULL_;
	  set &indata (where=(&where));
	  call symput ("empty", 1);
	run;

	goptions device=java hsize=4in vsize=4in;
	legend1 mode=share cshadow=gray;
	axis2 label=("&sumvarlabel");
	axis1 label=("&ccvarlabel");

	proc GCHART data= &indata;
                label &ccvar="&ccvarlabel";
		&charttype	&ccvar /
		SUMVAR=&sumvar
		%if %substr(&charttype,2,3) = BAR %then %do;
			Descending
			PATTERNID=MIDPOINT
			shape=cylinder
			CREF=BLACK
			AUTOREF
			raxis=axis2
		%end;
		%if %substr(&charttype,1,3) = PIE %then %do;
			value=arrow
			slice=arrow
		%end;
		legend=legend1;
	    where &where;
	run; quit;
%mend ;

/***********************************************************************************/
/******************************** Misc Macros #1 ***********************************/
/***********************************************************************************/


%macro fakerpt;						/* Designed to force ODS output */
	proc report data=_fake_ nowd LS=160 PS=50 SPLIT="*" CENTER; 
		column fakevar			;
		define fakevar / noprint ;
	run;
%mend;

%macro emptyrpt (phrase);			/* Tell user about missing data for report */
	proc report data=_fake_ nowd LS=160 PS=50 SPLIT="." CENTER;	
		column fakevar			;
		define fakevar / display width=50 spacing=100 "&phrase" ;
	run;quit;
%mend;



*** get Names for Cross Classification variables;
%macro combine_data_with_ccv_names(dataIn=, dataOut=, defaults=false);
	proc sql noprint;
	  create table &dataOut as
	      select distinct * 
	      from &dataIn i,
	          varprj.cc cc,
	          varprj.ccmapval_0 ccv0,
	          varprj.ccmapval_1 ccv1
	      where i.CCUID = cc.CCUID and
	            cc.CCVUID_0 = ccv0.CCVUID_0 and 
	            cc.CCVUID_1 = ccv1.CCVUID_1 ; 
	quit;
	%if "&defaults" eq "true" %then %do;
	  data _defaults; set &dataIn; where CCUID eq 0; 
	    CCVVALUE_0='default';
	    CCVVALUE_1='default';
	  run;
	  data &dataOut; set _defaults &dataOut; run;
	%end;
	proc sort data= &dataOut; by CCUID; run;
%mend combine_data_with_ccv_names;

